/* ----------------------------------------------------------------------------
 *         ATMEL Microcontroller Software Support
 * ----------------------------------------------------------------------------
 * Copyright (c) 2012, Atmel Corporation

 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "common.h"
#include "hardware.h"
#include "pmc.h"
#include "dbgu.h"
#include "debug.h"
#include "ddramc.h"
#include "spi.h"
#include "gpio.h"
#include "slowclk.h"
#include "timer.h"
#include "watchdog.h"
#include "string.h"
#include "onewire_info.h"

#include "arch/at91_pmc.h"
#include "arch/at91_rstc.h"
#include "arch/sama5_smc.h"
#include "arch/at91_pio.h"
#include "arch/at91_ddrsdrc.h"
#include "sama5d3xek.h"

#ifdef CONFIG_USER_HW_INIT
extern void hw_init_hook(void);
#endif

static void at91_dbgu_hw_init(void)
{
	/* Configure DBGU pin */
	const struct pio_desc dbgu_pins[] = {
		{"RXD", AT91C_PIN_PB(30), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"TXD", AT91C_PIN_PB(31), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{(char *)0, 0, 0, PIO_DEFAULT, PIO_PERIPH_A},
	};

	/*  Configure the dbgu pins */
	pio_configure(dbgu_pins);
	writel((1 << AT91C_ID_PIOB), (PMC_PCER + AT91C_BASE_PMC));

	/* Enable clock */
	writel(1 << AT91C_ID_DBGU, (PMC_PCER + AT91C_BASE_PMC));
}

static void initialize_dbgu(void)
{
	at91_dbgu_hw_init();
	dbgu_init(BAUDRATE(MASTER_CLOCK, 115200));
}



#ifdef CONFIG_DDR2
static void ddramc_reg_config(struct ddramc_register *ddramc_config)
{
	ddramc_config->mdr = (AT91C_DDRC2_DBW_32_BITS
				| AT91C_DDRC2_MD_DDR2_SDRAM);

	ddramc_config->cr = (AT91C_DDRC2_NC_DDR10_SDR9
				| AT91C_DDRC2_NR_13 //| AT91C_DDRC2_NR_14
				| AT91C_DDRC2_CAS_3 //3
				| AT91C_DDRC2_DLL_RESET_DISABLED /* DLL not reset */
				| AT91C_DDRC2_DIS_DLL_DISABLED   /* DLL not disabled */
				//| AT91C_DDRC2_DIC_DS
				| AT91C_DDRC2_ENRDM_ENABLE       /* Phase error correction is enabled */
				| AT91C_DDRC2_NB_BANKS_8
				| AT91C_DDRC2_NDQS_DISABLED      /* NDQS disabled (check on schematics) */
				//| AT91C_DDRC2_DECOD_INTERLEAVED  /* Interleaved decoding */
				| AT91C_DDRC2_UNAL_SUPPORTED);   /* Unaligned access is supported */

#if defined(CONFIG_BUS_SPEED_133MHZ)
	/*
	 * The DDR2-SDRAM device requires a refresh every 15.625 us or 7.81 us.
	 * With a 133 MHz frequency, the refresh timer count register must to be
	 * set with (15.625 x 133 MHz) ~ 2084 i.e. 0x824
	 * or (7.81 x 133 MHz) ~ 1040 i.e. 0x410.
	 */
	ddramc_config->rtr = 0x411;     /* Refresh timer: 7.8125us */

	/* One clock cycle @ 133 MHz = 7.5 ns */
	ddramc_config->t0pr = (AT91C_DDRC2_TRAS_6       /* 6 * 7.5 = 45 ns */
			| AT91C_DDRC2_TRCD_2            /* 2 * 7.5 = 15ns ,22.5 ns */
			| AT91C_DDRC2_TWR_2             /* 2 * 7.5 = 15   ns */
			| AT91C_DDRC2_TRC_8             /* 8 * 7.5 = 60ns, 75   ns */
			| AT91C_DDRC2_TRP_2             /* 2 * 7.5 = 15   ns */
//			| AT91C_DDRC2_TRRD_2            /* 2 * 7.5 = 15   ns */
			| AT91C_DDRC2_TRRD_1            /* 2 * 7.5 = 15   ns */
			| AT91C_DDRC2_TWTR_2            /* 2 clock cycles min */
			| AT91C_DDRC2_TMRD_2);          /* 2 clock cycles */
#if 0
	ddramc_config->t1pr = (AT91C_DDRC2_TXP_2        /*  2 clock cycles */
			| AT91C_DDRC2_TXSRD_208         /* 200 clock cycles */
			//| AT91C_DDRC2_TXSNR_28          /* 195 + 10 = 205ns ==> 28 * 7.5 = 210 ns : 2Gb*/
			| AT91C_DDRC2_TXSNR_16          /* 127.5 + 10 = 137.5ns ==> 19 * 7.5 = 142.5 ns : 1Gb*/
			//| AT91C_DDRC2_TRFC_26);         /* 26 * 7.5 = 195 ns : 2Gb*/
			| AT91C_DDRC2_TRFC_14);         /* 17 * 7.5 = 127.5 ns : 1Gb*/

	ddramc_config->t2pr = (AT91C_DDRC2_TFAW_10       /* 7 * 7.5 = 52.5 ns */
			| AT91C_DDRC2_TRTP_2            /* 2 clock cycles min */
			| AT91C_DDRC2_TRPA_2            /* 2 * 7.5 = 15 ns */
			| AT91C_DDRC2_TXARDS_7          /* 7 clock cycles */
			| AT91C_DDRC2_TXARD_7);         /* MR12 = 1 : slow exit power down */
#else
	ddramc_config->t1pr = (AT91C_DDRC2_TXP_2        /*  2 clock cycles */
			| AT91C_DDRC2_TXSRD_200         /* 200 clock cycles */
			//| AT91C_DDRC2_TXSNR_28          /* 195 + 10 = 205ns ==> 28 * 7.5 = 210 ns : 2Gb*/
			| AT91C_DDRC2_TXSNR_19          /* 127.5 + 10 = 137.5ns ==> 19 * 7.5 = 142.5 ns : 1Gb*/
			//| AT91C_DDRC2_TRFC_26);         /* 26 * 7.5 = 195 ns : 2Gb*/
			| AT91C_DDRC2_TRFC_17);         /* 17 * 7.5 = 127.5 ns : 1Gb*/

	ddramc_config->t2pr = (AT91C_DDRC2_TFAW_7       /* 7 * 7.5 = 52.5 ns */
			| AT91C_DDRC2_TRTP_2            /* 2 clock cycles min */
			| AT91C_DDRC2_TRPA_2            /* 2 * 7.5 = 15 ns */
			| AT91C_DDRC2_TXARDS_7          /* 7 clock cycles */
			| AT91C_DDRC2_TXARD_8);         /* MR12 = 1 : slow exit power down */
#endif

#elif defined(CONFIG_BUS_SPEED_166MHZ)
	/*
	 * The DDR2-SDRAM device requires a refresh of all rows every 64ms.
	 * ((64ms) / 8192) * 166MHz = 1296 i.e. 0x510
	 */
	ddramc_config->rtr = 0x500;

	/* One clock cycle @ 166 MHz = 6.0 ns */
	ddramc_config->t0pr = (AT91C_DDRC2_TRAS_8	/* 8 * 6 = 48 ns */
			| AT91C_DDRC2_TRCD_3		/* 3 * 6 = 18 ns */
			| AT91C_DDRC2_TWR_3		/* 3 * 6 = 18 ns */
			| AT91C_DDRC2_TRC_10		/* 10 * 6 = 60 ns */
			| AT91C_DDRC2_TRP_3		/* 3 * 6 = 18 ns */
			| AT91C_DDRC2_TRRD_2		/* 2 * 6 = 12 ns */
			| AT91C_DDRC2_TWTR_2		/* 2 clock cycles*/
			| AT91C_DDRC2_TMRD_2);		/* 2 clock cycles at least */

	ddramc_config->t1pr = (AT91C_DDRC2_TXP_3	/* 3 * 6 = 18ns, 2 clock cycles a least */
			| AT91C_DDRC2_TXSRD_202		/* 202 clock cycles: Exit self refresh delay to Read command */
			| AT91C_DDRC2_TXSNR_35		/* 35 * 6 = 210 ns*/
			| AT91C_DDRC2_TRFC_31);		/* 31 * 6 = 186 ns */

	ddramc_config->t2pr = (AT91C_DDRC2_TFAW_8	/* 45 ns for 16bit * 8 bank */
			| AT91C_DDRC2_TRTP_2		/* 2 * 6 = 15ns clock cycles min */
			| AT91C_DDRC2_TRPA_3		/* 15 ns */
			| AT91C_DDRC2_TXARDS_10		/* 7 ~ 10 clock cycles */
			| AT91C_DDRC2_TXARD_3);		/* 2 ~ 3 clock cycles */

#else
#error "No bus clock provided!"
#endif
}
static unsigned char ExtRAM_TestOk(void)
{
    unsigned int i;
    unsigned int *ptr = (unsigned int *) AT91C_BASE_DDRCS;

    for (i = 0; i < 10 * 1024; ++i) {

        if (i & 1) {
            ptr[i] = 0x55AA55AA | (1 << i);
        }
        else {
            ptr[i] = 0xAA55AA55 | (1 << i);
        }
    }

    for (i = 0; i < 10 * 1024; ++i) {
        if (i & 1) {
            if (ptr[i] != (0x55AA55AA | (1 << i))) {
                return 0;
            }
        }
        else {
            if (ptr[i] != (0xAA55AA55 | (1 << i))) {
                return 0;
            }
        }
    }

    return 1;
}

static void ddramc_init(void)
{
	struct ddramc_register ddramc_reg;
	unsigned int reg;


    /* DDR reset */
    #define   MPDDRC_LPR_CLK_FR_ENABLED (0x1u << 2)
    #define   MPDDRC_LPR_LPCB_DEEP_PWD (0x3u << 0)
    //MPDDRC->MPDDRC_LPR 
    reg = MPDDRC_LPR_LPCB_DEEP_PWD |MPDDRC_LPR_CLK_FR_ENABLED;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_LPR));

    /* Disable DDR clock. */
    //PMC->PMC_PCDR1 |= (1 << (ID_MPDDRC-32));
    //PMC->PMC_SCDR  |= PMC_SCER_DDRCK;

	writel(1 << (AT91C_ID_MPDDRC - 32),  (PMC_PCDR1 + AT91C_BASE_PMC));
	writel(AT91C_PMC_DDR, (PMC_SCDR + AT91C_BASE_PMC));


	ddramc_reg_config(&ddramc_reg);

#if 0
	/* enable ddr2 clock */
	writel(1 << (AT91C_ID_MPDDRC - 32),  (PMC_PCER1 + AT91C_BASE_PMC));
	writel(AT91C_PMC_DDR, (PMC_SCER + AT91C_BASE_PMC));


	writel(0, (AT91C_BASE_MPDDRC + HDDRSDRC2_LPR));
    *(unsigned int *)0xFFFFEA24 |= (1 << 5);  // DDRSDRC High Speed Register (MPDDRC_HS)  : hidden option -> calibration during autorefresh
    *(unsigned int *)0xF0038004 |= (0x3 << 16);   // SFR_DDRCFG  DDR Configuration  Force DDR_DQ and DDR_DQS input buffer always on

	/* Init the special register for sama5d3x */
	/* MPDDRC DLL Slave Offset Register: DDR2 configuration */
	reg = AT91C_MPDDRC_S0OFF_1
		| AT91C_MPDDRC_S2OFF_1
		| AT91C_MPDDRC_S3OFF_1;
	writel(reg, (AT91C_BASE_MPDDRC + MPDDRC_DLL_SOR));

	/* MPDDRC DLL Master Offset Register */
	/* write master + clk90 offset */
	reg = AT91C_MPDDRC_MOFF_7
		| AT91C_MPDDRC_CLK90OFF_31
		| AT91C_MPDDRC_SELOFF_ENABLED | AT91C_MPDDRC_KEY;
	writel(reg, (AT91C_BASE_MPDDRC + MPDDRC_DLL_MOR));

	/* MPDDRC I/O Calibration Register */
	/* DDR2 RZQ = 50 Ohm */
	/* TZQIO = 4 */

//#define AT91C_MPDDRC_TZQIO_3   (3<<8)
//#define AT91C_MPDDRC_RDIV_DDR2_RZQ_48   (3<<0)
//	reg = AT91C_MPDDRC_RDIV_DDR2_RZQ_48
//		| AT91C_MPDDRC_TZQIO_3;
	reg = AT91C_MPDDRC_RDIV_DDR2_RZQ_50
		| AT91C_MPDDRC_TZQIO_4;

	writel(reg, (AT91C_BASE_MPDDRC + MPDDRC_IO_CALIBR));

   *(unsigned int *)0xF0038004 = (0x3 << 16);   // SFR_DDRCFG  DDR Configuration  Force DDR_DQ and DDR_DQS input buffer always on
#else
	/* enable ddr2 clock */
	writel(1 << (AT91C_ID_MPDDRC - 32),  (PMC_PCER1 + AT91C_BASE_PMC));
	writel(AT91C_PMC_DDR, (PMC_SCER + AT91C_BASE_PMC));

	/* Init the special register for sama5d3x */
	/* MPDDRC DLL Slave Offset Register: DDR2 configuration */
	reg = AT91C_MPDDRC_S0OFF_1
		| AT91C_MPDDRC_S2OFF_1
		| AT91C_MPDDRC_S3OFF_1;
	writel(reg, (AT91C_BASE_MPDDRC + MPDDRC_DLL_SOR));

	/* MPDDRC DLL Master Offset Register */
	/* write master + clk90 offset */
	reg = AT91C_MPDDRC_MOFF_7
		| AT91C_MPDDRC_CLK90OFF_31
		| AT91C_MPDDRC_SELOFF_ENABLED | AT91C_MPDDRC_KEY;
	writel(reg, (AT91C_BASE_MPDDRC + MPDDRC_DLL_MOR));

	/* MPDDRC I/O Calibration Register */
	/* DDR2 RZQ = 50 Ohm */
	/* TZQIO = 4 */
	reg = AT91C_MPDDRC_RDIV_DDR2_RZQ_50
		| AT91C_MPDDRC_TZQIO_4;

	writel(reg, (AT91C_BASE_MPDDRC + MPDDRC_IO_CALIBR));

#endif


//	/* DDRAM2 Controller initialize */
//	ddram_initialize(AT91C_BASE_MPDDRC, AT91C_BASE_DDRCS, &ddramc_reg);


#define AT91C_DDRC2_OCD_COUNT(value) (value <<12)
#define AT91C_DDRC2_RTR_COUNT(value) (value << 0)
#define DDR2_BA0(r) (1 << (25 + r))
#define DDR2_BA1(r) (1 << (26 + r))

#define DEV_64M16    0
#define DEV_128M16   1

    unsigned char device = DEV_64M16;
    //unsigned char device = DEV_128M16;

    volatile unsigned char *pDdr = (volatile unsigned char *) AT91C_BASE_DDRCS;
    volatile unsigned int i;
    volatile unsigned int cr = 0;
    
/* Step 1: Program the memory device type */
    /* DBW = 0 (32 bits bus wide); Memory Device = 6 = DDR2-SDRAM = 0x00000006*/
    //MPDDRC->MPDDRC_MD = MPDDRC_MD_MD_DDR2_SDRAM;
    reg = ddramc_reg.mdr;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MDR));

/* Step 2: Program the features of DDR2-SDRAM device into the Timing Register.*/
    reg = ddramc_reg.cr;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));

	reg = ddramc_reg.t0pr;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_T0PR));

	reg = ddramc_reg.t1pr;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_T1PR));

	reg = ddramc_reg.t2pr;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_T2PR));

//#if 1
//    /* DDRSDRC Low-power Register */
//    for (i = 0; i < 13300; i++) {
//        asm("nop");
//    } 
//    //MPDDRC->MPDDRC_LPR 
//    reg = AT91C_DDRC2_LPCB_DISABLED /*| AT91C_DDRC2_CLK_FR_DISABLED*/ | AT91C_DDRC2_TIMEOUT_0 | AT91C_DDRC2_ADPE_FAST;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_LPR));
//
///* Step 3: An NOP command is issued to the DDR2-SDRAM. Program the NOP command into
//    the Mode Register, the application must set MODE to 1 in the Mode Register. */
//    //MPDDRC->MPDDRC_MR 
//    reg = AT91C_DDRC2_MODE_NOP_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    /* Perform a write access to any DDR2-SDRAM address to acknowledge this command */
//    *pDdr = 0;  /* Now clocks which drive DDR2-SDRAM device are enabled.*/
//   
//    /* A minimum pause of 200 s is provided to precede any signal toggle. (6 core cycles per iteration, core is at 396MHz: min 13200 loops) */
//    for (i = 0; i < 13300; i++) {
//        asm("nop");
//    } 
// 
///* Step 4:  An NOP command is issued to the DDR2-SDRAM */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_NOP_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    /* Perform a write access to any DDR2-SDRAM address to acknowledge this command.*/
//    *pDdr = 0; /* Now CKE is driven high.*/
//    /* wait 400 ns min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
// 
///* Step 5: An all banks precharge command is issued to the DDR2-SDRAM. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_PRCGALL_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    /* Perform a write access to any DDR2-SDRAM address to acknowledge this command.*/
//    *pDdr = 0;
//    /* wait 400 ns min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//
///* Step 6: An Extended Mode Register set (EMRS2) cycle is  issued to chose between commercialor high  temperature operations. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_EXT_LMR_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *((unsigned char *)(pDdr + DDR2_BA1(device))) = 0; /* The write address must be chosen so that BA[1] is set to 1 and BA[0] is set to 0. */
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//
///* Step 7: An Extended Mode Register set (EMRS3) cycle is issued to set all registers to 0. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_EXT_LMR_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *((unsigned char *)(pDdr + DDR2_BA1(device) + DDR2_BA0(device))) = 0;  /* The write address must be chosen so that BA[1] is set to 1 and BA[0] is set to 1.*/
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//
// /* Step 8:  An Extended Mode Register set (EMRS1) cycle is issued to enable DLL. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_EXT_LMR_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *((unsigned char *)(pDdr + DDR2_BA0(device))) = 0;  /* The write address must be chosen so that BA[1] is set to 0 and BA[0] is set to 1. */
//    /* An additional 200 cycles of clock are required for locking DLL */
//    for (i = 0; i < 10000; i++) {
//        asm("nop");
//    }
//
///* Step 9:  Program DLL field into the Configuration Register.*/
//    //cr = MPDDRC->MPDDRC_CR;
//    cr = readl((AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
//    //MPDDRC->MPDDRC_CR
//    reg = cr | AT91C_DDRC2_DLL_RESET_ENABLED;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
//
///* Step 10: A Mode Register set (MRS) cycle is issued to reset DLL. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_LMR_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *(pDdr) = 0;  /* The write address must be chosen so that BA[1:0] bits are set to 0. */
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
// 
///* Step 11: An all banks precharge command is issued to the DDR2-SDRAM. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_PRCGALL_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *(pDdr) = 0;  /* Perform a write access to any DDR2-SDRAM address to acknowledge this command */
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//
///* Step 12: Two auto-refresh (CBR) cycles are provided. Program the auto refresh command (CBR) into the Mode Register. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_RFSH_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *(pDdr) = 0;  /* Perform a write access to any DDR2-SDRAM address to acknowledge this command */
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//    /* Configure 2nd CBR. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_RFSH_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *(pDdr) = 0;  /* Perform a write access to any DDR2-SDRAM address to acknowledge this command */
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//
///* Step 13: Program DLL field into the Configuration Register to low(Disable DLL reset). */
//    //cr = MPDDRC->MPDDRC_CR;
//    cr = readl((AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
//    //MPDDRC->MPDDRC_CR
//    reg = cr & (~AT91C_DDRC2_DLL_RESET_ENABLED);
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
//
///* Step 14: A Mode Register set (MRS) cycle is issued to program the parameters of the DDR2-SDRAM devices. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_LMR_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *(pDdr) = 0;  /* The write address must be chosen so that BA[1:0] are set to 0. */
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//
///* Step 15: Program OCD field into the Configuration Register to high (OCD calibration default). */
//    //cr = MPDDRC->MPDDRC_CR;
//    cr = readl((AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
//    //MPDDRC->MPDDRC_CR
//    cr = cr | AT91C_DDRC2_OCD_COUNT(0x07);
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
//
//
///* Step 16: An Extended Mode Register set (EMRS1) cycle is issued to OCD default value. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_EXT_LMR_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *((unsigned char *)(pDdr + DDR2_BA0(device))) = 0;  /* The write address must be chosen so that BA[1] is set to 0 and BA[0] is set to 1.*/
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//
///* Step 17: Program OCD field into the Configuration Register to low (OCD calibration mode exit). */
//    // cr = MPDDRC->MPDDRC_CR;
////    cr = readl(AT91C_BASE_MPDDRC + HDDRSDRC2_CR);
////    // MPDDRC->MPDDRC_CR 
////    reg = cr & (~ AT91C_DDRC2_OCD_COUNT(0x07));
////    writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
//
///* Step 18: An Extended Mode Register set (EMRS1) cycle is issued to enable OCD exit.*/
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_EXT_LMR_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *((unsigned char *)(pDdr + DDR2_BA0(device))) = 0;  /* The write address must be chosen so that BA[1] is set to 0 and BA[0] is set to 1.*/
//    /* wait 2 cycles min */
//    for (i = 0; i < 100; i++) {
//        asm("nop");
//    }
//
///* Step 19,20: A mode Normal command is provided. Program the Normal mode into Mode Register. */
//    //MPDDRC->MPDDRC_MR
//    reg = AT91C_DDRC2_MODE_NORMAL_CMD;
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
//    *(pDdr) = 0;
//
//    //*(pDdr) = 0;
//
///* Step 21: Write the refresh rate into the count field in the Refresh Timer register. The DDR2-SDRAM device requires a refresh every 15.625 s or 7.81 s. 
//   With a 100MHz frequency, the refresh timer count register must to be set with (15.625 /100 MHz) = 1562 i.e. 0x061A or (7.81 /100MHz) = 781 i.e. 0x030d. */
//    /* For MT47H64M16HR, The refresh period is 64ms (commercial), This equates to an average
//       refresh rate of 7.8125s (commercial), To ensure all rows of all banks are properly 
//       refreshed, 8192 REFRESH commands must be issued every 64ms (commercial) */
//    /* ((64 x 10(^-3))/8192) x133 x (10^6) */
//    //MPDDRC->MPDDRC_RTR
//    reg = AT91C_DDRC2_RTR_COUNT(300); /* Set Refresh timer 7.8125 us*/
//	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_RTR));
//    /* OK now we are ready to work on the DDRSDR */
//    /* wait for end of calibration */
//    for (i = 0; i < 500; i++) {
//        asm("    nop");
//    }
////#else
//#else
    /* DDRSDRC Low-power Register */
	udelay(200);

    //MPDDRC->MPDDRC_LPR 
    reg = AT91C_DDRC2_LPCB_DISABLED /*| AT91C_DDRC2_CLK_FR_DISABLED*/ | AT91C_DDRC2_TIMEOUT_0 | AT91C_DDRC2_ADPE_FAST;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_LPR));

/* Step 3: An NOP command is issued to the DDR2-SDRAM. Program the NOP command into
    the Mode Register, the application must set MODE to 1 in the Mode Register. */
    //MPDDRC->MPDDRC_MR 
    reg = AT91C_DDRC2_MODE_NOP_CMD;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    /* Perform a write access to any DDR2-SDRAM address to acknowledge this command */
    *pDdr = 0;  /* Now clocks which drive DDR2-SDRAM device are enabled.*/
   
    /* A minimum pause of 200 s is provided to precede any signal toggle. (6 core cycles per iteration, core is at 396MHz: min 13200 loops) */
	udelay(200);
 
/* Step 4:  An NOP command is issued to the DDR2-SDRAM */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_NOP_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    /* Perform a write access to any DDR2-SDRAM address to acknowledge this command.*/
    *pDdr = 0; /* Now CKE is driven high.*/
    /* wait 400 ns min */
	udelay(1);
 
/* Step 5: An all banks precharge command is issued to the DDR2-SDRAM. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_PRCGALL_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    /* Perform a write access to any DDR2-SDRAM address to acknowledge this command.*/
    *pDdr = 0;
    /* wait 400 ns min */
	udelay(1);

/* Step 6: An Extended Mode Register set (EMRS2) cycle is  issued to chose between commercialor high  temperature operations. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_EXT_LMR_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *((unsigned char *)(pDdr + DDR2_BA1(device))) = 0; /* The write address must be chosen so that BA[1] is set to 1 and BA[0] is set to 0. */
    /* wait 2 cycles min */
	udelay(1);

/* Step 7: An Extended Mode Register set (EMRS3) cycle is issued to set all registers to 0. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_EXT_LMR_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *((unsigned char *)(pDdr + DDR2_BA1(device) + DDR2_BA0(device))) = 0;  /* The write address must be chosen so that BA[1] is set to 1 and BA[0] is set to 1.*/
    /* wait 2 cycles min */
	udelay(1);

 /* Step 8:  An Extended Mode Register set (EMRS1) cycle is issued to enable DLL. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_EXT_LMR_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *((unsigned char *)(pDdr + DDR2_BA0(device))) = 0;  /* The write address must be chosen so that BA[1] is set to 0 and BA[0] is set to 1. */
    /* An additional 200 cycles of clock are required for locking DLL */
	udelay(1);

/* Step 9:  Program DLL field into the Configuration Register.*/
    //cr = MPDDRC->MPDDRC_CR;
    cr = readl((AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
    //MPDDRC->MPDDRC_CR
    reg = cr | AT91C_DDRC2_DLL_RESET_ENABLED;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));

/* Step 10: A Mode Register set (MRS) cycle is issued to reset DLL. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_LMR_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *(pDdr) = 0;  /* The write address must be chosen so that BA[1:0] bits are set to 0. */
    /* wait 2 cycles min */
	udelay(1);
 
/* Step 11: An all banks precharge command is issued to the DDR2-SDRAM. */
    //MPDDRC->MPDDRC_MR
    reg = AT91C_DDRC2_MODE_PRCGALL_CMD;
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *(pDdr) = 0;  /* Perform a write access to any DDR2-SDRAM address to acknowledge this command */
    /* wait 2 cycles min */
	udelay(1);

/* Step 12: Two auto-refresh (CBR) cycles are provided. Program the auto refresh command (CBR) into the Mode Register. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_RFSH_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *(pDdr) = 0;  /* Perform a write access to any DDR2-SDRAM address to acknowledge this command */
    /* wait 2 cycles min */
	udelay(1);

    /* Configure 2nd CBR. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_RFSH_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *(pDdr) = 0;  /* Perform a write access to any DDR2-SDRAM address to acknowledge this command */
    /* wait 2 cycles min */
	udelay(1);

/* Step 13: Program DLL field into the Configuration Register to low(Disable DLL reset). */
    //cr = MPDDRC->MPDDRC_CR;
    cr = readl((AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
    //MPDDRC->MPDDRC_CR
    reg = cr & (~AT91C_DDRC2_DLL_RESET_ENABLED);
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));

/* Step 14: A Mode Register set (MRS) cycle is issued to program the parameters of the DDR2-SDRAM devices. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_LMR_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *(pDdr) = 0;  /* The write address must be chosen so that BA[1:0] are set to 0. */
    /* wait 2 cycles min */
	udelay(1);

/* Step 15: Program OCD field into the Configuration Register to high (OCD calibration default). */
    //cr = MPDDRC->MPDDRC_CR;
    cr = readl((AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
    //MPDDRC->MPDDRC_CR
    cr = cr | AT91C_DDRC2_OCD_COUNT(0x07);
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
	udelay(1);

/* Step 16: An Extended Mode Register set (EMRS1) cycle is issued to OCD default value. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_EXT_LMR_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *((unsigned char *)(pDdr + DDR2_BA0(device))) = 0;  /* The write address must be chosen so that BA[1] is set to 0 and BA[0] is set to 1.*/
    /* wait 2 cycles min */
	udelay(1);

/* Step 17: Program OCD field into the Configuration Register to low (OCD calibration mode exit). */
    // cr = MPDDRC->MPDDRC_CR;
    cr = readl(AT91C_BASE_MPDDRC + HDDRSDRC2_CR);
    // MPDDRC->MPDDRC_CR 
    reg = cr & (~ AT91C_DDRC2_OCD_COUNT(0x07));
    writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_CR));
	/* wait 2 cycles min (of tCK) = 15 ns min */
	udelay(1);

/* Step 18: An Extended Mode Register set (EMRS1) cycle is issued to enable OCD exit.*/
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_EXT_LMR_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *((unsigned char *)(pDdr + DDR2_BA0(device))) = 0;  /* The write address must be chosen so that BA[1] is set to 0 and BA[0] is set to 1.*/
    /* wait 2 cycles min */
	udelay(1);

/* Step 19: A mode Normal command is provided. Program the Normal mode into Mode Register. */
    //MPDDRC->MPDDRC_MR
	writel(AT91C_DDRC2_MODE_NORMAL_CMD, (AT91C_BASE_MPDDRC + HDDRSDRC2_MR));
    *(pDdr) = 0;

/* Step 20: Perform a write access to any DDR2-SDRAM address */
    *(pDdr) = 0;

/* Step 21: Write the refresh rate into the count field in the Refresh Timer register. The DDR2-SDRAM device requires a refresh every 15.625 s or 7.81 s. 
   With a 100MHz frequency, the refresh timer count register must to be set with (15.625 /100 MHz) = 1562 i.e. 0x061A or (7.81 /100MHz) = 781 i.e. 0x030d. */
    /* For MT47H64M16HR, The refresh period is 64ms (commercial), This equates to an average
       refresh rate of 7.8125s (commercial), To ensure all rows of all banks are properly 
       refreshed, 8192 REFRESH commands must be issued every 64ms (commercial) */
    /* ((64 x 10(^-3))/8192) x133 x (10^6) */
    //MPDDRC->MPDDRC_RTR
    reg = ddramc_reg.rtr;//AT91C_DDRC2_RTR_COUNT(300); /* Set Refresh timer 7.8125 us*/
	writel(reg, (AT91C_BASE_MPDDRC + HDDRSDRC2_RTR));
    /* OK now we are ready to work on the DDRSDR */
    /* wait for end of calibration */
	udelay(10);

////#else
//#endif
//	udelay(10000);
//
//
//	/* DDRAM2 Controller initialize */
//	ddram_initialize(AT91C_BASE_MPDDRC, AT91C_BASE_DDRCS, &ddramc_reg);
//#endif
//
//#endif

}
#endif /* #ifdef CONFIG_DDR2 */

#if 0
static void one_wire_hw_init(void)
{
	const struct pio_desc one_wire_pio[] = {
		{"1-Wire", AT91C_PIN_PE(25), 1, PIO_DEFAULT, PIO_OUTPUT},
		{(char *)0, 0, 0, PIO_DEFAULT, PIO_PERIPH_A},
	};

	writel((1 << AT91C_ID_PIOE), (PMC_PCER + AT91C_BASE_PMC));
	pio_configure(one_wire_pio);
}
#endif

#if defined(CONFIG_NANDFLASH_RECOVERY) || defined(CONFIG_DATAFLASH_RECOVERY)
static void recovery_buttons_hw_init(void)
{
	/* Configure recovery button PINs */
	const struct pio_desc recovery_button_pins[] = {
		{"RECOVERY_BUTTON", CONFIG_SYS_RECOVERY_BUTTON_PIN, 0, PIO_PULLUP, PIO_INPUT},
		{(char *)0, 0, 0, PIO_DEFAULT, PIO_PERIPH_A},
	};
asfsd
	writel((1 << AT91C_ID_PIOE), PMC_PCER + AT91C_BASE_PMC);
	pio_configure(recovery_button_pins);
}
#endif /* #if defined(CONFIG_NANDFLASH_RECOVERY) || defined(CONFIG_DATAFLASH_RECOVERY) */

#if 0
static void HDMI_Qt1070_workaround(void)
{
	/* For the HDMI and QT1070 shar the irq line
	 * if the HDMI does not initialize, the irq line is pulled down by HDMI,
	 * so, the irq line can not used by QT1070
	 */
	 sfasf
	pio_set_gpio_output(AT91C_PIN_PC(31), 1);
	udelay(33000);
	pio_set_gpio_output(AT91C_PIN_PC(31), 0);
	udelay(33000);
	pio_set_gpio_output(AT91C_PIN_PC(31), 1);
}
#endif

#ifdef CONFIG_HW_INIT
void hw_init(void)
{
	/* Disable watchdog */
	at91_disable_wdt();

	/* At this stage the main oscillator is supposed to be enabled PCK = MCK = MOSC */

	/* Configure PLLA = MOSC * (PLL_MULA + 1) / PLL_DIVA */
	pmc_cfg_plla(PLLA_SETTINGS, PLL_LOCK_TIMEOUT);

	/* Initialize PLLA charge pump */
	pmc_init_pll(AT91C_PMC_IPLLA_3);

	/* PCK = PLLA/2 = 3 * MCK */
	pmc_cfg_mck(BOARD_PRESCALER_MAIN_CLOCK, PLL_LOCK_TIMEOUT);

	/* Switch MCK on PLLA output */
	pmc_cfg_mck(BOARD_PRESCALER_PLLA, PLL_LOCK_TIMEOUT);

	/* Enable External Reset */
	writel(AT91C_RSTC_KEY_UNLOCK | AT91C_RSTC_URSTEN, AT91C_BASE_RSTC + RSTC_RMR);

	/* Init timer */
	timer_init();

#ifdef CONFIG_SCLK
	slowclk_enable_osc32();
#endif

	/* initialize the dbgu */
	initialize_dbgu();

#ifdef CONFIG_DDR2
	/* Initialize MPDDR Controller */
	ddramc_init();
	char ret = ExtRAM_TestOk();
if(ret == 0)
    dbg_log(1, " ret = %x ...\n\r", ret);

#endif

#if 0
	/* load one wire information */
	one_wire_hw_init();
#endif

#ifdef CONFIG_USER_HW_INIT
	hw_init_hook();
#endif

#if 0
	HDMI_Qt1070_workaround();
#endif

#if defined(CONFIG_NANDFLASH_RECOVERY) || defined(CONFIG_DATAFLASH_RECOVERY)
	/* Init the recovery buttons pins */
	recovery_buttons_hw_init();
#endif
}
#endif /* #ifdef CONFIG_HW_INIT */

#ifdef CONFIG_DATAFLASH
void at91_spi0_hw_init(void)
{
	/* Configure PIN for SPI0 */
	const struct pio_desc spi0_pins[] = {
		{"MISO",	AT91C_PIN_PD(10), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MOSI",	AT91C_PIN_PD(11), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"SPCK",	AT91C_PIN_PD(12), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"NPCS",	CONFIG_SYS_SPI_PCS, 1, PIO_DEFAULT, PIO_OUTPUT},
		{(char *)0, 0, 0, PIO_DEFAULT, PIO_PERIPH_A},
	};

	/* Configure the PIO controller */
	writel((1 << AT91C_ID_PIOD), (PMC_PCER + AT91C_BASE_PMC));
	pio_configure(spi0_pins);

	/* Enable the clock */
	writel((1 << AT91C_ID_SPI0), (PMC_PCER + AT91C_BASE_PMC));
}
#endif /* #ifdef CONFIG_DATAFLASH */

#ifdef CONFIG_SDCARD
static void sdcard_set_of_name_board(char *of_name)
{
	/* CPU TYPE*/
	switch (get_cm_sn()) {
	case BOARD_ID_SAMA5D31_CM:
		strcpy(of_name, "sama5d31ek");
		break;

	case BOARD_ID_SAMA5D33_CM:
		strcpy(of_name, "sama5d33ek");
		break;

	case BOARD_ID_SAMA5D34_CM:
		strcpy(of_name, "sama5d34ek");
		break;

	case BOARD_ID_SAMA5D35_CM:
		strcpy(of_name, "sama5d35ek");
		break;

	case BOARD_ID_SAMA5D36_CM:
		strcpy(of_name, "sama5d36ek");
		break;

	default:
		dbg_log(1, "WARNING: Not correct CPU board ID\n\r");
		break;
	}

	if (get_dm_sn() == BOARD_ID_PDA_DM)
		strcat(of_name, "_pda");

	strcat(of_name, ".dtb");
}

void at91_mci0_hw_init(void)
{
	const struct pio_desc mci_pins[] = {
		{"MCCK", AT91C_PIN_PD(9), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MCCDA", AT91C_PIN_PD(0), 0, PIO_DEFAULT, PIO_PERIPH_A},

		{"MCDA0", AT91C_PIN_PD(1), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MCDA1", AT91C_PIN_PD(2), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MCDA2", AT91C_PIN_PD(3), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MCDA3", AT91C_PIN_PD(4), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MCDA4", AT91C_PIN_PD(5), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MCDA5", AT91C_PIN_PD(6), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MCDA6", AT91C_PIN_PD(7), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{"MCDA7", AT91C_PIN_PD(8), 0, PIO_DEFAULT, PIO_PERIPH_A},
		{(char *)0, 0, 0, PIO_DEFAULT, PIO_PERIPH_A},
	};

	/* Configure the PIO controller */
	writel((1 << AT91C_ID_PIOD), (PMC_PCER + AT91C_BASE_PMC));
	pio_configure(mci_pins);

	/* Enable the clock */
	writel((1 << AT91C_ID_HSMCI0), (PMC_PCER + AT91C_BASE_PMC));

	/* Set of name function pointer */
	sdcard_set_of_name = &sdcard_set_of_name_board;
}
#endif /* #ifdef CONFIG_SDCARD */

#ifdef CONFIG_NANDFLASH
void nandflash_hw_init(void)
{
	/* Configure nand pins */
	const struct pio_desc nand_pins[] = {
		{"NANDALE", AT91C_PIN_PE(21), 0, PIO_PULLUP, PIO_PERIPH_A},
		{"NANDCLE", AT91C_PIN_PE(22), 0, PIO_PULLUP, PIO_PERIPH_A},
		{(char *)0, 0, 0, PIO_DEFAULT, PIO_PERIPH_A},
	};

	/* Configure the nand controller pins*/
	pio_configure(nand_pins);
	writel((1 << AT91C_ID_PIOE), (PMC_PCER + AT91C_BASE_PMC));

	/* Enable the clock */
	writel(1 << AT91C_ID_SMC, (PMC_PCER + AT91C_BASE_PMC));

	/* Configure SMC CS3 for NAND/SmartMedia */
	writel(AT91C_SMC_SETUP_NWE(1)
		| AT91C_SMC_SETUP_NCS_WR(1) 
		| AT91C_SMC_SETUP_NRD(2) 
		| AT91C_SMC_SETUP_NCS_RD(1), 
		(ATMEL_BASE_SMC + SMC_SETUP3));

	writel(AT91C_SMC_PULSE_NWE(5)
		| AT91C_SMC_PULSE_NCS_WR(7)
		| AT91C_SMC_PULSE_NRD(5)
		| AT91C_SMC_PULSE_NCS_RD(7), 
	 	(ATMEL_BASE_SMC + SMC_PULSE3));

	writel(AT91C_SMC_CYCLE_NWE(8)
		| AT91C_SMC_CYCLE_NRD(9), 
		(ATMEL_BASE_SMC + SMC_CYCLE3));

	writel(AT91C_SMC_TIMINGS_TCLR(3)
		| AT91C_SMC_TIMINGS_TADL(10)
		| AT91C_SMC_TIMINGS_TAR(3)
		| AT91C_SMC_TIMINGS_TRR(4)
		| AT91C_SMC_TIMINGS_TWB(5)
		| AT91C_SMC_TIMINGS_RBNSEL(3)
		| AT91C_SMC_TIMINGS_NFSEL,
		(ATMEL_BASE_SMC + SMC_TIMINGS3));

	writel(AT91C_SMC_MODE_READMODE_NRD_CTRL
		| AT91C_SMC_MODE_WRITEMODE_NWE_CTRL
		| AT91C_SMC_MODE_EXNWMODE_DISABLED
		| AT91C_SMC_MODE_DBW_8
		| AT91C_SMC_MODE_TDF_CYCLES(1),
		(ATMEL_BASE_SMC + SMC_MODE3));
}

void nandflash_config_buswidth(unsigned char buswidth)
{
	unsigned int mode; 

	mode = readl(ATMEL_BASE_SMC + SMC_MODE3);

	mode &= ~AT91C_SMC_MODE_DBW;
	if (buswidth == 0)	/* 8 bits bus */
		mode |= AT91C_SMC_MODE_DBW_8;	
	else 
		mode |= AT91C_SMC_MODE_DBW_16;

	writel(mode, (ATMEL_BASE_SMC + SMC_MODE3));
}
#endif /* #ifdef CONFIG_NANDFLASH */
