#include <types.h>

#include <console.h>

#define DEBUGMSG

#define XMODEM1K

#ifdef XMODEM1K
#define SOH	0x01
#define STX	0x02
#else
#define SOH	0x01
#endif
#define EOT	0x04
#define ACK	0x06
#define NAK	0x15
#define CAN	0x18

#if 0
#ifdef XMODEM1K
#define	BLKSIZE	1024
#else
#define	BLKSIZE	128
#endif

#define PKTSIZE	(BLKSIZE + 5)
#endif


#define BYTETIMEOUT	1	/* 1  */
#define PKTTIMEOUT	10	/* 10  */


VOID* Xmemcpy(VOID *dest,const VOID *src, INT32 count)
{
	CHAR *tmp = (CHAR *) dest, *s = (CHAR *) src;
	
	while (count--)
		*tmp++ = *s++;
	
	return dest;
}

VOID
XSendByte(BYTE data)
{
	///DEV_WriteComm(data);
	ser_putch(data);
}

INT32
getchDebugSerialNB(CHAR *pdata)
{
	INT32 ret;
	
	///if(DEV_KbhitComm()==0) return 0;
	///*pdata=DEV_ReadComm(); 

	ret = ser_getc();
	if( ( ret < 0) )
		return 0;

	*pdata = (CHAR)ret;
	
	return 1;
#if 0    
	BYTE   ucStatus;
	// UART 0: linestatus
	ucStatus = *((VOLATILE BYTE *)(0x80020000 + 0x14));
	if (ucStatus & 0x0E)
	{
		return 0;
	}
	
	if (ucStatus & 0x01) 
	{
		// UART 0: rxbuffer
		*pdata = *((VOLATILE BYTE *)(0x80020000 + 0x00));
		return 1;
	}
	
	return 0;
#endif    
}

VOID
delayFunc(VOID)
{
	VOLATILE INT32 a = 10, b = 20, c = 40, d = 50;

	a *= 10;
	b = a * c + d;
	c = 30 + b;
	d = a + b + c * d;
}

#define DELVALPERSEC	(25000 * 100)

INT32
XReceiveByte(BYTE *pdata, UINT32 timeout)
{
	UINT32	delVal = 0;
	
	while (!getchDebugSerialNB((CHAR *)pdata)) 
	{
		if (delVal > DELVALPERSEC * timeout)
			return 0;

		delayFunc();
		delVal++;
	}
	
	return 1;
}

/* update CRC */
UINT16
updcrc(UINT16 data, UINT16 crc)
{
	INT32		i;

	data <<= 8;
	for (i = 8; i > 0; i--) 
	{
		if ((data ^ crc) & 0x8000) 
			crc = (crc << 1) ^ 0x1021;
		else 
			crc <<= 1;
		
		data <<= 1;
	}

	return crc;
}

INT32
checkCRC(BYTE *dataBuf, INT32 datalen, BYTE *crcBuf)
{
	UINT16	crc = 0;
	INT32			i;

	for (i = 0; i < datalen; i++)
		crc = updcrc(dataBuf[i], crc);

	if (crcBuf[0] == ((crc & 0xFF00) >> 8) && crcBuf[1] == (crc & 0x00FF))
		return 1;

	return 0;
}

/*
 * Extern Function :
 *	XReceiveByte(BYTE *pdata, UINT32 timeout)
 *		0 : no data
 *		1 : data
 * Retern Value
 *	SOH, STX : general pkt
 *	EOT : eot 
 *	CAN : can 
 * 	0 : pkt timeout
 *	-1 : byte timeout
 *	-2 : header error
 */
INT32
getPacket(BYTE *pkt, INT32 pkttimeout)
{
	INT32		nrecv;
	INT32		pktsize;

	if (!XReceiveByte(pkt, pkttimeout)) 
	{
		return 0;
	}
	switch (pkt[0]) 
	{
	case SOH:
		pktsize = 128 + 5;
		break;
	case STX:
		pktsize = 1024 + 5;
		break;
	case EOT:
		return EOT;
	case CAN:
		return CAN;
	default:
		return -2;
	}
	nrecv = 1;
    
	while (1) 
	{
		if (!XReceiveByte(pkt + nrecv, BYTETIMEOUT)) 
			return -1;

		nrecv++;
	
		if (nrecv >= pktsize) 	
			return pkt[0];
	}
	
	return 0;
}

/*
 *  : ŵ byte
 *  :
 *	-1 : ʹ  timeout  error ߻
 *	-2 : ۽ ۽Ҹ 
 *	-3 : buffer size ʹ 
 */
INT32
recvXModem(BYTE *buf, INT32 bufsize)
{
	INT32			i;
	BYTE	pkt[1024 + 5];
	BYTE	pktno = 1;
	INT32		 	recvsize = 0;
	INT32			rv;
	INT32			nerr = 0;
	INT32			blksize = 128;
	INT32			isCrcErr, isNoErr, isRNoErr;

	/*
	 * X modem protocol with CRC16
	 */
	for (i = 0; i < 60; i++) 
	{
		XSendByte('C');
		rv = getPacket(pkt, BYTETIMEOUT);
		if (rv != 0) 
		{
			break;
		}
	}
	if (i >= 60) 
	{
		return -1;
	}

	do {
		switch (rv) 
		{
		case SOH:
			blksize = 128;
			break;
		case STX:
			blksize = 1024;
			break;
		case CAN:
			return -2;
		case EOT:
			XSendByte(ACK);
			return recvsize;
		default:
#ifdef DEBUGMSG
			console_printf("Time Out\n");
#endif
			nerr++;
			if (nerr > 10) 
			{
				return -1;
			}
			XSendByte(NAK);
			continue;
		}
	
		/*
		 * Packet Check
		 */
		if (pkt[1] != pktno)		isNoErr = 1;
		else				isNoErr = 0;

		if (pkt[2] != 255 - pkt[1])	isRNoErr = 1;
		else				isRNoErr = 0;

		if (!checkCRC(pkt + 3, blksize, pkt + 3 + blksize))	isCrcErr = 1;
		else 							isCrcErr = 0;

		if (isNoErr && !isRNoErr && !isCrcErr &&
			    pktno >= 2 && pkt[1] == pktno - 1)
		{
#ifdef DEBUGMSG
			console_printf("packet # error but continue (%d != %d)\n", pkt[1], pktno);
#endif	    
			nerr = 0;
			if (bufsize - recvsize < blksize)
				return -3;

			Xmemcpy(buf + recvsize, pkt + 3, blksize);
			recvsize += blksize;
			XSendByte(ACK);
		} 
		else if (isNoErr || isRNoErr || isCrcErr) 
		{
#ifdef DEBUGMSG
			if (isNoErr) 
				console_printf("packet # error (%d != %d)\n", pkt[1], pktno);

			if (isRNoErr) 
				console_printf("packet #  error (%d != %d)\n", pkt[2], 255 - pkt[1]);

			if (isCrcErr) 
				console_printf("CRC error\n");
#endif
			nerr++;
			XSendByte(NAK);
		} 
		else 
		{
			nerr = 0;
			if (bufsize - recvsize < blksize) 
				return -3;

			Xmemcpy(buf + recvsize, pkt + 3, blksize);
			recvsize += blksize;
			XSendByte(ACK);
			pktno++;
		}
	} while ((rv = getPacket(pkt, PKTTIMEOUT)) != CAN);

	return -2;
}
