#include <types.h>

typedef long                    LONG;
typedef unsigned long           ULONG; 
#include <gl/gl.h>

#include <lib/string.h>

#include <panic.h>



#define MAXCOLORMAPSIZE     256

typedef struct _GIFSCREEN_
{
    UINT32      Width;
    UINT32      Height;
    BYTE        ColorMap[3][MAXCOLORMAPSIZE];
    UINT32      BitPixel;
    UINT32      ColorResolution;
    UINT32      Background;
    UINT32      AspectRatio;
    INT32       GrayScale;
}GIFSCREEN;

typedef struct _GIF89_
{
    UINT32      cTrans;
    UINT32      DelayTime;
    UINT32      InputFlag;
    UINT32      Disposal;
}GIF89;

typedef struct _GIF_INFO_
{
    PBYTE       pStart;     // start pointer to the beginning of buffer;
    UINT32      Offset;     // current offset
    UINT32      Size;       // total size of buffer of gif image

    GIFSCREEN   GifScreen;
    GIF89       Gif89;

}GIF_INFO, *GIF_PINFO;

static GIF_INFO GifInfo;

//====================================================================================
#define MAX_LWZ_BITS        12
#define INTERLACE           0x40
#define LOCALCOLORMAP       0x80

#define CM_RED              0
#define CM_GREEN            1
#define CM_BLUE             2

//====================================================================================
// GIF decoding routine

#define TESTBITS( Byte, Bit )           ( ((Byte) & (Bit)) == (Bit) )

#define BITSET(byte, bit)               (((byte) & (bit)) == (bit))
#define ReadOK(Buffer,Len)              BufRead( Buffer, Len)
#define BYTE2_TO_UINT(a,b)              (((b)<<8)|(a))

//====================================================================================
EXTERN UINT32 PlatformReadFunction(PBYTE pBuffer, UINT32 StartOffset, INT32 ReadLen );

#if 0
#define GIF_BUF_SIZE    0x20000
static char GifBuf[GIF_BUF_SIZE];

PUBLIC BOOL
InitGIFBuffer( PVOID pBuf, UINT32 Size )
{
    GifInfo.pStart = pBuf;
    GifInfo.Offset = 0;
    GifInfo.Size = Size;

    #define ROM_BASE        0x40000
    PlatformReadFunction( GifBuf, GifInfo.pStart - ROM_BASE, GIF_BUF_SIZE); 

    return TRUE;
}

STATIC INT32
BufRead( PVOID pBuf, UINT32 Size )
{
	UINT32      CopySize;

    if( GifInfo.Offset >= GifInfo.Size )
        return 0;	/* EOF*/

    if( GifInfo.Offset + Size > GifInfo.Size ) 
        CopySize = GifInfo.Size - GifInfo.Offset;
    else 
        CopySize = Size;

    memcpy( pBuf, GifBuf + GifInfo.Offset, CopySize);
    GifInfo.Offset += CopySize;

//    char *p = pBuf;
//console_printf( "pBuf[0] : %x\n\r", p[0]);
//console_printf( "pBuf[1] : %x\n\r", p[1]);
//console_printf( "pBuf[2] : %x\n\r", p[2]);
//console_printf( "pBuf[3] : %x\n\r", p[3]);
//console_printf( "pBuf[4] : %x\n\r", p[4]);
//
//console_printf( "BufRead : %x\n\r", GifInfo.pStart + GifInfo.Offset);

	return CopySize;
}
#else
PUBLIC BOOL
InitGIFBuffer( PVOID pBuf, UINT32 Size )
{
    GifInfo.pStart = pBuf;
    GifInfo.Offset = 0;
    GifInfo.Size = Size;

    return TRUE;
}

STATIC INT32
BufRead( PVOID pBuf, UINT32 Size )
{
	UINT32      CopySize;

    if( GifInfo.Offset >= GifInfo.Size )
        return 0;	/* EOF*/

    if( GifInfo.Offset + Size > GifInfo.Size ) 
        CopySize = GifInfo.Size - GifInfo.Offset;
    else 
        CopySize = Size;

    //memcpy( pBuf, GifInfo.pStart + GifInfo.Offset, CopySize);
    #define ROM_BASE        0x40000 
    PlatformReadFunction( pBuf, GifInfo.pStart + GifInfo.Offset - ROM_BASE, CopySize); 
    GifInfo.Offset += CopySize;

//    char *p = pBuf;
//console_printf( "pBuf[0] : %x\n\r", p[0]);
//console_printf( "pBuf[1] : %x\n\r", p[1]);
//console_printf( "pBuf[2] : %x\n\r", p[2]);
//console_printf( "pBuf[3] : %x\n\r", p[3]);
//console_printf( "pBuf[4] : %x\n\r", p[4]);
//
//console_printf( "BufRead : %x\n\r", GifInfo.pStart + GifInfo.Offset);

	return CopySize;
}
#endif

STATIC INT32
BufSeek( INT32 Offset )
{
    INT32       CurOffset;

    CurOffset = GifInfo.Offset;
    GifInfo.Offset = Offset;

    return CurOffset;
}

//=================================================================================


#define PIX2BYTES(n)	(((n)+7)/8)
/*
 * compute image line size and bytes per pixel
 * from bits per pixel and width
 */
STATIC VOID
ComputePitch(int bpp, int width, int *pitch, int *bytesperpixel)
{
	int	linesize;
	int	bytespp = 1;

	if(bpp == 1)
		linesize = PIX2BYTES(width);
	else if(bpp <= 4)
		linesize = PIX2BYTES(width<<2);
	else if(bpp <= 8)
		linesize = width;
	else if(bpp <= 16) {
		linesize = width * 2;
		bytespp = 2;
	} else if(bpp <= 24) {
		linesize = width * 3;
		bytespp = 3;
	} else {
		linesize = width * 4;
		bytespp = 4;
	}

	/* rows are DWORD right aligned*/
	*pitch = (linesize + 3) & ~3;
	*bytesperpixel = bytespp;
}

STATIC INT32 ZeroDataBlock = FALSE;

STATIC INT32
GetDataBlock( unsigned char *buf)
{
    unsigned char count;

    if (!ReadOK( &count, 1))
    return -1;
    ZeroDataBlock = count == 0;

    if ((count != 0) && (!ReadOK( buf, count)))
    return -1;
    return count;
}

//=================================================================================

STATIC INT32
ReadColorMap( int number, unsigned char buffer[3][MAXCOLORMAPSIZE], int *gray)
{
    int i;
    unsigned char rgb[3];
    int flag;

    flag = TRUE;

    for (i = 0; i < number; ++i) 
    {
        if (!ReadOK( rgb, sizeof(rgb)))
            return 1;
#if 1
        buffer[CM_RED][i] = rgb[0];
        buffer[CM_GREEN][i] = rgb[1];
        buffer[CM_BLUE][i] = rgb[2];
#else
        buffer[CM_RED][i] = rgb[2];
        buffer[CM_GREEN][i] = rgb[1];
        buffer[CM_BLUE][i] = rgb[0];
#endif
        flag &= (rgb[0] == rgb[1] && rgb[1] == rgb[2]);
    }

#if 0
    if (flag)
    *gray = (number == 2) ? PBM_TYPE : PGM_TYPE;
    else
    *gray = PPM_TYPE;
#else
    *gray = 0;
#endif

    return FALSE;
}

STATIC INT32
DoExtension( int label)
{
    STATIC unsigned char buf[256];

    switch (label)
    {
    case 0x01:            /* Plain Text Extension */
        break;
    case 0xff:            /* Application Extension */
        break;
    case 0xfe:            /* Comment Extension */
        while (GetDataBlock( (unsigned char *) buf) != 0);
        return FALSE;
    case 0xf9:            /* Graphic Control Extension */
        GetDataBlock( (unsigned char *) buf);
        GifInfo.Gif89.Disposal = (buf[0] >> 2) & 0x7;
        GifInfo.Gif89.InputFlag = (buf[0] >> 1) & 0x1;
        GifInfo.Gif89.DelayTime = BYTE2_TO_UINT(buf[1], buf[2]);
        if ((buf[0] & 0x1) != 0)
            GifInfo.Gif89.cTrans = buf[3];
    
        while (GetDataBlock( (unsigned char *) buf) != 0);

//DEBUG2(1,"DoExtension : Gif89.Disposal = %x\n", Gif89.Disposal ); 
//DEBUG2(1,"DoExtension : Gif89.InputFlag = %x\n", Gif89.InputFlag ); 
//DEBUG2(1,"DoExtension : Gif89.DelayTime = %x\n", Gif89.DelayTime ); 
//DEBUG2(1,"DoExtension : Gif89.cTrans = %x\n", Gif89.cTrans ); 

        return FALSE;
    default:
        break;
    }

    while (GetDataBlock( (unsigned char *) buf) != 0);

    return FALSE;
}


STATIC INT32
GetCode( int code_size, int flag)
{
    STATIC unsigned char buf[280];
    STATIC int curbit, lastbit, done, last_byte;
    int i, j, ret;
    unsigned char count;

    if (flag) 
    {
        curbit = 0;
        lastbit = 0;
        done = FALSE;
        return 0;
    }
    if ((curbit + code_size) >= lastbit) 
    {
        if (done) 
        {
            if (curbit >= lastbit)
            {
                //DEBUG1(1,"LoadGIF: bad decode\n");
            }
            return -1;
        }
        buf[0] = buf[last_byte - 2];
        buf[1] = buf[last_byte - 1];
    
        if ((count = GetDataBlock( &buf[2])) == 0)
            done = TRUE;
    
        last_byte = 2 + count;
        curbit = (curbit - lastbit) + 16;
        lastbit = (2 + count) * 8;
    }

    ret = 0;
    for (i = curbit, j = 0; j < code_size; ++i, ++j)
        ret |= ((buf[i / 8] & (1 << (i % 8))) != 0) << j;

    curbit += code_size;

    return ret;
}

STATIC INT32
LWZReadByte( int flag, int input_code_size)
{
    int code, incode;
    register int i;
    STATIC int fresh = FALSE;
    STATIC int code_size, set_code_size;
    STATIC int max_code, max_code_size;
    STATIC int firstcode, oldcode;
    STATIC int clear_code, end_code;
    STATIC int table[2][(1 << MAX_LWZ_BITS)];
    STATIC int stack[(1 << (MAX_LWZ_BITS)) * 2], *sp;

    if (flag) 
    {
        set_code_size = input_code_size;
        code_size = set_code_size + 1;
        clear_code = 1 << set_code_size;
        end_code = clear_code + 1;
        max_code_size = 2 * clear_code;
        max_code = clear_code + 2;

        GetCode( 0, TRUE);

        fresh = TRUE;

        for (i = 0; i < clear_code; ++i) 
        {
            table[0][i] = 0;
            table[1][i] = i;
        }
        for (; i < (1 << MAX_LWZ_BITS); ++i)
            table[0][i] = table[1][0] = 0;

        sp = stack;

        return 0;
    } 
    else 
    if (fresh) 
    {
        fresh = FALSE;
        do 
        {
            firstcode = oldcode = GetCode(code_size, FALSE);
        } while (firstcode == clear_code);

        return firstcode;
    }
    if (sp > stack)
        return *--sp;

    while ((code = GetCode(code_size, FALSE)) >= 0) 
    {
        if (code == clear_code) 
        {
            for (i = 0; i < clear_code; ++i) 
            {
                table[0][i] = 0;
                table[1][i] = i;
            }
            for (; i < (1 << MAX_LWZ_BITS); ++i)
            table[0][i] = table[1][i] = 0;
            code_size = set_code_size + 1;
            max_code_size = 2 * clear_code;
            max_code = clear_code + 2;
            sp = stack;
            firstcode = oldcode = GetCode(code_size, FALSE);
            return firstcode;
        } 
        else 
        if (code == end_code) 
        {
            int count;
            unsigned char buf[260];
    
            if (ZeroDataBlock)
            return -2;
    
            while ((count = GetDataBlock(buf)) > 0);
    
            if (count != 0) 
            {
            /*
             * DEBUG2(1,"missing EOD in data stream (common occurence)");
             */
            }
            return -2;
        }
        incode = code;

        if (code >= max_code) 
        {
            *sp++ = firstcode;
            code = oldcode;
        }
        while (code >= clear_code) 
        {
            *sp++ = table[1][code];
            if (code == table[0][code])
            {
                //DEBUG1(1,"LoadGIF: circular table entry\n");
            }
            code = table[0][code];
        }

        *sp++ = firstcode = table[1][code];

        if ((code = max_code) < (1 << MAX_LWZ_BITS)) 
        {
            table[0][code] = oldcode;
            table[1][code] = firstcode;
            ++max_code;
            if ((max_code >= max_code_size) &&
                (max_code_size < (1 << MAX_LWZ_BITS))) 
            {
                max_code_size *= 2;
                ++code_size;
            }
        }
        oldcode = incode;

        if (sp > stack)
            return *--sp;
    }
    return code;
}

STATIC INT32
ReadImage( PIMAGEHDR pImage,
           int startx, int starty, int width, int height, int cmapSize,
           unsigned char cmap[3][MAXCOLORMAPSIZE],
           int gray, int interlace, int ignore)
{
    unsigned char c;
    int i, v;
    int xpos = 0, ypos = 0, pass = 0;

//DEBUG2(1,"ReadImage: startx =  %d\n", startx);
//DEBUG2(1,"ReadImage: starty =  %d\n", starty);
//DEBUG2(1,"ReadImage: width  =  %d\n", width);
//DEBUG2(1,"ReadImage: height =  %d\n", height);

    /*
     *    Initialize the compression routines
     */
    if (!ReadOK( &c, 1)) 
    {
        //DEBUG1(1,"LoadGIF: EOF on image data\n");
        return 0;
    }
    if (LWZReadByte( TRUE, c) < 0) 
    {
        //DEBUG1(1,"LoadGIF: error reading image\n");
        return 0;
    }

    /*
     *    If this is an "uninteresting picture" ignore it.
     */
    if (ignore) 
    {
        while (LWZReadByte( FALSE, c) >= 0);
        return 0;
    }

    /*image = ImageNewCmap(width, height, cmapSize);*/

    //GifScreen.Width  = BYTE2_TO_UINT(Buf[0], Buf[1]);
    //GifScreen.Height = BYTE2_TO_UINT(Buf[2], Buf[3]);
    //GifScreen.BitPixel = 2 << (Buf[4] & 0x07);
    //GifScreen.ColorResolution = (((Buf[4] & 0x70) >> 3) + 1);
    //GifScreen.Background = Buf[5];
    //GifScreen.AspectRatio = Buf[6];

    pImage->Width = GifInfo.GifScreen.Width;
    pImage->Height = GifInfo.GifScreen.Height;
    pImage->nPlane = 1;
    pImage->nBpp = 8;
    ComputePitch(8, GifInfo.GifScreen.Width, &pImage->nStride, &pImage->nBytePerPixel);
    pImage->fCompression = IMAGE_FLAG_RGB;
    pImage->sPalette = cmapSize;

    while ((v = LWZReadByte( FALSE, c)) >= 0) 
    {
#if 1
        char *p = pImage->pImage + ((ypos + starty)* pImage->nStride + xpos + startx) * 4;
        *p++ = cmap[CM_RED][v];
        *p++ = cmap[CM_GREEN][v];
        *p++ = cmap[CM_BLUE][v];
        *p = 0xFF;
#else
        if (v != GifInfo.Gif89.cTrans)
            pImage->pImage[(ypos + starty)* pImage->nStride + xpos + startx] = v;
#endif
        ++xpos;
        if (xpos == width)
        {
            xpos = 0;
            if (interlace)
            {
                switch (pass)
                {
                case 0:
                case 1:
                    ypos += 8;
                    break;
                case 2:
                    ypos += 4;
                    break;
                case 3:
                    ypos += 2;
                    break;
                }

                if (ypos >= height) 
                {
                    ++pass;
                    switch (pass) 
                    {
                    case 1:
                        ypos = 4;
                        break;
                    case 2:
                        ypos = 2;
                        break;
                    case 3:
                        ypos = 1;
                        break;
                    default:
                        goto fini;
                    }
                }
            }
            else
            {
                ++ypos;
            }
        }
        if(ypos >= height)
            break;
    }

fini:
    return 1;
}

PUBLIC BOOL
TestGIF( VOID )
{
    CHAR        Version[4];
    BYTE        Buf[16];

    BufSeek( 0 );

    if( !ReadOK( Buf, 6) )
        return FALSE;

    if( strncmp( Buf, "GIF", 3) != 0 )
        return FALSE;

    strncpy( Version, Buf + 3, 3);
    Version[3] = NULL;

    if( strcmp(Version, "87a") != 0 && strcmp(Version, "89a") != 0 )
    {
        //DEBUG1(1,"LoadGIF: GIF version number not 87a or 89a\n");
        return FALSE;
    }

    GifInfo.Gif89.cTrans = -1;
    GifInfo.Gif89.DelayTime = -1;
    GifInfo.Gif89.InputFlag = -1;
    GifInfo.Gif89.Disposal = 0;

    if( !ReadOK( Buf, 7) )
    {
        //DEBUG1(1,"LoadGIF: bad screen descriptor\n");
        return FALSE;
    }

    GifInfo.GifScreen.Width  = BYTE2_TO_UINT(Buf[0], Buf[1]);
    GifInfo.GifScreen.Height = BYTE2_TO_UINT(Buf[2], Buf[3]);
    GifInfo.GifScreen.BitPixel = 2 << (Buf[4] & 0x07);
    GifInfo.GifScreen.ColorResolution = (((Buf[4] & 0x70) >> 3) + 1);
    GifInfo.GifScreen.Background = Buf[5];
    GifInfo.GifScreen.AspectRatio = Buf[6];

    // Global Colormap
    if( TESTBITS( Buf[4], LOCALCOLORMAP) )
    {
        if( ReadColorMap( GifInfo.GifScreen.BitPixel, GifInfo.GifScreen.ColorMap, &GifInfo.GifScreen.GrayScale))
        {
            //DEBUG1(1,"LoadGIF: bad global colormap\n");
            return FALSE;
        }
    }

    return TRUE;
}

PUBLIC BOOL
LoadGIF( PIMAGEHDR pImage, PVOID pGifImage )
{
    BYTE        c;
    int         ImageCount = 0;
    int         ImageNumber = 1;
    BYTE        Buf[16];
    BYTE        LocalColorMap[3][MAXCOLORMAPSIZE];
    BOOL        bRes = FALSE;
    UINT32      BitPixel;
    int         GrayScale;

    BOOL        bUseGlobalColorMap; //߿

    pImage->pImage = pGifImage;


    do
    {
        if( !ReadOK(&c, 1) )
        {
            //DEBUG1(1,"LoadGIF: EOF on image data\n");
            goto ERR_GIF;
        }

        // GIF terminator
        if( c == ';')
        {
            if( ImageCount < ImageNumber )
            {
                //DEBUG2(1,"LoadGIF: no image %d\n", ImageNumber);
                //DEBUG2(1," of %d\n", ImageCount);
                goto ERR_GIF;
            }
        }

        // Extension
        if (c == '!')
        {
            if( !ReadOK(&c, 1) )
            {
                //DEBUG1(1,"LoadGIF: EOF on extension function code\n");
                goto ERR_GIF;
            }
            DoExtension( c);
            continue;
        }

        // Not a valid start character
        if( c != ',' )
        {
            continue;
        }
        ++ImageCount;

        if (!ReadOK(Buf, 9))
        {
            //DEBUG1(1,"LoadGIF: bad image size\n");
            goto ERR_GIF;
        }

        bUseGlobalColorMap = !TESTBITS( Buf[8], LOCALCOLORMAP );
        BitPixel = 1 << ((Buf[8] & 0x07) + 1);

        if( !bUseGlobalColorMap )
        {
            if (ReadColorMap( BitPixel, LocalColorMap, &GrayScale))
            {
                //DEBUG1(1,"LoadGIF: bad local colormap\n");
                goto ERR_GIF;
            }
            bRes = ReadImage( pImage, 
                  BYTE2_TO_UINT( Buf[0], Buf[1] ),
                  BYTE2_TO_UINT( Buf[2], Buf[3] ),
                  BYTE2_TO_UINT( Buf[4], Buf[5] ),
                  BYTE2_TO_UINT( Buf[6], Buf[7] ),
                  BitPixel, LocalColorMap, 
                  GrayScale, BITSET(Buf[8], INTERLACE),
                  ImageCount != ImageNumber);
        }
        else
        {
            bRes = ReadImage( pImage, 
                  BYTE2_TO_UINT( Buf[0], Buf[1] ),
                  BYTE2_TO_UINT( Buf[2], Buf[3] ),
                  BYTE2_TO_UINT( Buf[4], Buf[5] ),
                  BYTE2_TO_UINT( Buf[6], Buf[7] ),
                  GifInfo.GifScreen.BitPixel, GifInfo.GifScreen.ColorMap,
                  GifInfo.GifScreen.GrayScale, TESTBITS(Buf[8], INTERLACE),
                  ImageCount != ImageNumber);
        }
    }while( bRes == 0 );

    // set transparent color, if any
    pImage->cTrans = GifInfo.Gif89.cTrans;

    if( bRes )
        return TRUE;

ERR_GIF:

    return FALSE;
}



