#include "asm/linkage.h"
#include "H7202.h"

#define DRAM_SIZE       0x04000000
#define STACK_SIZE      0x80000




#define SC_BASE_PHYSICAL        0x90000000
#define MEMC_BASE_PHYSICAL      0xA0000000
#define DRAM_BANK1A_PHYSICAL    0x40000000


#define MDCNFG_OFFSET           (0)
#define MDCAS00_OFFSET          (0x4) 
#define MDCAS01_OFFSET          (0x8)
#define MDCAS02_OFFSET          (0xC)
#define MSC0_OFFSET             (0x10)
#define MSC1_OFFSET             (0x14)
#define MECR_OFFSET             (0x18)
#define MDREFR_OFFSET           (0x1C)
#define MDCAS20_OFFSET          (0x20) 
#define MDCAS21_OFFSET          (0x24)
#define MDCAS22_OFFSET          (0x28)
#define MSC2_OFFSET             (0x2C)
#define SMCNFG_OFFSET           (0x30)

#define MC_BASE_PHYSICAL        (MEMC_BASE_PHYSICAL)
#define MDCNFG_PHYSICAL         (MC_BASE_PHYSICAL+MDCNFG_OFFSET)
#define MDCAS00_PHYSICAL        (MC_BASE_PHYSICAL+MDCAS00_OFFSET) 
#define MDCAS01_PHYSICAL        (MC_BASE_PHYSICAL+MDCAS01_OFFSET)
#define MDCAS02_PHYSICAL        (MC_BASE_PHYSICAL+MDCAS02_OFFSET) 
#define MSC0_PHYSICAL           (MC_BASE_PHYSICAL+MSC0_OFFSET)
#define MSC1_PHYSICAL           (MC_BASE_PHYSICAL+MSC1_OFFSET)
#define MECR_PHYSICAL           (MC_BASE_PHYSICAL+MECR_OFFSET)
#define MDRERF_PHYSICAL         (MC_BASE_PHYSICAL+MDREFR_OFFSET)
#define MDCAS20_PHYSICAL        (MC_BASE_PHYSICAL+MDCAS20_OFFSET) 
#define MDCAS21_PHYSICAL        (MC_BASE_PHYSICAL+MDCAS21_OFFSET)
#define MDCAS22_PHYSICAL        (MC_BASE_PHYSICAL+MDCAS22_OFFSET)
#define MSC2_PHYSICAL           (MC_BASE_PHYSICAL+MSC2_OFFSET)
#define SMCNFG_PHYSICAL         (MC_BASE_PHYSICAL+SMCNFG_OFFSET)
        #define UNENABLED_MDCNFG_VAL        0xa164a164
        #define ENABLED_MDCNFG_VAL          0xa165a165
        #define SMCNFG_VAL                  0xafccafcc
        #define MSC0_VAL                    0x46924692
        #define MSC1_VAL                    0x00110013
        #define MSC2_VAL                    0xFFFCFFFC
        #define MDCAS00_VAL                 0xaaaaaaa7
        #define MDCAS01_VAL                 0xaaaaaaaa
        #define MDCAS02_VAL                 0xaaaaaaaa
        #define MDCAS20_VAL                 0xaaaaaa7f
        #define MDCAS21_VAL                 0xaaaaaaaa
        #define MDCAS22_VAL                 0xaaaaaaaa
        #define MECR_VAL                    0x994a994a
        #define MDREFR_VAL                  0x04340051


#define RC_OFFSET           (0x30000)
#define RCSR_OFFSET         (0x4)
#define RC_BASE_PHYSICAL    (SC_BASE_PHYSICAL+RC_OFFSET)
#define RCSR_PHYSICAL       (RC_BASE_PHYSICAL+RCSR_OFFSET)
        #define RCSR_CLEAR        0xF			/* 0x7 */

#define GPIO_OFFSET          (0x40000)
#define GPLR_OFFSET          (0)
#define GPDR_OFFSET          (0x4)
#define GPSR_OFFSET          (0x8)
#define GPCR_OFFSET          (0xC)
#define GRER_OFFSET          (0x10)
#define GFER_OFFSET          (0x14)
#define GEDR_OFFSET          (0x18)
#define GAFR_OFFSET          (0x1C)

#define GPIO_BASE_PHYSICAL   (SC_BASE_PHYSICAL+GPIO_OFFSET)
#define GPLR_PHYSICAL        (GPIO_BASE_PHYSICAL+GPLR_OFFSET)
#define GPDR_PHYSICAL        (GPIO_BASE_PHYSICAL+GPDR_OFFSET)
#define GPSR_PHYSICAL        (GPIO_BASE_PHYSICAL+GPSR_OFFSET)
#define GPCR_PHYSICAL        (GPIO_BASE_PHYSICAL+GPCR_OFFSET)
#define GRER_PHYSICAL        (GPIO_BASE_PHYSICAL+GRER_OFFSET)
#define GFER_PHYSICAL        (GPIO_BASE_PHYSICAL+GFER_OFFSET)
#define GEDR_PHYSICAL        (GPIO_BASE_PHYSICAL+GEDR_OFFSET)
#define GAFR_PHYSICAL        (GPIO_BASE_PHYSICAL+GAFR_OFFSET)
        #define GPDR_SA1110DB_INIT        0x0201A7FC  @  0000 0010 0000 0001 1010 0111 1111 1100  MONES

#define PMC_OFFSET           0x20000
#define PMCR_OFFSET          (0)
#define PSSR_OFFSET          (0x4)
#define PSPR_OFFSET          (0x8)
#define PWER_OFFSET          (0xC)
#define PCFR_OFFSET          (0x10)
#define PPCR_OFFSET          (0x14)
#define PGSR_OFFSET          (0x18)

#define PMC_BASE_PHYSICAL    (SC_BASE_PHYSICAL+PMC_OFFSET)
#define PMCR_PHYSICAL        (PMC_BASE_PHYSICAL+PMCR_OFFSET)
#define PSSR_PHYSICAL        (PMC_BASE_PHYSICAL+PSSR_OFFSET)
#define PSPR_PHYSICAL        (PMC_BASE_PHYSICAL+PSPR_OFFSET)
#define PWER_PHYSICAL        (PMC_BASE_PHYSICAL+PWER_OFFSET)
#define PCFR_PHYSICAL        (PMC_BASE_PHYSICAL+PCFR_OFFSET)
#define PPCR_PHYSICAL        (PMC_BASE_PHYSICAL+PPCR_OFFSET)
#define PGSR_PHYSICAL        (PMC_BASE_PHYSICAL+PGSR_OFFSET)
        @ PPCR values 0=59mhz 3=103.2mhz,6=147Mhz, 7=162.2mhz, 10=206.4mhz 
        #define PPCR_CLOCK        0x0000000a



	.text

@ Entry
	.globl	_entry
_entry:

	add	pc, pc, #(vector_reset - . - 8)
	ldr	pc, [pc, #Lundefined - . - 8]
	ldr	pc, [pc, #Lswi - . - 8]
	ldr	pc, [pc, #Labort_prefetch - . - 8]
	ldr	pc, [pc, #Labort_data - . - 8]
	ldr	pc, [pc, #Lempty - . - 8]
	ldr	pc, [pc, #Lirq - . - 8]
	ldr	pc, [pc, #Lfiq - . - 8]
	
Lundefined:
	.word	0x40000004
Lswi:
	.word	0x40000008
Labort_prefetch:
	.word	0x4000000C
Labort_data:
	.word	0x40000010
Lempty:
	.word	0x40000014
Lirq:
	.word	0x40000018
Lfiq:
	.word	0x4000001C


vector_reset:

    /*
     * set the cpu to SVC32 mode
     */
    mrs r0, cpsr
    bic	r0, r0, #0x1f
    orr	r0, r0, #0x13
    msr	cpsr, r0

    /*
     * setting PMU default PMU[0x80001000]
     * 
     */ 
    ldr	r0, =0x80001000
    ldr	r1, =0x0000010f
    str	r1, [r0, #0x20]	
	
    /*
     * setting PLL3 update(0x40) | 70.0416MHz(0x26) 
     */
    ldr	r1, =0x00000066
    str	r1, [r0, #0x28]
	
    /*
     * goto RUN Mode 
     */
    mov	r1, #1
    str	r1, [r0]


    /*
     * flush v4 I/D caches
     */
    mov	r0, #0
    mcr	p15, 0, r0, c7, c7, 0	/* flush v3/v4 cache */
    mcr	p15, 0, r0, c8, c7, 0	/* flush v4 TLB */

    /*
     * disable MMU stuff and caches
     */
    mrc	p15,0,r0,c1,c0
    bic	r0, r0, #0x00002300	@ clear bits 13, 9:8 (--V- --RS)
    bic	r0, r0, #0x0000008f	@ clear bits 7, 3:0 (B--- WCAM)
    orr	r0, r0, #0x00000002	@ set bit 2 (A) Align
    mcr	p15,0,r0,c1,c0


InitMemoryController:

    ldr	r2, =0x00e200cc
    mov	r1, #0x80000000
    ldr	r3, [r1, #0]
    cmp	r3, r2
    beq	50f

    /*
     * idle status,CAS latency2:2(check SDRAM DataSheet),Write buffer enable
     * E[1:0] = 00, R = 0
     */
    mov	r1, #0x220000
    mov	r2, #0x80000000
    str	r1, [r2, #0]
	
    /* Wait for a while */
    mov	r1, #0x10000
10:
    sub	r1, r1, #1
    cmp	r1, #0
    bne 	10b

    /*
     * precharge
     * E[1:0] = 01, R = 0	
     */
    mov	r1, #0x08
    add	r1, r1, #0x220000
    mov	r2, #0x80000000
    str	r1, [r2, #0]			

    /* Wait for a while */
    mov	r1, #0x10000
20:
    sub	r1, r1, #1
    cmp	r1, #0
    bne	20b

    /*			
     *IO_SdramRefresh = 0x10;
     */
    mov	r2, #0x10
    mov	r1, #0x80000000
    str	r2, [r1, #4]

    /*
     *auto refresh 
     *E[1:0] = 00, R = 1
     */
    mov	r1, #0xa20000
    mov	r2, #0x80000000
    str	r1, [r2, #0]

    /* wait for a while */
    mov	r1, #0x10000
30:
    sub	r1, r1, #1
    cmp	r1, #0
    bne	30b

    /*
     *SDRAM Mode Register Setting
     *Tmp = IO(SdramModeReg | SdramModeBurstLength8 | SdramModeCAS2 |Device0);
     */
    mov	r1, #0x10000        /* SDRAM_CASL */
    add	r1, r1, #0x1800     /* SDRAM_MODE_BL8 */
    add	r1, r1, #0x44000000
    ldr	r1, [r1, #0]

    /*
     * Tmp = IO(SdramModeReg | SdramModeBurstLength8 | SdramModeCAS2 |Device1);
     */
    mov	r1, #0x10000        /* SDRAM_CASL */
    add	r1, r1, #0x1800     /* SDRAM_MODE_BL8 */
    add	r1, r1, #0x46000000
    ldr	r1, [r1, #0]

    ldr	r2, =0x00e200cc
    mov	r1, #0x80000000
    str	r2, [r1, #0]

    mov	r2, #0
    mov	r1, #0x80000000
    str	r2, [r1, #8]

    /*
     * Refresh Value = 0x320 for 16MB(128Mbit)x2
     * Refresh Value = 0x320/2 for 32MB(256Mbit)x2
     * For simplicity, Refresh value for 32MBx2 is used, and it would
     * be no problem for 16MBx2
     * 0x320 >> 1 = 0x190
     */
    mov	r2, #0x190
    mov	r1, #0x80000000
    str	r2, [r1, #4]

50:
    /*
     * static memory config cs0 (depending on boot bit BOOTSBIT[1:0]), 8 waits
     */
    ldr	r1, =0x80003000
    ldr	r0, [r1]
    and	r0, r0, #0x3
    orr	r0, r0, #0x40
    str	r0, [r1]
	
    /*
     * static memory config cs1 16bit, 8 waits
     */
    ldr	r0, =0x41
    str	r0, [r1,#0x04]
	
    /*
     * static memory config cs2 16bit, 8 waits
     */
    ldr	r0, =0x41
    str	r0, [r1,#0x08]
	
    /*
     * static memory config cs3 16bit, 8 waits
     */
	ldr	r0, =0x41
	str	r0, [r1,#0x0c]			
	



	/* copy text */
	mov	r1, #DRAM_BANK1A_PHYSICAL
	add	r1, r1, #DRAM_SIZE
	sub	r1, r1, #0x200000

	ldr	sp, =SYMBOL_NAME(_etext)
	sub	r4, sp, r1
	mov	r5, r1
	mov	r2, #0
1:	ldr	r0, [r2], #4
	str	r0, [r5], #4
	subs	r4, r4,	#4
	bgt	1b

	/* copy data */
	ldr	r2, =SYMBOL_NAME(_etext)
	sub	r2, r2, r1

	ldr	r2, =0x40000

	add	r1, r1, #0x40000
	ldr	sp, =SYMBOL_NAME(_edata)
	sub	sp, sp, r1
1:	ldr	r0, [r2], #4
	str	r0, [r1], #4
	subs	sp, sp, #4
	bgt	1b

	/* initialize bss */
	mov	r0, #0
	ldr	r1, =SYMBOL_NAME(_bss_start)
	ldr	r2, =SYMBOL_NAME(_end)
1:	str	r0, [r1], #4
	cmp	r1, r2
	blt	1b
	
	/* set stack location and size */
	add	sp, r2, #STACK_SIZE


	/* store ram size */
	mov	r4, #DRAM_BANK1A_PHYSICAL
	ldr	r0, =SYMBOL_NAME(ram_base)
	str	r4, [r0]
	mov	r4, #DRAM_SIZE
	ldr	r0, =SYMBOL_NAME(ram_size)
	str	r4, [r0]

	ldr	pc, 1f
1:	.word	switch

switch:	
	bl	SYMBOL_NAME(start_main)
l:	b	l

