#include "asm/linkage.h"

#define STACK_SIZE              0x00080000

#define Mode_SVC	            0x13
#define NoIntsMask              0x000000C0
@------------------------------------
@ MMU Cache/TLB/etc on/off functions
@------------------------------------
#define R1_M  		(1)
#define R1_A		(1<<1)
#define R1_C		(1<<2)
#define R1_W		(1<<3)
#define R1_SW		(1<<10)
#define R1_Z		(1<<11)
#define R1_I		(1<<12)
#define R1_V		(1<<13)
#define R1_RR       (1<<14)
#define R1_EE       (1<<25)
#define R1_TRE      (1<<28)
#define R1_AFE      (1<<29)
#define R1_TE       (1<<30) 

@This defines the area where the MMU descriptors are stored. Somehow if the stack is too big, the two will overlap and the sytem'll trap
#define SAMA5D3X_BASE_MMU_CFG              (0x20004000)	
#define SAMA5D3X_BASE_MMU_SMALL_CFG        (0x20009000)	

#define SAMA5D3X_BASE_IBOOT                0x00000000
#define SAMA5D3X_BASE_IROM                 0x00100000
#define SAMA5D3X_BASE_NFC_RAM              0x00200000
#define SAMA5D3X_BASE_SRAM                 0x00300000
#define SAMA5D3X_BASE_SMD                  0x00400000
#define SAMA5D3X_BASE_UDPHS_RAM            0x00500000
#define SAMA5D3X_BASE_UHP_OHCI             0x00600000
#define SAMA5D3X_BASE_UHP_EHCI             0x00700000
#define SAMA5D3X_BASE_AXI_MATRIX           0x00800000
#define SAMA5D3X_BASE_DAP                  0x00900000

#define SAMA5D3X_BASE_DDR_1	               0x20100000
#define SAMA5D3X_BASE_EBI0                 0x10000000
#define SAMA5D3X_BASE_DDR 	               0x20000000
#define SAMA5D3X_BASE_EBI1                 0x40000000
#define SAMA5D3X_BASE_EBI2                 0x50000000
#define SAMA5D3X_BASE_EBI3                 0x60000000
#define SAMA5D3X_BASE_NFC                  0x70000000
#define SAMA5D3X_BASE_UNDEF                0x80000000
#define SAMA5D3X_BASE_REG                  0xF0000000

#define SAMA5D3X_VIRT_EBI0                 0xD0000000
#define SAMA5D3X_VIRT_IBOOT                0xE0000000
#define SAMA5D3X_VIRT_IROM                 0xE0100000
#define SAMA5D3X_VIRT_NFC_RAM              0xE0200000
#define SAMA5D3X_VIRT_SRAM                 0xE0300000
#define SAMA5D3X_VIRT_SMD                  0xE0400000
#define SAMA5D3X_VIRT_UDPHS_RAM            0xE0500000
#define SAMA5D3X_VIRT_UHP_OHCI             0xE0600000
#define SAMA5D3X_VIRT_UHP_EHCI             0xE0700000
#define SAMA5D3X_VIRT_AXI_MATRIX           0xE0800000
#define SAMA5D3X_VIRT_DAP                  0xE0900000



/* **** CPWAIT ******************************************************** */
/* * Params: $Rd      : temporary read register                         */
/* *                                                                    */
/* * Returns: nothing                                                   */
/* *                                                                    */
/* * Effects: corrupts $Rd                                              */
/* *                                                                    */
/* *  This macro is used to wait for coprocessor operations to complete */
/* *                                                                    */
/* Ex:                                                                  */
/*                                                                      */
/*    CPWAIT r0                                                         */
/*                                                                      */
/* ******************************************************************** */

#if 1
    .macro CPWAIT,  Rd
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
    .endm
#else
    .macro CPWAIT,  Rd
        mrc     p15, 0, \Rd, c2, c0, 0       @ arbitrary read of CP15
        mov     \Rd, \Rd                     @ wait for it (foward dependency)
        sub     pc, pc, #4                   @ branch to next instruction
    .endm
#endif


    .text

        .section ".text.init",#alloc,#execinstr

    .globl    _entry
_entry:
    add    pc, pc, #(vector_reset - . - 8)
    ldr    pc, [pc, #Lundefined - . - 8]
    ldr    pc, [pc, #Lswi - . - 8]
    ldr    pc, [pc, #Labort_prefetch - . - 8]
    ldr    pc, [pc, #Labort_data - . - 8]
    ldr    pc, [pc, #Lempty - . - 8]
    ldr    pc, [pc, #Lirq - . - 8]
    ldr    pc, [pc, #Lfiq - . - 8]
    
Lundefined:
    .word    0xE0300004
Lswi:
    .word    0xE0300008
Labort_prefetch:
    .word    0xE030000C
Labort_data:
    .word    0xE0300010
Lempty:
    .word    0xE0300014
Lirq:
    .word    0xE0300018
Lfiq:
    .word    0xE030001C


vector_reset:

    /* Put the CPU in Supervisor mode (SVC) and disable IRQ and FIQ interrupts. */
    ldr     r0, =(Mode_SVC | NoIntsMask)
    msr     cpsr_c, r0

    
#if 0
    ldr     r0, =0x2043             @ enable access to all coprocessors
@    mcr     p15, 0, r0, c15, c1, 0  @
    CPWAIT  r0                      @

    mov     r0, #0
    mrc     p15, 0, r0, c1, c0, 0
    orr     r0, #(1<<0)      
    mcr     p15, 0, r0, c15, c1, 0  @
    CPWAIT  r0                      @

    /* Disable the MMU, caches, and write-buffer and flush. */
    mov     r0, #0
    mrc     p15, 0, r0, c1, c0, 0
    bic     r0, #(1<<0)|(1<<2)      
    mcr     p15, 0, r0, c15, c1, 0  @
    CPWAIT  r0                      @

    ldr     r0, =0x00000000         @ get a zero to turn things off
    mcr     p15, 0, r0, c8, c7, 0   @ flush (invalidate) I/D TLBs
    mcr     p15, 0, r0, c7, c7, 0   @ flush (invalidate) I/D caches
    mcr     p15, 0, r0, c7, c10, 4  @ drain the write buffer
    nop                             @
    nop                             @
    nop                             @


    mvn     r0, #0                  @ grant manager access to all domains
    mcr     p15, 0, r0, c3, c0, 0   @


    /* Icache enable and asynchronous clocking mode */
	mrc     p15,    0,r0,c1,c0,0
    @orr     r0,     r0,#0xC0000000
    orr     r0,     r0,#0x1000
	mcr     p15,    0,r0,c1,c0,0
#endif

#if 1
@    b   VirtualStart

    /* build the MMU table */

    /* ******************************************************************/
    /*  Load RVA of MMUAddressTable[]                                   */
    /*                                                                  */

    add     r0, pc, #MMUAddressTable-(.+8)      @ (r0) = MMUAddressTable phys addr
    mov     r11, r0                             @ (r11) = &MemoryMap (save pointer)


    b   BUILDTTB
MMUAddressTable:


MMUAddressTable:
    /*----------------------------------------------------------------------------------------*/
    /*       virutal                     phisical                    size(MB)       Cachable  */
    /*----------------------------------------------------------------------------------------*/
#if 0
    .word   SAMA5D3X_BASE_IBOOT,      SAMA5D3X_BASE_SRAM,       0x00100000,     0
    .word   SAMA5D3X_BASE_IROM,       SAMA5D3X_BASE_DDR_1,      0x1F000000,     1
#else
    .word   SAMA5D3X_BASE_IBOOT,      SAMA5D3X_BASE_DDR,        0x20000000,     1
#endif
    .word   SAMA5D3X_BASE_DDR ,       SAMA5D3X_BASE_DDR,   	    0x20000000,     0
    .word   SAMA5D3X_BASE_EBI1,       SAMA5D3X_BASE_EBI1,       0x10000000,     0
    .word   SAMA5D3X_BASE_EBI2,       SAMA5D3X_BASE_EBI2,       0x10000000,     0
    .word   SAMA5D3X_BASE_EBI3,       SAMA5D3X_BASE_EBI3,       0x10000000,     0
    .word   SAMA5D3X_BASE_NFC,        SAMA5D3X_BASE_NFC,        0x10000000,     0
    .word   SAMA5D3X_BASE_UNDEF,      SAMA5D3X_BASE_UNDEF,      0x50000000,     0

    .word   SAMA5D3X_VIRT_EBI0,       SAMA5D3X_BASE_EBI0,       0x10000000,     0
    .word   SAMA5D3X_VIRT_IBOOT,      SAMA5D3X_BASE_IBOOT,   	0x00100000,     1
    .word   SAMA5D3X_VIRT_IROM,       SAMA5D3X_BASE_IROM,   	0x00100000,     1
    .word   SAMA5D3X_VIRT_NFC_RAM,    SAMA5D3X_BASE_NFC_RAM,    0x00100000,     0
    .word   SAMA5D3X_VIRT_SRAM,       SAMA5D3X_BASE_SRAM,    	0x00100000,     0
    .word   SAMA5D3X_VIRT_SMD,        SAMA5D3X_BASE_SMD,    	0x00100000,     0
    .word   SAMA5D3X_VIRT_UDPHS_RAM,  SAMA5D3X_BASE_UDPHS_RAM,  0x00100000,     0
    .word   SAMA5D3X_VIRT_UHP_OHCI,   SAMA5D3X_BASE_UHP_OHCI,   0x00100000,     0
    .word   SAMA5D3X_VIRT_UHP_EHCI,   SAMA5D3X_BASE_UHP_EHCI,   0x00100000,     0
    .word   SAMA5D3X_VIRT_AXI_MATRIX, SAMA5D3X_BASE_AXI_MATRIX, 0x00100000,     0
    .word   SAMA5D3X_VIRT_DAP,        SAMA5D3X_BASE_DAP,        0x0F700000,     0

    .word   SAMA5D3X_BASE_REG,        SAMA5D3X_BASE_REG,     	0x10000000,     0
    .word   0x00000000,                  0x00000000,                0         ,     0


BUILDTTB:
    /*  ============================================================================== */
    /*    Fill in first-level Section descriptors to create 1MB mapped regions         */
    /* from the contents of the MemoryMap array. (MMUAddressTable[])                   */
    /*                                                                                 */
    /*    (r9) = TTB                                                                   */
    /*    (r11) = ptr to MemoryMap array                                               */
        

    /*  Set TTB */
    ldr     r9, =SAMA5D3X_BASE_MMU_CFG     @ Physical address of the first-level table
    ldr     r0, =0xFFFFC000                     @
    and     r9, r9, r0                          @ Mask off TTB[31:0] (must be 0's).
    mcr     p15, 0, r9, c2, c0, 0               @ Set the TTB.

@    ldr     r9, =0x30000000
@    ldr     r1, =0x00010000
@    orr     r9, r9, r1                      @ r9 = 0x30010000
@@    mcr     p15, 0, r9, c2, c0, 0           @ set TTB

    /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
    /* ~~~~~~~~~~ MAP CACHEABLE. BUFFERABLE SECTION DESCRIPTORS ~~~~~~~~~~~~~~~~ */
    /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

    mov     r1, r11                @ (r1) = ptr to MemoryMap array

    /* r1 = pointer to MemoryMap array */
    /* r9 = TTB                        */

20:
    mov     r0, #0x0E               @ (r0) = Section (1MB) Descriptor@ C=B=1.  Write-Back, Read-Allocate
    orr     r0, r0, #0x400          @ set AP

    /*                                                  */
    /* start crunching through the MMUAddressTable[]    */
    /*                                                  */
    /* r2 temporarily holds MMUAddressTable[VA]         */
    /* r3 temporarily holds MMUAddressTable[PHY]        */
    /* r4 temporarily holds MMUAddressTable[#MB]        */
    /* r5 cachable                                      */
25:
    ldr     r2, [r1], #4            @ (r2) = virtual, Cached address to map Bank at
    ldr     r3, [r1], #4            @ (r3) = physical address to map from
    ldr     r4, [r1], #4            @ (r4) = num MB to map
    ldr     r5, [r1], #4            @ (r5) = cachable

    mov     r4, r4, lsr #20         @ convert to MB unit

    cmp     r5, #0x0
    biceq   r0, r0, #0x0C           @ clear cachable & bufferable bits in Descriptor (clear C&B bits)

    cmp     r4, #0                  @ End of table?
    beq     29f

    /*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
    /* (r2) holds the &descriptor                                                      */
    /* (r0) holds the actual section descriptor                                        */
        
    /* Create Descriptor Address */
    ldr     r6, =0xFF000000
    orr     r6, r6, #0x00F00000     @ r6 = 0xFFF00000
    and     r2, r2, r6              @ only VA[31:20] are valid
    add     r2, r9, r2, LSR #18     @ build the descriptor address:  r2 = (TTB[31:14} | VA[31:20] >> 18)

    /* Create the Descriptor */
    and     r3, r3, r6              @ only PA[31:20] are valid for the descriptor@ rest will be static
    orr     r0, r3, r0              @ build the descriptor: r0 = (PA[31:20] | the rest of the descriptor)

    /* Store the descriptor at the proper (phy) address */
28:
    str     r0, [r2], #4
    add     r0, r0, #0x00100000     @ (r0) = section descriptor for the next 1MB mapping (just add 1MB)
        
    sub     r4, r4, #1              @ Decrement number of MB left 
    cmp     r4, #0
    bne     28b                     @ Map next MB

    b       20b                     @ Get and process next table element                

29:

#if 0
    /* set up MMU : the 1MB of address 0 */
    ldr     r9, =SAMA5D3X_BASE_MMU_CFG     @ Physical address of the first-level table
    ldr     r0, =0xFFFFC000                     @
    and     r9, r9, r0                          @ Mask off TTB[31:0] (must be 0's).

    mov     r0, #0x01               @ (r0) = Page Table (4KB) Descriptor@ C=B=0.  
    ldr     r2, =SAMA5D3X_BASE_MMU_SMALL_CFG
    orr     r0, r0, r2
    str     r0, [r9], #4

    /* mapping 128K SRAM to address 0*/
    mov     r4, #32
    mov     r0, #0x002
    mov     r1, #0xAA0
    orr     r0, r1, r0
    ldr     r1, =0x00300000
    orr     r0, r0, r1
10:
    str     r0, [r2], #4
    add     r0, r0, #0x00001000     @ (r0) = page descriptor for the next 4KB mapping (just add 4K)
        
    sub     r4, r4, #1              @ Decrement number of 4KB left 
    cmp     r4, #0
    bne     10b                     @ Map next 4KB

    /* mapping (1MB - 128KB) DDRAM to address 128KB*/
    mov     r4, #(256-32)
    mov     r0, #0x002
    mov     r1, #0xAA0
    orr     r0, r1, r0
    ldr     r1, =0x20020000
    orr     r0, r0, r1
10:
    str     r0, [r2], #4
    add     r0, r0, #0x00001000     @ (r0) = page descriptor for the next 4KB mapping (just add 4K)
        
    sub     r4, r4, #1              @ Decrement number of 4KB left 
    cmp     r4, #0
    bne     10b                     @ Map next 4KB
#endif



ACTIVATEMMU:
    /* The 1st Level Section Descriptors are setup. Initialize the MMU and turn it on. */
    mov     r1, #3
    mcr     p15, 0, r1, c3, c0, 0   @ Set up access to domain 0.
    mcr     p15, 0, r0, c8, c7, 0   @ Flush the instruction and data TLBs.
    mcr     p15, 0, r1, c7, c10, 4  @ Drain the write and fill buffers.

/*
    mov     r1, #0x78               @ Bits [6:3] must be written as 1's.
    orr     r1, r1, #0x1            @ Enable MMU.
    orr     r1, r1, #0x1000         @ Enable IC.
    orr     r1, r1, #0x0800         @ Enable BTB.
    orr     r1, r1, #0x4            @ Enable DC.
*/

    mov     r1, #0x78               @ Bits [6:3] must be written as 1's.
    orr     r1, r1, #R1_M           @ Enable MMU.
    @orr     r1, r1, #R1_A           @ Enable A bit. (1<<1)
    orr     r1, r1, #R1_C           @ Enable Enable the Dcache
    orr     r1, r1, #R1_SW          @ Enable SWP, SWPB
    orr     r1, r1, #R1_Z           @ Enable prediction control
    orr     r1, r1, #R1_I           @ Enable the Icache
        @orr     r1, r1, #R1_V           @ Enable the vector bit 
        @orr     r1, r1, #R1_RR          @ Enable the cache replacement strategy
        @orr     r1, r1, #R1_EE          @ Enable the big endian
        @orr     r1, r1, #R1_TRE         @ Enable TEX remap
        @orr     r1, r1, #R1_AFE         @ Enable access flag
        @orr     r1, r1, #R1_TE          @ Enable thumb exception  
    
    ldr     r2, =VirtualStart       @ Get virtual address of 'VirtualStart' label.
    @cmp     r2, #0                  @ Make sure no stall on "mov pc,r2" below.
	@beq		STALL2


    /* Enable the MMU. */
    mcr     p15, 0, r1, c1, c0, 0   @ MMU ON:  All memory accesses are now virtual.
    /* Jump to the virtual address of the 'VirtualStart' label. */
    mov     pc, r2
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
#endif


    /*=========================================================================*/
    /* The MMU and caches are now enabled and we're running in a virtual       */
    /* address space.                                                          */

VirtualStart:
@    /* copy text */
@    mov    r1, #DRAM_BANK1A_PHYSICAL
@    add    r1, r1, #DRAM_SIZE
@    sub    r1, r1, #0x100000
@
@    ldr    sp, =SYMBOL_NAME(_edata)
@    sub    r4, sp, r1
@    mov    r5, r1
@    mov    r2, #0
@1:  ldr    r0, [r2], #4
@    str    r0, [r5], #4
@    subs    r4, r4,    #4
@    bgt    1b


#if 1
    /* initialize bss */
    mov    r0, #0
    ldr    r1, =SYMBOL_NAME(_bss_start)
    ldr    r2, =SYMBOL_NAME(_end)
1:  str    r0, [r1], #4
    cmp    r1, r2
    blt    1b
#else
    ldr    r2, =SYMBOL_NAME(_end)
#endif
    /* set stack location and size */
    add    sp, r2, #STACK_SIZE


    ldr     pc, 1f
1:    .word    switch

    .global start_main
switch:    
    bl      SYMBOL_NAME(start_main)

l:    b    l

