/*    $ DOOROOS: drv_display.c v 0.1 2013/08/011 $    */

/*
 * Copyright (C) 2013    iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: drv_display.c
 *
 * Purpose: display device driver.
 *          
 *
 */

#include <types.h>
//#include <Window.h>
//#include <cache.h>

typedef long LONG;
typedef unsigned long ULONG;

//#include <dev/gd.h>

#include <lib/string.h>
#include "display.h"

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3Xbd.h>
#include <mach/gpio.h>

#include <hal_panic.h>

#include <dev/displaydrv.h>

#define DBG_LVL_LCD     0

//#define __USE_PIO_BL__

#undef  DMA_ADDR_OFFSET
#define DMA_ADDR_OFFSET 0x20000000

/*----------------------------------------------------------------------------
 *        Defines
 *----------------------------------------------------------------------------*/

/* \addtogroup lcdd_disp_id LCD display layers IDs */
/** LCD controller ID, no display, configuration ONLY */
#define LCDD_CONTROLLER     0
/** LCD base layer, display fixed size image */
#define LCDD_BASE           1
/** LCD Overlay 1 */
#define LCDD_OVR1           2
/** LCD Overlay 2 */
#define LCDD_OVR2           4
/** LCD HighEndOverlay, support resize */
#define LCDD_HEO            3
/** LCD Cursor, max size 128x128 */
#define LCDD_CUR            6


/*----------------------------------------------------------------------------
 *        Local types
 *----------------------------------------------------------------------------*/

/** DMA descriptor for LCDC */
typedef struct _LCDCDescriptor
{
    UINT32 addr;
    UINT32 ctrl;
    UINT32 next;
}sLCDCDescriptor;

/** CULT information */
typedef struct _CLUTInfo
{
    UINT8   bpp;
    UINT8   nbColors;
}sCLUTInfo;

/** LCDC General Layer information */
typedef struct _Layer
{
    sLCDCDescriptor dmaD;
    void*           pBuffer;
    sCLUTInfo       clut;
    UINT16          reserved;
} sLayer;

/** LCDC HEO Layer information */
typedef struct _HeoLayer
{
    sLCDCDescriptor dmaD[3];
    void*           pBuffer;
    sCLUTInfo       clut;
    UINT16          reserved;
} sHeoLayer;



/** Current selected canvas information */
//static sLCDDLayer lcddCanvas;
/** Base Layer */
__attribute__((aligned(16)))
 sLayer lcddBase;

/** Overlay 1 Layer */
__attribute__((aligned(64)))
 sLayer lcddOvr1;

__attribute__((aligned(64)))
 sLayer lcddOvr2;

/** High End Overlay Layer */
__attribute__((aligned(64)))
 sHeoLayer lcddHeo;

/** Hardware cursor Layer */
__attribute__((aligned(64)))
 sLayer lcddHcc;



extern void CP15_flush_dcache_for_dma (UINT32 startAddr, UINT32 endAddr );

/*----------------------------------------------------------------------------
 *        Local functions
 *----------------------------------------------------------------------------*/

/**
 * Return a pointer to layer.
 * \param bLayer Layer ID.
 */
static sLayer *pLayer( UINT8 bLayer )
{
    switch( bLayer )
    {
        case LCDD_BASE:       return &lcddBase;
        case LCDD_OVR1:       return &lcddOvr1;
        case LCDD_OVR2:       return &lcddOvr2;
        case LCDD_HEO:        return (sLayer*)&lcddHeo;
        case LCDD_CUR:        return &lcddHcc;
    }
    return NULL;
}

/**
 * Return true if Pixel stride supported.
 * \param bLayer Layer ID.
 */
static UINT8 LCDD_IsPStrideSupported( UINT8 bLayer )
{
    switch( bLayer )
    {
        case LCDD_OVR1: case LCDD_OVR2:case LCDD_HEO: return 1;
        default:                       return 0;
    }
}

/**
 * Return a pointer to enable register.
 * (Starts following register list: _ER, _DR, _SR, _IER, _IDR, _IMR, _ISR)
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pEnableReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_CONTROLLER: return (volatile UINT32*)&pLCDC->LCDC_LCDEN;
        case LCDD_BASE:       return (volatile UINT32*)&pLCDC->LCDC_BASECHER;
        case LCDD_OVR1:       return (volatile UINT32*)&pLCDC->LCDC_OVR1CHER;
        case LCDD_OVR2:       return (volatile UINT32*)&pLCDC->LCDC_OVR2CHER;
        case LCDD_HEO:        return (volatile UINT32*)&pLCDC->LCDC_HEOCHER;
        case LCDD_CUR:        return (volatile UINT32*)&pLCDC->LCDC_HCRCHER;
    }
    return NULL;
}

/**
 * Return a pointer to blender configuration register.
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pBlenderReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE: return (volatile UINT32 *)&pLCDC->LCDC_BASECFG4;
        case LCDD_OVR1: return (volatile UINT32 *)&pLCDC->LCDC_OVR1CFG9;
        case LCDD_OVR2: return (volatile UINT32 *)&pLCDC->LCDC_OVR2CFG9;
        case LCDD_HEO:  return (volatile UINT32 *)&pLCDC->LCDC_HEOCFG12;
        case LCDD_CUR:  return (volatile UINT32 *)&pLCDC->LCDC_HCRCFG9;
    }
    return NULL;
}

/**
 * Return a pointer to DMA head register.
 * (Starts following register list: _HEAD, _ADDRESS, _CONTROL, _NEXT)
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pHeadReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE:       return (volatile UINT32*)&pLCDC->LCDC_BASEHEAD;
        case LCDD_OVR1:       return (volatile UINT32*)&pLCDC->LCDC_OVR1HEAD;
        case LCDD_OVR2:       return (volatile UINT32*)&pLCDC->LCDC_OVR2HEAD;
        case LCDD_HEO:        return (volatile UINT32*)&pLCDC->LCDC_HEOHEAD;
        case LCDD_CUR:        return (volatile UINT32*)&pLCDC->LCDC_HCRHEAD;
    }
    return NULL;
}

/**
 * Return a pointer to layer configure register.
 * (Including: _CFG0, _CFG1 (RGB mode ...))
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pCfgReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE:       return (volatile UINT32*)&pLCDC->LCDC_BASECFG0;
        case LCDD_OVR1:       return (volatile UINT32*)&pLCDC->LCDC_OVR1CFG0;
        case LCDD_OVR2:       return (volatile UINT32*)&pLCDC->LCDC_OVR2CFG0;
        case LCDD_HEO:        return (volatile UINT32*)&pLCDC->LCDC_HEOCFG0;
        case LCDD_CUR:        return (volatile UINT32*)&pLCDC->LCDC_HCRCFG0;
    }
    return NULL;
}

/**
 * Return a pointer to Window configure register.
 * (Including: X Y register, W H register)
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pWinReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_OVR1:       return (volatile UINT32*)&pLCDC->LCDC_OVR1CFG2;
        case LCDD_OVR2:       return (volatile UINT32*)&pLCDC->LCDC_OVR2CFG2;
        case LCDD_HEO:        return (volatile UINT32*)&pLCDC->LCDC_HEOCFG2;
        case LCDD_CUR:        return (volatile UINT32*)&pLCDC->LCDC_HCRCFG2;
    }
    return NULL;
}

/**
 * Return a pointer to striding regiters.
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pStrideReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE: return (volatile UINT32*)&pLCDC->LCDC_BASECFG2;
        case LCDD_OVR1: return (volatile UINT32*)&pLCDC->LCDC_OVR1CFG4;
        case LCDD_OVR2: return (volatile UINT32*)&pLCDC->LCDC_OVR2CFG4;
        case LCDD_HEO:  return (volatile UINT32*)&pLCDC->LCDC_HEOCFG5;
        case LCDD_CUR:  return (volatile UINT32*)&pLCDC->LCDC_HCRCFG4;
    }
    return NULL;
}

/**
 * Return a pointer to Color configure regiters.
 * (Including: RGB Default, RGB Key, RGB Mask)
 * Note that base layer only has one register (default).
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pColorReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_BASE: return (volatile UINT32*)&pLCDC->LCDC_BASECFG3;
        case LCDD_OVR1: return (volatile UINT32*)&pLCDC->LCDC_OVR1CFG6;
        case LCDD_OVR2: return (volatile UINT32*)&pLCDC->LCDC_OVR2CFG6;
        case LCDD_HEO:  return (volatile UINT32*)&pLCDC->LCDC_HEOCFG9;
        case LCDD_CUR:  return (volatile UINT32*)&pLCDC->LCDC_HCRCFG6;
    }
    return NULL;
}

/**
 * Return a pointer to scaling register.
 * \param bLayer Layer ID.
 */
static volatile UINT32 *pScaleReg( UINT8 bLayer )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    switch( bLayer )
    {
        case LCDD_HEO:  return (volatile UINT32*)&pLCDC->LCDC_HEOCFG13;
    }
    return NULL;
}

#if 0
/**
 * Return bits per pixel from RGB mode settings.
 * (Note the bits is bits occupied in memory, including reserved)
 */
static UINT32 LCDD_GetBitsPerPixel(UINT32 modeReg)
{
    switch (modeReg)
    {
        /* RGB mode */
        case LCDC_HEOCFG1_RGBMODE_12BPP_RGB_444:
        case LCDC_HEOCFG1_RGBMODE_16BPP_ARGB_4444:
        case LCDC_HEOCFG1_RGBMODE_16BPP_RGBA_4444:
        case LCDC_HEOCFG1_RGBMODE_16BPP_RGB_565:
        case LCDC_HEOCFG1_RGBMODE_16BPP_TRGB_1555:
            return 2*8;

        case LCDC_HEOCFG1_RGBMODE_18BPP_RGB_666PACKED:
        case LCDC_HEOCFG1_RGBMODE_19BPP_TRGB_PACKED:
        case LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888_PACKED:
            return 3*8;

        case LCDC_HEOCFG1_RGBMODE_18BPP_RGB_666:
        case LCDC_HEOCFG1_RGBMODE_19BPP_TRGB_1666:
        case LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888:
        case LCDC_HEOCFG1_RGBMODE_25BPP_TRGB_1888:
        case LCDC_HEOCFG1_RGBMODE_32BPP_ARGB_8888:
        case LCDC_HEOCFG1_RGBMODE_32BPP_RGBA_8888:
            return 3*8;

        /* CLUT mode */
 
        case LCDC_HEOCFG1_CLUTMODE_CLUT_1BPP | LCDC_HEOCFG1_CLUTEN: return 1;
        case LCDC_HEOCFG1_CLUTMODE_CLUT_2BPP | LCDC_HEOCFG1_CLUTEN: return 2;
        case LCDC_HEOCFG1_CLUTMODE_CLUT_4BPP | LCDC_HEOCFG1_CLUTEN: return 4;
        case LCDC_HEOCFG1_CLUTMODE_CLUT_8BPP | LCDC_HEOCFG1_CLUTEN: return 8;

        /* YUV mode */
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_32BPP_AYCBCR:
            return 32;
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE0:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE1:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE2:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE3:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_SEMIPLANAR:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_PLANAR:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_12BPP_YCBCR_SEMIPLANAR:
        case LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_12BPP_YCBCR_PLANAR:
            return 16;
    }
    return 0;
}
#endif

/**
 * Enable a LCDC DMA channel
 */
static void LCDD_SetDMA( void* pBuffer,
                         sLCDCDescriptor *pTD,
                         UINT32 regAddr)
{
    volatile UINT32 *pDmaR = (volatile UINT32*)regAddr;
    /* Modify descriptor */
    pTD->addr = (UINT32)pBuffer;
    pTD->ctrl = LCDC_BASECTRL_DFETCH;
    pTD->next = (UINT32)pTD + DMA_ADDR_OFFSET;
    /* Modify registers */
    pDmaR[0] = (UINT32)pTD + DMA_ADDR_OFFSET;
    pDmaR[1] = (UINT32)pBuffer;
    pDmaR[2] = LCDC_BASECTRL_DFETCH;
    pDmaR[3] = (UINT32)pTD + DMA_ADDR_OFFSET;

    //CacheRangeFlush(pTD, sizeof(sLCDCDescriptor), CACHE_SYNC_WRITEBACK);
    CP15_flush_dcache_for_dma((UINT32)pTD, ((UINT32)pTD) + sizeof(sLCDCDescriptor));
}

/**
 * Disable a LCDC DMA channel
 */
static void LCDD_ClearDMA( sLCDCDescriptor *pTD, UINT32 regAddr )
{
    UINT32 *pReg = (UINT32 *)regAddr;
    volatile UINT32 *pRegCtrl = (volatile UINT32*)&pReg[1];
    volatile UINT32 *pRegNext = (volatile UINT32*)&pReg[2];

    /* Modify descriptor */
    if (pTD)
    {
        pTD->ctrl &= ~LCDC_BASECTRL_DFETCH;
        pTD->next  =  (UINT32)pTD + DMA_ADDR_OFFSET;

        //CacheRangeFlush(pTD, sizeof(sLCDCDescriptor), CACHE_SYNC_WRITEBACK);
        CP15_flush_dcache_for_dma(pTD, ((UINT32)pTD) + sizeof(sLCDCDescriptor));
    }
    /* Modify control registers */
    *pRegCtrl &= ~LCDC_BASECTRL_DFETCH;
    if( pTD == 0 )
        *pRegNext  =  (UINT32)pTD;
    else
        *pRegNext  =  (UINT32)pTD + DMA_ADDR_OFFSET;
}

/**
 * Return scaling factor
 */
static UINT32 LCDD_CalcScaleFactor(UINT32 targetW, UINT32 imgW)
{
    UINT32 factor;

    factor = 2048 * (imgW + 1) / (targetW + 1);
    
    //factor = 1024 * (imgW + 1) / (targetW + 1);
    //if (targetW > imgW * 2)
    //    factor -= 7;
    return factor;
}



LCDC_DESC    LDCD_Desc;

//#define DEFAULT_BRIGHT_LEVEL     0x10
#define DEFAULT_BRIGHT_LEVEL     0xFF


#define LCDC_HCLK_ENABLE    (1 << 3)
#define LCDC_CORECLK_ENABLE (1 << 12)
#define LCDC_PCKR_ID        4

PUBLIC VOID GD_Contrast( UINT32 nContrast );
//PUBLIC VOID GD_BackLight( UINT32 nBaclight );


/**
 * \brief Turn on the LCD.
 */
void LCDD_On(void)
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;

    /* Enable peripheral clock */
    pPMC->PMC_PCER1 = (unsigned int) 1 << (SAMA5D3_ID_LCDC - 32);
    pPMC->PMC_SCER = (0x1u << 3);

    /* 1. Configure LCD timing parameters, signal polarity and clock period. */
    pLCDC->LCDC_LCDCFG0 = LCDC_LCDCFG0_CLKDIV((BOARD_MCK*2)/BOARD_LCD_PIXELCLOCK-2)
                       |LCDC_LCDCFG0_CGDISHCR
                       |LCDC_LCDCFG0_CGDISHEO
                       |LCDC_LCDCFG0_CGDISOVR1
                       |LCDC_LCDCFG0_CGDISOVR2
                       |LCDC_LCDCFG0_CGDISBASE
                       |LCDC_LCDCFG0_CLKPWMSEL
                       |LCDC_LCDCFG0_CLKSEL
                       |0;//LCDC_LCDCFG0_CLKPOL;

    pLCDC->LCDC_LCDCFG1 = LCDC_LCDCFG1_VSPW(BOARD_LCD_TIMING_VPW-1)
                       |LCDC_LCDCFG1_HSPW(BOARD_LCD_TIMING_HPW-1);

    pLCDC->LCDC_LCDCFG2 = LCDC_LCDCFG2_VBPW(BOARD_LCD_TIMING_VBP)
                       |LCDC_LCDCFG2_VFPW(BOARD_LCD_TIMING_VFP-1);

    pLCDC->LCDC_LCDCFG3 = LCDC_LCDCFG3_HBPW(BOARD_LCD_TIMING_HBP-1)
                       |LCDC_LCDCFG3_HFPW(BOARD_LCD_TIMING_HFP-1);

    pLCDC->LCDC_LCDCFG4 = LCDC_LCDCFG4_RPF(DISPLAY_HEIGHT-1)
                       |LCDC_LCDCFG4_PPL(DISPLAY_WIDTH-1);

    pLCDC->LCDC_LCDCFG5 = LCDC_LCDCFG5_GUARDTIME(30)
                       |LCDC_LCDCFG5_MODE_OUTPUT_24BPP
                       |LCDC_LCDCFG5_DISPDLY
                       |LCDC_LCDCFG5_VSPDLYS
                       |LCDC_LCDCFG5_VSPOL
                       |LCDC_LCDCFG5_HSPOL;

    pLCDC->LCDC_LCDCFG6 = LCDC_LCDCFG6_PWMCVAL(DEFAULT_BRIGHT_LEVEL)//0)
                       |LCDC_LCDCFG6_PWMPOL
                       |LCDC_LCDCFG6_PWMPS(6);

    /* 2. Enable the Pixel Clock by writing one to the CLKEN field of the
          LCDC_LCDEN register. */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_CLKEN;
    /* 3. Poll CLKSTS field of the LCDC_LCDSR register to check that the clock
       is running. */
    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_CLKSTS));

    /* 4. Enable Horizontal and Vertical Synchronization by writing one to the
          SYNCEN field of the LCDC_LCDEN register. */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_SYNCEN;
    /* 5. Poll LCDSTS field of the LCDC_LCDSR register to check that the
          synchronization is up. */
    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_LCDSTS));

    /* 6. Enable the display power signal writing one to the DISPEN field of the
          LCDC_LCDEN register. */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_DISPEN;
    /* 7. Poll DISPSTS field of the LCDC_LCDSR register to check that the power
          signal is activated. */


#if 0
    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_DISPSTS));
    /* 8. Enable backlight */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_PWMEN;

    pio_set_value(AT91C_PIN_PA(24), 1 ); 
#ifdef __USE_PIO_BL__
    pio_set_value(AT91C_PIN_PA(25), 1 ); 
#endif

#endif

}

void LCDD_PowerOn( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

//    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_DISPSTS));
    /* 8. Enable backlight */
    pLCDC->LCDC_LCDEN = LCDC_LCDEN_PWMEN;

    pio_set_value(AT91C_PIN_PA(24), 1 ); 
#ifdef __USE_PIO_BL__
    pio_set_value(AT91C_PIN_PA(25), 1 ); 
#endif
}


/**
 * \brief Turn off the LCD.
 */
void LCDD_Off(void)
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;


    /* 1. Clear the DFETCH bit in the DSCR.CHXCTRL field of the DSCR structure
          will disable the channel at the end of the frame. */

    /* 2. Set the DSCR.CHXNEXT field of the DSCR structure will disable the
          channel at the end of the frame. */

    /* Disable all DMA channel descriptors */
    LCDD_ClearDMA(&lcddBase.dmaD, (UINT32)&pLCDC->LCDC_BASEADDR);
    LCDD_ClearDMA(&lcddOvr1.dmaD, (UINT32)&pLCDC->LCDC_OVR1ADDR);
    LCDD_ClearDMA(&lcddOvr2.dmaD, (UINT32)&pLCDC->LCDC_OVR2ADDR);
    LCDD_ClearDMA(&lcddHeo.dmaD[0], (UINT32)&pLCDC->LCDC_HEOADDR);
    LCDD_ClearDMA(&lcddHeo.dmaD[1], (UINT32)&pLCDC->LCDC_HEOUADDR);
    LCDD_ClearDMA(&lcddHeo.dmaD[2], (UINT32)&pLCDC->LCDC_HEOVADDR);
    LCDD_ClearDMA(&lcddHcc.dmaD, (UINT32)&pLCDC->LCDC_HCRADDR);

    /* 3. Writing one to the CHDIS field of the CHXCHDR register will disable
          the channel at the end of the frame. */

    /* Disable DMA channels */
    pLCDC->LCDC_BASECHDR = LCDC_BASECHDR_CHDIS;
    pLCDC->LCDC_OVR1CHDR = LCDC_OVR1CHDR_CHDIS;
    pLCDC->LCDC_HEOCHDR  = LCDC_HEOCHDR_CHDIS;
    pLCDC->LCDC_HCRCHDR  = LCDC_HCRCHDR_CHDIS;
    pLCDC->LCDC_BASECFG4 = 0;

    /* 4. Writing one to the CHRST field of the CHXCHDR register will disable
          the channel immediately. This may occur in the middle of the image. */

    /* 5. Poll CHSR field in the CHXCHSR register until the channel is
          successfully disabled. */
    while (pLCDC->LCDC_BASECHSR & LCDC_BASECHSR_CHSR);
    while (pLCDC->LCDC_OVR1CHSR & LCDC_OVR1CHSR_CHSR);
    while (pLCDC->LCDC_HEOCHSR  & LCDC_HEOCHSR_CHSR);
    while (pLCDC->LCDC_HCRCHDR  & LCDC_HCRCHSR_CHSR);

   
    /* Timing Engine Power Down Software Operation */

    /* Disable backlight */
    pLCDC->LCDC_LCDDIS = LCDC_LCDDIS_PWMDIS;
    while (pLCDC->LCDC_LCDSR & LCDC_LCDSR_PWMSTS);

    /* 1. Disable the DISP signal writing DISPDIS field of the LCDC_LCDDIS
          register. */
    pLCDC->LCDC_LCDDIS = LCDC_LCDDIS_DISPDIS;
    /* 2. Poll DISPSTS field of the LCDC_LCDSR register to verify that the DISP
          is no longer activated. */
    while (pLCDC->LCDC_LCDSR & LCDC_LCDSR_DISPSTS);
    
    /* 3. Disable the hsync and vsync signals by writing one to SYNCDIS field of
          the LCDC_LCDDIS register. */
    pLCDC->LCDC_LCDDIS = LCDC_LCDDIS_SYNCDIS;
    /* 4. Poll LCDSTS field of the LCDC_LCDSR register to check that the
          synchronization is off. */
    while (pLCDC->LCDC_LCDSR & LCDC_LCDSR_LCDSTS);
    
    /* 5. Disable the Pixel clock by writing one in the CLKDIS field of the
          LCDC_LCDDIS register. */
    pLCDC->LCDC_LCDDIS = LCDC_LCDDIS_CLKDIS;
    /* 6. Poll CLKSTS field of the LCDC_LCDSR register to check that Pixel Clock
          is disabled. */
    while(pLCDC->LCDC_LCDSR & LCDC_LCDSR_CLKSTS);

    /* Disable peripheral clock */
    pPMC->PMC_PCDR1 = (unsigned int) 1 << (SAMA5D3_ID_LCDC - 32);
    /* LCD Clock Disable */
     pPMC->PMC_SCDR = (0x1u << 3);

    pio_set_value(AT91C_PIN_PA(24), 0); 
#ifdef __USE_PIO_BL__
    pio_set_value(AT91C_PIN_PA(25), 0); 
#endif     

}


// Internal function, get pointers to registers & set PIO/Peripheral configuration
STATIC VOID
IOConfig( VOID )
{
//    #define LCD_PIO_SET     (PIO_OPENDRAIN|PIO_PULLUP|PIO_DEFAULT)
//    #define LCD_PIO_SET     (PIO_OPENDRAIN|PIO_DEFAULT)
//    #define LCD_PIO_SET     (PIO_PULLUP|PIO_DEFAULT)
//    #define LCD_PIO_SET     (PIO_DEFAULT)
//    #define LCD_PIO_UP      (PIO_PULLUP)
//    #define LCD_PIO_UP      (PIO_DEFAULT)

    #define LCD_PIO_SET     (PIO_PULLDN)
    #define LCD_PIO_UP      (PIO_PULLDN)
    const struct pio_desc hw_pio[] = {

        {"B0",        AT91C_PIN_PA(0),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B1",        AT91C_PIN_PA(1),    0, LCD_PIO_SET, PIO_PERIPH_A},
//        {"B0",        AT91C_PIN_PA(0),    0, LCD_PIO_SET, PIO_OUTPUT_0},
//        {"B1",        AT91C_PIN_PA(1),    0, LCD_PIO_SET, PIO_OUTPUT_0},
        {"B2",        AT91C_PIN_PA(2),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B3",        AT91C_PIN_PA(3),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B4",        AT91C_PIN_PA(4),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B5",        AT91C_PIN_PA(5),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B6",        AT91C_PIN_PA(6),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"B7",        AT91C_PIN_PA(7),    0, LCD_PIO_SET, PIO_PERIPH_A},

        {"G0",        AT91C_PIN_PA(8),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G1",        AT91C_PIN_PA(9),    0, LCD_PIO_SET, PIO_PERIPH_A},
//        {"G0",        AT91C_PIN_PA(8),     0, LCD_PIO_SET, PIO_OUTPUT_0},
//        {"G1",        AT91C_PIN_PA(9),     0, LCD_PIO_SET, PIO_OUTPUT_0},
        {"G2",        AT91C_PIN_PA(10),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G3",        AT91C_PIN_PA(11),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G4",        AT91C_PIN_PA(12),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G5",        AT91C_PIN_PA(13),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G6",        AT91C_PIN_PA(14),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"G7",        AT91C_PIN_PA(15),    0, LCD_PIO_SET, PIO_PERIPH_A},

#ifdef _SH_BD_
        {"R0",        AT91C_PIN_PA(16),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R1",        AT91C_PIN_PA(17),    0, LCD_PIO_SET, PIO_PERIPH_A},
//        {"R0",        AT91C_PIN_PA(16),    0, LCD_PIO_SET, PIO_OUTPUT_0},
//        {"R1",        AT91C_PIN_PA(17),    0, LCD_PIO_SET, PIO_OUTPUT_0},
        {"R2",        AT91C_PIN_PA(18),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R3",        AT91C_PIN_PA(19),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R4",        AT91C_PIN_PA(20),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R5",        AT91C_PIN_PA(21),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R6",        AT91C_PIN_PA(22),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"R7",        AT91C_PIN_PA(23),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"MCLK",	  AT91C_PIN_PC(15),    0, PIO_DEFAULT, PIO_PERIPH_B}, // for 12M output for USB HUB
#else
        {"R0",        AT91C_PIN_PC(14),    0, LCD_PIO_SET, PIO_PERIPH_C},
        {"R1",        AT91C_PIN_PC(13),    0, LCD_PIO_SET, PIO_PERIPH_C},
//        {"R0",        AT91C_PIN_PC(14),    0, LCD_PIO_SET, PIO_OUTPUT_0},
//        {"R1",        AT91C_PIN_PC(13),    0, LCD_PIO_SET, PIO_OUTPUT_0},
        {"R2",        AT91C_PIN_PC(12),    0, LCD_PIO_SET, PIO_PERIPH_C},
        {"R3",        AT91C_PIN_PC(11),    0, LCD_PIO_SET, PIO_PERIPH_C},
        {"R4",        AT91C_PIN_PC(10),    0, LCD_PIO_SET, PIO_PERIPH_C},
        {"R5",        AT91C_PIN_PC(15),    0, LCD_PIO_SET, PIO_PERIPH_C},
        {"R6",        AT91C_PIN_PE(27),    0, LCD_PIO_SET, PIO_PERIPH_C},
        {"R7",        AT91C_PIN_PE(28),    0, LCD_PIO_SET, PIO_PERIPH_C},
#endif

    #ifdef __USE_PIO_BL__
        {"LCDPWM",    AT91C_PIN_PA(24),    0, LCD_PIO_UP, PIO_OUTPUT_0},
    #else
        {"LCDPWM",    AT91C_PIN_PA(24),    0, LCD_PIO_SET, PIO_PERIPH_A},
    #endif
//#ifdef __7INCH_DISPLAY__ 
#if 0
//        {"VSYNC",    AT91C_PIN_PA(26),    0, LCD_PIO_UP, PIO_INPUT},
//        {"HSYNC",    AT91C_PIN_PA(27),    0, LCD_PIO_UP, PIO_INPUT},
        {"VSYNC",    AT91C_PIN_PA(26),    0, LCD_PIO_UP, PIO_OUTPUT_1},
        {"HSYNC",    AT91C_PIN_PA(27),    0, LCD_PIO_UP, PIO_OUTPUT_1},
        {"DEN",        AT91C_PIN_PA(29),    0, LCD_PIO_SET, PIO_PERIPH_A},
#else
        {"VSYNC",    AT91C_PIN_PA(26),    0, LCD_PIO_SET, PIO_PERIPH_A},
        {"HSYNC",    AT91C_PIN_PA(27),    0, LCD_PIO_SET, PIO_PERIPH_A},
//        {"DEN",        AT91C_PIN_PA(29),    0, PIO_DEFAULT, PIO_OUTPUT_0},
        {"DEN",       AT91C_PIN_PA(29),    0, LCD_PIO_SET, PIO_PERIPH_A},

#endif

    #ifdef __USE_PIO_BL__
        {"LCDDISP",    AT91C_PIN_PA(25),    0, LCD_PIO_UP, PIO_OUTPUT_0},
    #else
        {"LCDDISP",    AT91C_PIN_PA(25),    0, LCD_PIO_SET, PIO_PERIPH_A},
    #endif

        {"LCDPCK",    AT91C_PIN_PA(28),    0, LCD_PIO_SET, PIO_PERIPH_A},

        /* peripheral  */
//        {"SPI1_NPCS",	AT91C_PIN_PC(28),	0, LCD_PIO_UP, PIO_OUTPUT_0},
//		{"SPI1_MISO",	AT91C_PIN_PC(22),	0, LCD_PIO_UP, PIO_OUTPUT_0},
//		{"SPI1_MOSI",	AT91C_PIN_PC(23),	0, LCD_PIO_UP, PIO_OUTPUT_0},
//		{"SPI1_SPCK",	AT91C_PIN_PC(24),	0, LCD_PIO_UP, PIO_OUTPUT_0},

//		{"TOUCH_PD24",	AT91C_PIN_PD(24),	0, LCD_PIO_UP, PIO_OUTPUT_0},
		{(char *)0, 0, 0, PIO_DEFAULT, PIO_PERIPH_A},
    };

    pio_setup(hw_pio, sizeof(hw_pio)/sizeof(struct pio_desc));
    //pio_configure(hw_pio);

#ifdef _SH_BD_
{
	AT91PS_PMC pPMC = (AT91PS_PMC)AT91C_BASE_PMC;

	AT91C_BASE_PMC->PMC_PCER0 = 1 << SAMA5D3_ID_PIOC;

    pPMC->PMC_SCDR = PMC_SCER_PCK2;
    while((pPMC->PMC_SCSR)&PMC_SCSR_PCK2)
        /*Sleep(10)*/;

	pPMC->PMC_PCK[2] = PMC_MCKR_CSS_MAIN_CLK;
	pPMC->PMC_SCER = PMC_SCER_PCK2;
	while ((pPMC->PMC_SR & PMC_SR_PCKRDY2) == 0)
		/*Sleep(10)*/;
}
#endif

    return;
}


/*----------------------------------------------------------------------------
 *        Exported functions
 *----------------------------------------------------------------------------*/
/**
 * \brief Initializes the LCD controller.
 * Configure SMC to access LCD controller at 64MHz MCK.
 */
static void LCDD_Initialize( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;


    /* Configure PIO */
    IOConfig();

    LCDD_Off();
    
    /* Reset CLUT information */
    lcddBase.clut.bpp = 0;
    lcddOvr1.clut.bpp = 0;
    lcddOvr2.clut.bpp = 0;
    lcddHeo.clut.bpp  = 0;
    lcddHcc.clut.bpp  = 0;

    /* Reset layer information */
    lcddBase.pBuffer = NULL;
    lcddOvr1.pBuffer = NULL;
    lcddOvr2.pBuffer = NULL;
    lcddHeo.pBuffer  = NULL;
    lcddHcc.pBuffer  = NULL;

    ///* No canvas selected */
    //lcddCanvas.pBuffer = NULL;

    /* Enable peripheral clock */
    pPMC->PMC_PCER1 = (unsigned int) 1 << (SAMA5D3_ID_LCDC - 32);
    pPMC->PMC_SCER = (0x1u << 3);

    /* Timing Engine Configuration */
   
    /* Disable interrupt */
    pLCDC->LCDC_LCDIDR = 0xFFFFFFFF;

    /* Configure channels */

    /* Base */
    pLCDC->LCDC_BASECFG0 =  LCDC_BASECFG0_DLBO | LCDC_BASECFG0_BLEN_AHB_INCR16;
    pLCDC->LCDC_BASECFG1 =  //LCDC_BASECFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_OVR1CFG1_RGBMODE_32BPP_ARGB_8888;
    //pLCDC->LCDC_BASECFG1 =  LCDC_BASECFG1_RGBMODE_32BPP_ARGB_8888;

    /* Overlay 1, GA 0xFF, CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_OVR1CFG0 =  LCDC_OVR1CFG0_DLBO | LCDC_OVR1CFG0_BLEN_AHB_BLEN_INCR16
                       |  LCDC_OVR1CFG0_ROTDIS
                       ;
    pLCDC->LCDC_OVR1CFG1 =  //LCDC_OVR1CFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_OVR1CFG1_RGBMODE_32BPP_ARGB_8888;
    pLCDC->LCDC_OVR1CFG9 =  LCDC_OVR1CFG9_GA(0x8F) | LCDC_OVR1CFG9_GAEN;
    pLCDC->LCDC_OVR1CFG6 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_OVR1CFG7 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_OVR1CFG8 = 0xFFFFFF;
    pLCDC->LCDC_OVR1CFG9 |= LCDC_OVR1CFG9_CRKEY;

    /* Overlay 2, GA 0xFF, CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_OVR2CFG0 =  LCDC_OVR2CFG0_DLBO | LCDC_OVR2CFG0_BLEN_AHB_INCR16
                       |  LCDC_OVR2CFG0_ROTDIS
                       ;
    pLCDC->LCDC_OVR2CFG1 =  //LCDC_OVR2CFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_OVR2CFG1_RGBMODE_32BPP_ARGB_8888;
    pLCDC->LCDC_OVR2CFG9 =  LCDC_OVR2CFG9_GA(0xFF) | LCDC_OVR2CFG9_GAEN;
    pLCDC->LCDC_OVR2CFG6 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_OVR2CFG7 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_OVR2CFG8 = 0xFFFFFF;
    pLCDC->LCDC_OVR2CFG9 |= LCDC_OVR2CFG9_CRKEY;

#if 1
    /* High End Overlay, GA 0xFF, CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_HEOCFG0  =  LCDC_HEOCFG0_DLBO | LCDC_HEOCFG0_BLEN_AHB_BLEN_INCR16
                            |  LCDC_HEOCFG0_ROTDIS
                            ;
    pLCDC->LCDC_HEOCFG1  =  LCDC_HEOCFG1_YUVEN |
                            LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE1 /*|LCDC_HEOCFG1_YUV422SWP*/
                            ;
    pLCDC->LCDC_HEOCFG12 =  LCDC_HEOCFG12_GA(0xFF) | LCDC_HEOCFG12_GAEN;
    pLCDC->LCDC_HEOCFG9  = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HEOCFG10 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HEOCFG11 = 0xFFFFFF;
    pLCDC->LCDC_HEOCFG12 |= LCDC_HEOCFG12_CRKEY;

	/* XXX: evaluate pix->colorspace */
    pLCDC->LCDC_HEOCFG14 = 0x4c900091;
    pLCDC->LCDC_HEOCFG15 = 0x7a5f5090;
    pLCDC->LCDC_HEOCFG16 = 0x40040890;

#else    
    /* High End Overlay, GA 0xFF, CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_HEOCFG0  =  LCDC_HEOCFG0_DLBO | LCDC_HEOCFG0_BLEN_AHB_BLEN_INCR16
                            |  LCDC_HEOCFG0_ROTDIS
                            ;
    pLCDC->LCDC_HEOCFG1  =  //LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_HEOCFG1_RGBMODE_32BPP_ARGB_8888;
    pLCDC->LCDC_HEOCFG12 =  LCDC_HEOCFG12_GA(0xFF) | LCDC_HEOCFG12_GAEN;
    pLCDC->LCDC_HEOCFG9  = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HEOCFG10 = DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HEOCFG11 = 0xFFFFFF;
    pLCDC->LCDC_HEOCFG12 |= LCDC_HEOCFG12_CRKEY;
#endif
    /* Hardware Cursor, GA 0xFF, , CHROMA/MASK 0xFF00FF/0xFFFFFF */
    pLCDC->LCDC_HCRCFG0  =  LCDC_HCRCFG0_DLBO | LCDC_HCRCFG0_BLEN_AHB_BLEN_INCR16;
    pLCDC->LCDC_HCRCFG1  =  //LCDC_HCRCFG1_RGBMODE_24BPP_RGB_888_PACKED;
                            LCDC_HCRCFG1_RGBMODE_32BPP_ARGB_8888;
    pLCDC->LCDC_HCRCFG6  =  DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HCRCFG7  =  DEFAULT_TRANSPARENT_COLOR;
    pLCDC->LCDC_HCRCFG8  =  0xFFFFFF;
    pLCDC->LCDC_HCRCFG9  =  LCDC_HCRCFG9_GA(0xFF) | LCDC_HCRCFG9_GAEN | LCDC_HCRCFG9_CRKEY;

    //LCDD_On();
}


/**
 * Check if specified layer is working.
 * \param bLayer Layer ID.
 * \return 1 if layer is on.
 */
UINT8 LCDD_IsLayerOn( UINT8 bLayer )
{
    volatile UINT32 *pReg = pEnableReg(bLayer);
    if (pReg) return ((pReg[2] & LCDC_BASECHSR_CHSR) > 0);
    return 0;
}

/**
 * Enable(turn on)/Disable(hide) specified layer.
 * \param bLayer Layer ID.
 * \param bEnDis Enable/Disable.
 */
void LCDD_EnableLayer( UINT8 bLayer, UINT8 bEnDis )
{
    volatile UINT32 *pReg = pEnableReg(bLayer);
    volatile UINT32 *pBlR = pBlenderReg(bLayer);
    if (pReg && bLayer > LCDD_CONTROLLER)
    {
        if (bEnDis)
        {
            pReg[0]  = LCDC_BASECHER_CHEN | LCDC_BASECHER_UPDATEEN;
            pBlR[0] |= LCDC_HEOCFG12_DMA | LCDC_HEOCFG12_OVR;
        }
        else
        {
            pReg[1]  = LCDC_BASECHDR_CHDIS;
            pBlR[0] &= ~(LCDC_HEOCFG12_DMA | LCDC_HEOCFG12_OVR);
        }
    }
}


/**
 * Display an image on specified layer.
 * (Image scan origion: Left -> Right, Top -> Bottom.)
 * \note w & h should be the rotated result.
 * \note for LCDD_BASE: x, y don't care. w always > 0.
 * \note for LCDD_HEO:imgW & imgH is used.
 * \param bLayer  Layer ID (OVR1, HEO or CUR).
 * \param pBuffer Pointer to image data.
 * \param bPP     Bits Per Pixel.
 *                - 16: TRGB 1555
 *                - 24:  RGB  888  packed
 *                - 32: ARGB 8888
 * \param x       X position.
 * \param y       Y position.
 * \param w       Width  (<0 means Right  -> Left data).
 * \param h       Height (<0 means Bottom -> Top data).
 * \param imgW    Source image width.
 * \param imgH    Source image height.
 * \param wRotate Rotation (clockwise, 0, 90, 180, 270 accepted).
 */
void *LCDD_ShowBMPRotated( UINT8 bLayer,
                           void* pBuffer, UINT8 bpp,
                           UINT32 x, UINT32 y,
                           INT32  w, INT32  h,
                           UINT32 imgW, UINT32 imgH,
                           INT16 wRotate)
{
//AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    sLayer            *pLD   = pLayer(bLayer);
    //sCLUTInfo         *pClut = &pLD->clut;
    sLCDCDescriptor   *pTD   = &pLD->dmaD;
    volatile UINT32 *pEnR  = pEnableReg(bLayer);
    volatile UINT32 *pDmaR = pHeadReg(bLayer);
    volatile UINT32 *pWinR = pWinReg(bLayer);
    volatile UINT32 *pStrR = pStrideReg(bLayer);
    volatile UINT32 *pSclR = pScaleReg(bLayer);
    volatile UINT32 *pBlR  = pBlenderReg(bLayer);
    volatile UINT32 *pCfgR = pCfgReg(bLayer);
    UINT8 bPStride = LCDD_IsPStrideSupported(bLayer);

    UINT8 bBottomUp  = (h < 0);
    UINT8 bRightLeft = (w < 0);
    UINT32 padding = 0;
    INT32 srcW, srcH;
    UINT32 bitsPRow, bytesPRow;
    UINT32 bytesPPix = bpp >> 3;

    void* pOldBuffer = pLD->pBuffer;

    if (pCfgR == NULL)  return pOldBuffer;

//kprintf("Show %x @ %d: (%d,%d)+(%d,%d) img %d x %d * %d\n\r", pBuffer, bLayer, x, y, w, h, imgW, imgH, bpp);

    if( bLayer == LCDD_HEO)
    {
        pCfgR[1] = LCDC_HEOCFG1_YUVEN | LCDC_HEOCFG1_YUVMODE_16BPP_YCBCR_MODE1 /*| LCDC_HEOCFG1_YUV422SWP*/;
    }
    else
    {
        switch (bpp)
        {
            case 16: /*  TRGB 1555 */
                pCfgR[1] = LCDC_HEOCFG1_RGBMODE_16BPP_TRGB_1555;
                break;
            case 24: /*  RGB  888 packed */
                pCfgR[1] = LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888_PACKED;
                break;
            case 32: /* ARGB 8888 */
                pCfgR[1] = LCDC_HEOCFG1_RGBMODE_32BPP_ARGB_8888;
                break;
            default: return pOldBuffer;
        }
    }

    /* Windows position & size check */
    if (h < 0)          h =- h;
    if (w < 0)          w =- w;
    if (x + w > DISPLAY_WIDTH)
    {
        //printf("! w %d -> %d\n\r", w, DISPLAY_WIDTH-x);
        w = DISPLAY_WIDTH - x;
    }
    if (y + h > DISPLAY_HEIGHT)
    {
        //printf("! h %d -> %d\n\r", h, DISPLAY_HEIGHT-y);
        h = DISPLAY_HEIGHT - y;
    }
    if (w == 0) w ++;
    if (h == 0) h ++;
    if (imgW == 0) imgW ++;
    if (imgH == 0) imgH ++;

    /* Only 0,(-)90,(-)180,(-)270 accepted */
    switch(wRotate)
    {
        case 0: case 90: case 180: case 270:
            break;
        case -90: case -180: case -270:
            wRotate += 360;
            break;
        default: return NULL;
    }

    /* Setup display buffer & window */
    if (pBuffer) pLD->pBuffer = pBuffer;
    else         pBuffer = pLD->pBuffer;

    /* Set display buffer & mode */
    bitsPRow  = imgW * bpp;
    bytesPRow = bitsPRow >> 3;
    if (bitsPRow & 0x7) bytesPRow ++;
    if (bytesPRow & 0x3) padding = 4 - (bytesPRow & 0x3);

    /* No X mirror supported layer, no Right->Left scan */
    if (!bPStride)  bRightLeft = 0;

    /* --------- Mirror & then rotate --------- */
    /* Normal direction: Left,Top -> Right,Down */
    if (   (!bRightLeft && !bBottomUp && wRotate ==   0)
        || ( bRightLeft &&  bBottomUp && wRotate == 180) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* X0 ++ */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0);
        /* Y0 ++ */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(padding);
        /* Pointer to Left,Top (x0,y0) */
    }
    /* X mirror: Right,Top -> Left,Down */
    else if ( ( bRightLeft && !bBottomUp && wRotate ==   0)
            ||(!bRightLeft &&  bBottomUp && wRotate == 180) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* X1 -- */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0-2*bytesPPix);
        /* Y0 ++ */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(bytesPRow*2+padding-2*bytesPPix);
        /* Pointer to Right,Top (x1,y0) */
        pBuffer = (void*)((UINT32)pBuffer
                + bytesPPix*(imgW-1));
    }
    /* Y mirror: Left,Down -> Right,Top */
    else if ( (!bRightLeft &&  bBottomUp && wRotate ==   0)
            ||( bRightLeft && !bBottomUp && wRotate == 180) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* X0 ++ */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0);
        /* Y1 -- */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-(bytesPRow*2+padding));
        /* Pointer to Left,Down (x0,y1) */
        pBuffer = (void*)((UINT32)pBuffer
                + (bytesPRow+padding)*(imgH-1));
    }
    /* X,Y mirror: Right,Top -> Left,Down */
    else if ( ( bRightLeft &&  bBottomUp && wRotate ==   0)
            ||(!bRightLeft && !bBottomUp && wRotate == 180) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* X1 -- */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0-2*bytesPPix);
        /* Y1 -- */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-(bytesPPix*2+padding));
        /* Pointer to Left,Down (x1,y1) */
        pBuffer = (void*)((UINT32)pBuffer
                + (bytesPRow+padding)*(imgH-1)
                + (bytesPPix)*(imgW-1));
    }
    /* Rotate  90: Down,Left -> Top,Right (with w,h swap) */
    else if ( (!bRightLeft && !bBottomUp && wRotate ==  90)
            ||( bRightLeft &&  bBottomUp && wRotate == 270) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* Y -- as pixels in row */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0-(bytesPPix+bytesPRow+padding));
        /* X ++ as rows */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE((bytesPRow+padding)*(imgH-1));
        /* Pointer to Bottom,Left */
        pBuffer = (void*)((UINT32)pBuffer
                + (bytesPRow+padding)*(imgH-1));
    }
    /* Rotate 270: Top,Right -> Down,Left (with w,h swap) */
    else if ( (!bRightLeft && !bBottomUp && wRotate == 270)
            ||( bRightLeft &&  bBottomUp && wRotate ==  90) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* Y ++ as pixels in row */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(bytesPRow+padding-bytesPPix);
        /* X -- as rows */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-2*bytesPPix-(bytesPRow+padding)*(imgH-1));
        /* Pointer to top right */
        pBuffer = (void*)((UINT32)pBuffer
                + bytesPPix*(imgW-1));
    }
    /* Mirror X then Rotate 90: Down,Right -> Top,Left */
    else if ( ( bRightLeft && !bBottomUp && wRotate ==  90)
            ||(!bRightLeft &&  bBottomUp && wRotate == 270) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* Y -- as pixels in row */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(0-(bytesPPix+bytesPRow+padding));
        /* X -- as rows */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-2*bytesPPix+(bytesPRow+padding)*(imgH-1));
        /* Pointer to down right (x1,y1) */
        pBuffer = (void*)((UINT32)pBuffer
                + (bytesPRow+padding)*(imgH-1)
                + (bytesPPix)*(imgW-1));
    }
    /* Mirror Y then Rotate 90: Top,Left -> Down,Right */
    else if ( (!bRightLeft &&  bBottomUp && wRotate ==  90)
            ||( bRightLeft && !bBottomUp && wRotate == 270) )
    {
        /* No rotation optimization */
        pCfgR[0] |= LCDC_HEOCFG0_ROTDIS;
        /* Y ++ as pixels in row */
        if (bPStride)   pStrR[1] = LCDC_HEOCFG6_PSTRIDE(bytesPRow+padding-bytesPPix);
        /* X ++ as rows */
        pStrR[0] = LCDC_HEOCFG5_XSTRIDE(0-(bytesPRow+padding)*(imgH-1));
        /* Pointer to top left (x0,y0) */
    }
    /** DMA is running, just add new descriptor to queue */
    if (pBlR[0] & LCDC_HEOCFG12_DMA)
    {
        pTD->addr = (UINT32)pBuffer;
        pTD->ctrl = LCDC_HEOCTRL_DFETCH;
        pTD->next = (UINT32)pTD + DMA_ADDR_OFFSET;
        pDmaR[0] = (UINT32)pTD + DMA_ADDR_OFFSET;
        pEnR[0] = LCDC_HEOCHER_A2QEN;
    }
    else
    {
        /* 2. Write the channel descriptor (DSCR) structure in the system memory by
              writing DSCR.CHXADDR Frame base address, DSCR.CHXCTRL channel control
              and DSCR.CHXNEXT next descriptor location.        
           3. If more than one descriptor is expected, the DFETCH field of
              DSCR.CHXCTRL is set to one to enable the descriptor fetch operation.      
           4. Write the DSCR.CHXNEXT register with the address location of the
              descriptor structure and set DFETCH field of the DSCR.CHXCTRL register
              to one. */
        LCDD_SetDMA(pBuffer, pTD, (UINT32)pDmaR);
    }
    CP15_flush_dcache_for_dma ((UINT32)pTD, ((UINT32)pTD) + sizeof(sLCDCDescriptor));
    //CacheRangeFlush(pTD, sizeof(sLCDCDescriptor), CACHE_SYNC_WRITEBACK);

    /* Set window & position */
    if (pWinR)
    {
        pWinR[0] = LCDC_HEOCFG2_XPOS(x) | LCDC_HEOCFG2_YPOS(y);
        pWinR[1] = LCDC_HEOCFG3_XSIZE(w-1) | LCDC_HEOCFG3_YSIZE(h-1);
    }
    /* Scaling setup */
    if (pSclR)
    {

        /* Image size only used in scaling */
        /* Scaling target */
        if (wRotate == 90 || wRotate == 270)
        {
            srcW = imgH; srcH = imgW;
        }
        else
        {
            srcW = imgW; srcH = imgH;
        }
        pWinR[2] = LCDC_HEOCFG4_XMEM_SIZE(srcW-1) | LCDC_HEOCFG4_YMEM_SIZE(srcH-1);
        /* Scaled */
        if (w != srcW || h != srcH)
        {
            UINT16 wYf, wXf;
            wXf = LCDD_CalcScaleFactor(w, srcW);
            wYf = LCDD_CalcScaleFactor(h, srcH);
            //printf("- Scale(%d,%d)\n\r", wXf, wYf);
            pSclR[0] = LCDC_HEOCFG13_YFACTOR(wYf)
                     | LCDC_HEOCFG13_XFACTOR(wXf)
                     | LCDC_HEOCFG13_SCALEN
                     ;
        }
        /* Disable scaling */
        else
        {
            pSclR[0] = 0;
        }
    }
    /* Enable DMA */
    if (pBuffer)
    {
        pBlR[0] |= LCDC_HEOCFG12_DMA
                 | LCDC_HEOCFG12_OVR
                 ;
    }
    /* Enable & Update */
    /* 5. Enable the relevant channel by writing one to the CHEN field of the
          CHXCHER register. */
    pEnR[0] = LCDC_HEOCHER_UPDATEEN | LCDC_HEOCHER_CHEN;

    /* 6. An interrupt may be raised if unmasked when the descriptor has been
          loaded.  */

    return pOldBuffer;
}

/**
 * Display an image on specified layer.
 * (Image scan: Left -> Right, Top -> Bottom.)
 * \param bLayer  Layer ID (OVR1, HEO or CUR).
 * \param pBuffer Pointer to image data.
 * \param bPP     Bits Per Pixel.
 *                - 16: TRGB 1555
 *                - 24:  RGB  888  packed
 *                - 32: ARGB 8888
 * \param x       X position.
 * \param y       Y position.
 * \param w       Width
 * \param h       Height (<0 means Bottom -> Top data).
 * \return Pointer to old display image data.
 */
void *LCDD_ShowBMP( UINT8 bLayer,
                    void* pBuffer, UINT8 bpp,
                    UINT32 x, UINT32 y,
                    INT32  w, INT32  h )
{
    return LCDD_ShowBMPRotated(bLayer, pBuffer, bpp,
                               x, y, w, h,
                               w, (h < 0) ? (-h) : h,
                               0);
}

/**
 * Start display on base layer
 * \param pBuffer   Pointer to image data.
 * \param bpp       Bits Per Pixel.
 * \param bBottomUp Scan from bottom to top.
 * \return Pointer to old display image data.
 */
void *LCDD_ShowBase( void* pBuffer, UINT8 bpp, UINT8 bBottomUp )
{
    return LCDD_ShowBMP(LCDD_BASE, pBuffer, bpp,
                        0, 0,
                        DISPLAY_WIDTH,
                        bBottomUp ? -DISPLAY_HEIGHT : DISPLAY_HEIGHT);
}

/**
 * Stop display on base layer
 */
void LCDD_StopBase( void )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;

    if (!(pLCDC->LCDC_BASECHSR & LCDC_BASECHSR_CHSR))
        return;

    /* 1. Clear the DFETCH bit in the DSCR.CHXCTRL field of the DSCR structure
          will disable the channel at the end of the frame. */
    /* 2. Set the DSCR.CHXNEXT field of the DSCR structure will disable the
          channel at the end of the frame. */
    LCDD_ClearDMA(&lcddBase.dmaD, (UINT32)&pLCDC->LCDC_BASEADDR);

    /* 3. Writing one to the CHDIS field of the CHXCHDR register will disable
          the channel at the end of the frame. */
    pLCDC->LCDC_BASECHDR = LCDC_BASECHDR_CHDIS;

    /* 4. Writing one to the CHRST field of the CHXCHDR register will disable
          the channel immediately. This may occur in the middle of the image. */

    /* 5. Poll CHSR field in the CHXCHSR register until the channel is
          successfully disabled. */
    while (pLCDC->LCDC_BASECHSR & LCDC_BASECHSR_CHSR);
}



//// Internal function, setup all register in the LCDC
//STATIC VOID
//ChipConfig( UINT32 FrameBuffer )
//{
//
//    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
//    //AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_VIRT_LCDC;
//
//
//    /* Disable interrupt */
//    pLCDC->LCDC_LCDIDR = 0xFFFFFFFF;
//
//    /* Configure channels */
//
//    /* Base */
//    pLCDC->LCDC_BASECFG0 =  LCDC_BASECFG0_DLBO | LCDC_BASECFG0_BLEN_AHB_INCR16;
//    //pLCDC->LCDC_BASECFG1 =  LCDC_BASECFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_BASECFG1 =  LCDC_BASECFG1_RGBMODE_32BPP_ARGB_8888;
//
//    /* Overlay 1, GA 0xFF */
//    pLCDC->LCDC_OVR1CFG0 =  LCDC_OVR1CFG0_DLBO | LCDC_OVR1CFG0_BLEN_AHB_BLEN_INCR16
//                       |  LCDC_OVR1CFG0_ROTDIS
//                       ;
//    pLCDC->LCDC_OVR1CFG1 =  LCDC_OVR1CFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_OVR1CFG9 =  LCDC_OVR1CFG9_GA(0xFF) | LCDC_OVR1CFG9_GAEN;
//
//    /* Overlay 2, GA 0xFF */
//    pLCDC->LCDC_OVR2CFG0 =  LCDC_OVR2CFG0_DLBO | LCDC_OVR2CFG0_BLEN_AHB_INCR16
//                       |  LCDC_OVR2CFG0_ROTDIS
//                       ;
//    pLCDC->LCDC_OVR2CFG1 =  LCDC_OVR2CFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_OVR2CFG9 =  LCDC_OVR2CFG9_GA(0xFF) | LCDC_OVR2CFG9_GAEN;
//
//    /* High End Overlay, GA 0xFF */
//    pLCDC->LCDC_HEOCFG0  =  LCDC_HEOCFG0_DLBO | LCDC_HEOCFG0_BLEN_AHB_BLEN_INCR16
//                       |  LCDC_HEOCFG0_ROTDIS
//                       ;
//    pLCDC->LCDC_HEOCFG1  =  LCDC_HEOCFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_HEOCFG12 =  LCDC_HEOCFG12_GA(0xFF) | LCDC_HEOCFG12_GAEN;
//
//    /* Hardware Cursor, GA 0xFF, Key #000000 */
//    pLCDC->LCDC_HCRCFG0  =  LCDC_HCRCFG0_DLBO | LCDC_HCRCFG0_BLEN_AHB_BLEN_INCR16;
//    pLCDC->LCDC_HCRCFG1  =  LCDC_HCRCFG1_RGBMODE_24BPP_RGB_888_PACKED;
//    pLCDC->LCDC_HCRCFG7  =  0x000000;
//    pLCDC->LCDC_HCRCFG8  =  0xFFFFFF;
//    pLCDC->LCDC_HCRCFG9  =  LCDC_HCRCFG9_GA(0xFF) | LCDC_HCRCFG9_GAEN;
//
//    /* 1. Configure LCD timing parameters, signal polarity and clock period. */
//    pLCDC->LCDC_LCDCFG0 = LCDC_LCDCFG0_CLKDIV((BOARD_MCK*2)/BOARD_LCD_PIXELCLOCK-2)
//                       |LCDC_LCDCFG0_CGDISHCR
//                       |LCDC_LCDCFG0_CGDISHEO
//                       |LCDC_LCDCFG0_CGDISOVR1
//                       |LCDC_LCDCFG0_CGDISOVR2
//                       |LCDC_LCDCFG0_CGDISBASE
//                       |LCDC_LCDCFG0_CLKPWMSEL  
//                       |LCDC_LCDCFG0_CLKSEL
//                       |LCDC_LCDCFG0_CLKPOL;
//
//    pLCDC->LCDC_LCDCFG1 = LCDC_LCDCFG1_VSPW(BOARD_LCD_TIMING_VPW-1)
//                       |LCDC_LCDCFG1_HSPW(BOARD_LCD_TIMING_HPW-1);
//
//    pLCDC->LCDC_LCDCFG2 = LCDC_LCDCFG2_VBPW(BOARD_LCD_TIMING_VBP)
//                       |LCDC_LCDCFG2_VFPW(BOARD_LCD_TIMING_VFP-1);
//
//    pLCDC->LCDC_LCDCFG3 = LCDC_LCDCFG3_HBPW(BOARD_LCD_TIMING_HBP-1)
//                       |LCDC_LCDCFG3_HFPW(BOARD_LCD_TIMING_HFP-1);
//
//    pLCDC->LCDC_LCDCFG4 = LCDC_LCDCFG4_RPF(DISPLAY_HEIGHT-1)
//                       |LCDC_LCDCFG4_PPL(DISPLAY_WIDTH-1);
//
//    pLCDC->LCDC_LCDCFG5 = LCDC_LCDCFG5_GUARDTIME(30)
//                       |LCDC_LCDCFG5_MODE_OUTPUT_24BPP
//                       |LCDC_LCDCFG5_DISPDLY
//                       |LCDC_LCDCFG5_VSPDLYS
//                       |LCDC_LCDCFG5_VSPOL
//                       |LCDC_LCDCFG5_HSPOL;
//
//    pLCDC->LCDC_LCDCFG6 = LCDC_LCDCFG6_PWMCVAL(0xF0)
//                       |LCDC_LCDCFG6_PWMPOL
//                       |LCDC_LCDCFG6_PWMPS(6);
//
//    /* 2. Enable the Pixel Clock by writing one to the CLKEN field of the
//          LCDC_LCDEN register. */
//    pLCDC->LCDC_LCDEN = LCDC_LCDEN_CLKEN;
//    /* 3. Poll CLKSTS field of the LCDC_LCDSR register to check that the clock
//       is running. */
//    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_CLKSTS));
//
//    /* 4. Enable Horizontal and Vertical Synchronization by writing one to the
//          SYNCEN field of the LCDC_LCDEN register. */
//    pLCDC->LCDC_LCDEN = LCDC_LCDEN_SYNCEN;
//    /* 5. Poll LCDSTS field of the LCDC_LCDSR register to check that the
//          synchronization is up. */
//    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_LCDSTS));
//
//    /* 6. Enable the display power signal writing one to the DISPEN field of the
//          LCDC_LCDEN register. */
//    pLCDC->LCDC_LCDEN = LCDC_LCDEN_DISPEN;
//    /* 7. Poll DISPSTS field of the LCDC_LCDSR register to check that the power
//          signal is activated. */
//    while(!(pLCDC->LCDC_LCDSR & LCDC_LCDSR_DISPSTS));
//    /* 8. Enable backlight */
//    pLCDC->LCDC_LCDEN = LCDC_LCDEN_PWMEN;
//
//    pLCDC->LCDC_BASECFG1 =  LCDC_BASECFG1_RGBMODE_32BPP_ARGB_8888;
//
//    LDCD_Desc.addr = (UINT32)FrameBuffer;
//    LDCD_Desc.ctrl = LCDC_BASECTRL_DFETCH;
//    LDCD_Desc.next = (UINT32)&LDCD_Desc + DMA_ADDR_OFFSET;
//
//    DEBUGMSG(DBG_LVL_LCD,"%s(%d) :  LDCD_Desc %X LDCD_Desc->next =%X\n", __FUNCTION__ ,__LINE__,&LDCD_Desc,LDCD_Desc.next);
//    pLCDC->LCDC_BASEHEAD = (UINT32)&LDCD_Desc + DMA_ADDR_OFFSET;;
//    pLCDC->LCDC_BASEADDR = (UINT32)FrameBuffer;
//    pLCDC->LCDC_BASECTRL |= LCDC_BASECTRL_DFETCH;
//    pLCDC->LCDC_BASENEXT = (UINT32)&LDCD_Desc + DMA_ADDR_OFFSET;
//
//    DEBUGMSG(DBG_LVL_LCD,"%s(%d) :  &pLCDC->LCDC_BASEHEAD =%X\n", __FUNCTION__ ,__LINE__,&pLCDC->LCDC_BASEHEAD);
//    DEBUGMSG(DBG_LVL_LCD,"%s(%d) :  pLCDC->LCDC_HCRCLUT =%X\n", __FUNCTION__ ,__LINE__,pLCDC->LCDC_HCRCLUT);
//
//    pLCDC->LCDC_BASECFG4 |= LCDC_BASECFG4_DMA;
//    pLCDC->LCDC_BASECHER = (LCDC_BASECHER_CHEN | LCDC_BASECHER_UPDATEEN);
////    =========================================================
//#if 1//AC fill yellow to LCD
//{
//    int i;
//    PUINT32 p = (PUINT32)FrameBuffer;
//
//    for(i = 0;i < DISPLAY_STRIDE*DISPLAY_HEIGHT/4;i++)
//        *p++ = 0xFFFF00;
//}
//#endif
//
//    return;
//}

STATIC VOID
BackLightInit( VOID )
{
    AT91PS_LCDC pLCDC = (AT91PS_LCDC)AT91C_BASE_LCDC;
    UINT32 cfg = pLCDC->LCDC_LCDCFG6 & ~LCDC_LCDCFG6_PWMCVAL_Msk;
/*
    const struct pio_desc hw_pio[] = {
        {"EN",    AT91C_PIN_PC(26),    1, PIO_DEFAULT, PIO_PERIPH_A}
    };

    pio_setup(hw_pio, sizeof(hw_pio)/sizeof(struct pio_desc));
*/
    pLCDC->LCDC_LCDCFG6 = cfg | LCDC_LCDCFG6_PWMCVAL(DEFAULT_BRIGHT_LEVEL);
    return;
}

#define DISPLAY_FRAMEBUFFER 0x07000000

//STATIC VOID
VOID
DisplayEnableLayer( VOID ) 
{
    LCDD_ShowBase((void *)(DISPLAY_FRAMEBUFFER + DMA_ADDR_OFFSET)/*FrameBuffer*/, 
                           DISPLAY_BPP, FALSE);
    LCDD_EnableLayer(LCDD_BASE, TRUE);

    return;
}

PUBLIC BOOL
DisplayInit( VOID ) 
{
    AT91PS_PMC  pPMC  = (AT91PS_PMC)AT91C_BASE_PMC;

    pPMC->PMC_PCER0 = (1 << SAMA5D3_ID_PIOA );       //power up the controller
    pPMC->PMC_PCER0 = (1 << SAMA5D3_ID_PIOC );       //power up the controller
    pPMC->PMC_PCER0 = (1 << SAMA5D3_ID_PIOE );       //power up the controller



//=========================================================
{
    int i,j,k;
    PUINT32 q = (PUINT32)(DISPLAY_FRAMEBUFFER + DMA_ADDR_OFFSET);


#if 1
#define GIF_ROM_BASE    0x20000
#define GIF_MAX_SIZE    0x20000
    EXTERN BOOL InitGIFBuffer( PVOID pBuf, UINT32 Size );
    InitGIFBuffer((void *)GIF_ROM_BASE, GIF_MAX_SIZE);

    for(i = 0;i < DISPLAY_WIDTH*DISPLAY_HEIGHT;i++)
        *q++ = 0x00000000;

    EXTERN BOOL TestGIF( VOID );
    if( TestGIF() )
    {
        EXTERN BOOL LoadGIF( PIMAGEHDR pImage, PVOID pGifImage );
        IMAGEHDR GifImageHdr;
        LoadGIF( &GifImageHdr, (void *)(DISPLAY_FRAMEBUFFER + DMA_ADDR_OFFSET) );
        //LoadGIF( &GifImageHdr, (void *)(DISPLAY_FRAMEBUFFER) );
//console_printf( "LoadGIF\n\r");
    }

#else
    for(i = 0;i < DISPLAY_WIDTH*DISPLAY_HEIGHT;i++)
        *q++ = 0xFFFFFFFF;

    #include "dooroos_logo.c"
    PBYTE   p;
    
    p = (PBYTE)(DISPLAY_FRAMEBUFFER + DMA_ADDR_OFFSET);
    k = ((DISPLAY_STRIDE  - Image_dooroos_logo.Width*4) / 2)&~3;
    k += ((DISPLAY_HEIGHT - Image_dooroos_logo.Height)/2) * DISPLAY_STRIDE;
    p += k;
    k = 0;
    for(i = 0;i < Image_dooroos_logo.Height;i++)
    {
        for(j = 0;j < Image_dooroos_logo.Width;j++)
        {
            *p++ = Image[k++];
            *p++ = Image[k++];
            *p++ = Image[k++];
            *p++ = 0;
        }

        p += (DISPLAY_STRIDE - Image_dooroos_logo.Width*4);
        k += (Image_dooroos_logo.nStride - Image_dooroos_logo.Width*Image_dooroos_logo.nBytePerPixel);
    }
#endif
}
   
    {
        LCDD_Initialize();

        LCDD_On();

        // enable layer
        DisplayEnableLayer( );

        BackLightInit();

        //GD_BackLight( TRUE );

        // Initialize the contrast
        //GD_Contrast( DEFAULT_BRIGHT_LEVEL );
    }


    return TRUE;
}



//
//PUBLIC VOID
//GD_BackLight( UINT32 nBaclight )
//{
//    pio_set_value(AT91C_PIN_PC(26), nBaclight );
//
//    return;
//}
