 //  ----------------------------------------------------------------------------
 //          ATMEL Microcontroller Software Support  -  ROUSSET  -
 //  ----------------------------------------------------------------------------
 //  DISCLAIMER:  THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 //  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 //  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 //  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 //  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 //  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 //  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 //  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 //  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 //  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //  ----------------------------------------------------------------------------
/*
 * AT91SAM9261 LCD Controller
 *
 * (C) Copyright 2001-2002
 * Wolfgang Denk, DENX Software Engineering -- wd@denx.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


/************************************************************************/
/* ** HEADER FILES														*/
/************************************************************************/
#include "main.h"
#include "stdlib.h"
#undef  CONFIG_LCD_LOGO
#include "lcd.h"
#include "lcd_lut.h"
#include "cmd_nand.h" // added by km192 - 20070411
#include "usartu.h"


unsigned char Loading[FULL_SIZE*30];			// SDRAM TOTAL 60M POSSABLE
unsigned char AnimationBuffer[FULL_SIZE*30];			// SDRAM TOTAL 60M POSSABLE
unsigned char IconBuffer[FULL_SIZE*130];			// SDRAM TOTAL 60M POSSABLE
unsigned char Spare[FULL_SIZE*1];			// SDRAM TOTAL 60M POSSABLE

/* #define DEBUG */
#define FB_SYNC_HOR_HIGH_ACT	1	/* horizontal sync high active	*/
#define FB_SYNC_VERT_HIGH_ACT	2	/* vertical sync high active	*/

/* More or less configurable parameters for LCDC controller*/
#define SIDSAFB_FIFO_SIZE		512
#define SIDSAFB_DMA_BURST_LEN	16
#define SIDSAFB_CRST_VAL        0xc8   // 0xda

#define AT91C_PMC_HCK1        ((unsigned int) 0x1 << 17) // (PMC) AHB LCDCK Clock Output
AT91PS_SYS  AT91_SYS = (AT91PS_SYS)AT91C_BASE_SYS;

#if 0
/* 640x480x16 @ 61 Hz */
vidinfo_t panel_info = {
	240, //	vl_col:
	320, // vl_row:
	64,  // vl_width:
	86,  //	vl_height:
    4965000, //	vl_pixclock:

   	/* LCD configuration register */
	0,	// vl_clkp;	 Clock polarity
	0,	// vl_oep;	 Output Enable polarity
	FB_SYNC_HOR_HIGH_ACT,  //	vl_hsp:		Horizontal Sync polarity
	FB_SYNC_VERT_HIGH_ACT, //	vl_vsp:		Vertical Sync polarity
	0, //  vl_dp;	 Data polarity
	4, //  vl_bpix;	 Bits per pixel, 0 = 1, 1 = 2, 2 = 4, 3 = 8, 4 = 16 
	0, //  vl_lbw;	 LCD Bus width, 0 = 4, 1 = 8
	0, //  vl_splt;	 Split display, 0 = single-scan, 1 = dual-scan
	0, //  vl_clor;	 Color, 0 = mono, 1 = color
	1, // 	vl_tft:  0 = passive, 1 = TFT

	/* Horizontal control Timing */
	5, //	vl_hpw:
	1, // 	vl_blw:
	33,// 	vl_elw:
	/* Vertical control Timing */
	1, // 	vl_vpw:
	1, // 	vl_bfw:
	0, //	vl_efw:
};
#else // Modified by km192 - 20070411
vidinfo_t panel_info = {
	480, //	vl_col:
	272, // vl_row:
	64,  // vl_width:
	86,  //	vl_height:
    9000000, //	vl_pixclock:

   	/* LCD configuration register */
	0,	// vl_clkp;	 Clock polarity
	0,	// vl_oep;	 Output Enable polarity
	0,  //	vl_hsp:		Horizontal Sync polarity
	0, //	vl_vsp:		Vertical Sync polarity
	0, //  vl_dp;	 Data polarity
	4, //  vl_bpix;	 Bits per pixel, 0 = 1, 1 = 2, 2 = 4, 3 = 8, 4 = 16 
	1, //  vl_lbw;	 LCD Bus width, 0 = 4, 1 = 8
	1, //  vl_splt;	 Split display, 0 = single-scan, 1 = dual-scan
	1, //  vl_clor;	 Color, 0 = mono, 1 = color
	1, // 	vl_tft:  0 = passive, 1 = TFT

	/* Horizontal control Timing */
	41, //	vl_hpw:
	2, // 	vl_blw:
	2,// 	vl_elw:
	/* Vertical control Timing */
	10, // 	vl_vpw:
	12, // 	vl_bfw:
	6, //	vl_efw:
};
#endif 

void lcd_setcolreg (void);
void lcd_ctrl_init (void *lcdbase);

static void lcd_init_lut(void);
extern int drv_lcd_init (void);
extern void lcd_clearscreen (void);
extern void AT91F_PIT_ASM_Handler(void);
extern void AT91F_GPIO_ASM_Handler(void);

unsigned int AniOffset(unsigned int address);
void nand_chip_type(void);
void nand_read_page(unsigned long file_pos_ll, unsigned char * buffer, unsigned int bytes_to_read, int type);
void AT91F_NandFlash_Init(void);
void LedStateOut_KeyCheck(void);
static void AT91F_LCDC_TFT_CfgPIO (void);
static void lcd_init_lut(void);
void lcd_ctrl_init (void *lcdbase);
void Delay(unsigned int DelayValue);
void change_framebuffer_base(void *lcdbase);
void read_image_from_nand(unsigned char* TestBuffer, unsigned int TestBufferSize, unsigned long start_address);
void AT91F_Configure_GPIO (void);
void AT91F_Configure_Timer(void);
void EndodeMath(void);
void LedMath(void);
void TestAction(void);
void TestScreenNumber(void);

int lcd_line_length;
int lcd_color_fg;
int lcd_color_bg;

char *lcd_base;			/* Start of framebuffer memory	*/
char *lcd_console_address;		/* Start of console buffer	*/
unsigned short ani_offset=0;

unsigned short Timer_1ms_diplay=0;
unsigned char Timer_1ms=0;
unsigned char Timer_10ms=0;
unsigned char Timer_100ms=0;
unsigned char Timer_1S=0;
unsigned char Timer_1S_FLAG=0;
unsigned int TestDisplay=10;
unsigned char Timer_1M=0;

unsigned int animation_frame = 0;
unsigned char DisplayFlag = 0;

unsigned char ANI_MODE=0;
#define ANI_MODE_NULL		0	
#define ANI_MODE_ON			1	
#define ANI_MODE_OFF		2	
#define ANI_MODE_RINSE		3	
#define ANI_MODE_SPIN		4	
#define ANI_MODE_STEAM		5	
#define ANI_MODE_TUB		6	
#define ANI_MODE_WASH		7	
#define ANI_MODE_LOADING	8	

struct _LED_STATE 
{
    unsigned char L1;
    unsigned char L2;
    unsigned char L3;
    unsigned char L4;
  
    unsigned char LED4;
    unsigned char LED5;
    unsigned char LED6;
    unsigned char LED7;
    unsigned char LED8;
    unsigned char LED9;

    unsigned char LED10_1;		// 1
    unsigned char LED11_5;		// 5
    unsigned char LED12_8;		// 8
    unsigned char LED13_R;		// RIGHT
    unsigned char LED14_2;		// 2
    unsigned char LED15_6;		// 6
    unsigned char LED16_9;		// 9
    unsigned char LED17_3;		// 3
    unsigned char LED18_7;		// 7
    unsigned char LED19_L;		// LEFT
    unsigned char LED20_4;		// 4
};
struct _LED_STATE LED_STATE;

void LedStateInit(void)
{
	LED_STATE.L1 = 0;
	LED_STATE.L2 = 0;
	LED_STATE.L3 = 0;
	LED_STATE.L4 = 0;

	LED_STATE.LED4 = 0;
	LED_STATE.LED5 = 0;
	LED_STATE.LED6 = 0;
	LED_STATE.LED7 = 0;
	LED_STATE.LED8 = 0;
	LED_STATE.LED9 = 0;

	LED_STATE.LED10_1 = 0;
	LED_STATE.LED11_5 = 0;
	LED_STATE.LED12_8 = 0;
	LED_STATE.LED13_R = 0;
	LED_STATE.LED14_2 = 0;
	LED_STATE.LED15_6 = 0;
	LED_STATE.LED16_9 = 0;
	LED_STATE.LED17_3 = 0;
	LED_STATE.LED18_7 = 0;
	LED_STATE.LED19_L = 0;
	LED_STATE.LED20_4 = 0;
	AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA30 | AT91C_PIO_PA31));
}

struct _KEY_STATE 
{
	unsigned char SW1;
	unsigned char SW2;
	unsigned char SW3;
	unsigned char SW4;
	unsigned char SW5;
	unsigned char SWITCH1;
	unsigned char SWITCH2;
	unsigned char SWITCH3;
	unsigned char SWITCH4;
	unsigned char SWITCH5;
	unsigned char SWITCH6;
	
	unsigned char ENCODEPA5;
	//unsigned char ENCODEPA12;
	unsigned char ENCODE_DIRECTION;

	unsigned char SW1_FLAG;
	unsigned char SW2_FLAG;
	unsigned char SW3_FLAG;
	unsigned char SW4_FLAG;
	unsigned char SW5_FLAG;
	unsigned char SWITCH1_FLAG;
	unsigned char SWITCH2_FLAG;
	unsigned char SWITCH3_FLAG;
	unsigned char SWITCH4_FLAG;
	unsigned char SWITCH5_FLAG;
	unsigned char SWITCH6_FLAG;

	unsigned char ENCODEPA5_FLAG;
	//unsigned char ENCODEPA12_FLAG;
};
struct _KEY_STATE KEY_STATE;

void KeyStateInit(void)
{
	KEY_STATE.SW1 = 0;
	KEY_STATE.SW2 = 0;
	KEY_STATE.SW3 = 0;
	KEY_STATE.SW4 = 0;
	KEY_STATE.SW5 = 0;
	KEY_STATE.SWITCH1 = 0;
	KEY_STATE.SWITCH2 = 0;
	KEY_STATE.SWITCH3 = 0;
	KEY_STATE.SWITCH4 = 0;
	KEY_STATE.SWITCH5 = 0;
	KEY_STATE.SWITCH6 = 0;

	KEY_STATE.ENCODEPA5 = 0;
	//KEY_STATE.ENCODEPA12 = 0;
	KEY_STATE.ENCODE_DIRECTION = 0;

	KEY_STATE.SW1_FLAG = 0;
	KEY_STATE.SW2_FLAG = 0;
	KEY_STATE.SW3_FLAG = 0;
	KEY_STATE.SW4_FLAG = 0;
	KEY_STATE.SW5_FLAG = 0;
	KEY_STATE.SWITCH1_FLAG = 0;
	KEY_STATE.SWITCH2_FLAG = 0;
	KEY_STATE.SWITCH3_FLAG = 0;
	KEY_STATE.SWITCH4_FLAG = 0;
	KEY_STATE.SWITCH5_FLAG = 0;
	KEY_STATE.SWITCH6_FLAG = 0;

	KEY_STATE.ENCODEPA5_FLAG = 0;
	//KEY_STATE.ENCODEPA12_FLAG = 0;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_Test_LCD
//* \brief Test LCD
//*----------------------------------------------------------------------------
void main (void)
{
	unsigned int kkk = 0;
	AT91S_PMC	*pPMC;
	AT91S_AIC	*pAIC;

	pPMC = AT91C_BASE_PMC;
	pAIC = AT91C_BASE_AIC;

//	AT91F_DBGU_Printk("\r\n#############################\r\n");
//	AT91F_DBGU_Printk("# LCDC/TFT DISPLAY          #\r\n");
//	AT91F_DBGU_Printk("# Basic for SAM9261         #\r\n");
//	AT91F_DBGU_Printk("#############################\r\n");

	drv_lcd_init();
	Delay(100);
	lcd_clearscreen(); 

	AT91F_Configure_GPIO();
	AT91F_Configure_Timer();
	LedStateInit();
	KeyStateInit();

	AT91F_NandFlash_Init(); 
	read_image_from_nand((unsigned char*)&Loading, FULL_SIZE*3, NAND_ANI_LOADING_ADD);
	animation_frame = 0;
	ANI_MODE=ANI_MODE_LOADING;

	AT91F_NandFlash_Init(); 
	read_image_from_nand((unsigned char*)&IconBuffer, FULL_SIZE*130, NAND_ICON_ADD);
	animation_frame = 0;
	//ANI_MODE=ANI_MODE_LOADING;

	AT91F_NandFlash_Init(); 
	read_image_from_nand((unsigned char*)&AnimationBuffer, FULL_SIZE*30, NAND_ANI_ON_ADD);
	animation_frame = 0;
	ANI_MODE=ANI_MODE_ON;

	while(1)
	{	
		if(KEY_STATE.SW1)
		{
			KEY_STATE.SW1=0;
			AT91F_NandFlash_Init(); 
			read_image_from_nand((unsigned char*)&AnimationBuffer, FULL_SIZE*22, NAND_ANI_OFF_ADD);
			animation_frame = 0;
			ANI_MODE=ANI_MODE_OFF;
		}
		if(KEY_STATE.SW2)
		{
			KEY_STATE.SW2=0;
			ANI_MODE=ANI_MODE_LOADING;
			AT91F_NandFlash_Init(); 
			read_image_from_nand((unsigned char*)&AnimationBuffer, BACK_SIZE*30, NAND_ANI_RINSE_ADD);
			animation_frame = 0;
			ANI_MODE=ANI_MODE_RINSE;
		}
		if(KEY_STATE.SW3)
		{
			KEY_STATE.SW3=0;
			ANI_MODE=ANI_MODE_LOADING;
			AT91F_NandFlash_Init(); 
			read_image_from_nand((unsigned char*)&AnimationBuffer, BACK_SIZE*30, NAND_ANI_SPIN_ADD);
			animation_frame = 0;
			ANI_MODE=ANI_MODE_SPIN;
		}
		
		if(KEY_STATE.SW4)
		{
			KEY_STATE.SW4=0;
			ANI_MODE=ANI_MODE_LOADING;
			AT91F_NandFlash_Init(); 
			read_image_from_nand((unsigned char*)&AnimationBuffer, BACK_SIZE*30, NAND_ANI_STEAM_ADD);
			animation_frame = 0;
			ANI_MODE=ANI_MODE_STEAM;
		}

		if(KEY_STATE.SW5)
		{
			KEY_STATE.SW5=0;
			ANI_MODE=ANI_MODE_LOADING;
			AT91F_NandFlash_Init(); 
			read_image_from_nand((unsigned char*)&AnimationBuffer, BACK_SIZE*30, NAND_ANI_TUB_ADD);
			animation_frame = 0;
			ANI_MODE=ANI_MODE_TUB;
		}

		if(KEY_STATE.SWITCH1)
		{
			KEY_STATE.SWITCH1=0;
			ANI_MODE=ANI_MODE_LOADING;
			AT91F_NandFlash_Init(); 
			read_image_from_nand((unsigned char*)&AnimationBuffer, BACK_SIZE*30, NAND_ANI_WASH_ADD);
			animation_frame = 0;
			ANI_MODE=ANI_MODE_WASH;
		}


		if(ANI_MODE==ANI_MODE_RINSE || ANI_MODE==ANI_MODE_SPIN || ANI_MODE==ANI_MODE_STEAM || ANI_MODE==ANI_MODE_TUB || ANI_MODE==ANI_MODE_WASH)
		{
			if(DisplayFlag == 1) 
			{
				DisplayFlag = 0;
				//nand_image_display(480-480-0, 272-235-3, 480, 235, AnimationBuffer+(BACK_SIZE*animation_frame), 0);
				TestAction();


				/*if(animation_frame >= 29)
					animation_frame=0;	
				else
					animation_frame+=1;
				*/
			}
		}
		
		/*if(DisplayFlag == 1) 
		{
			nand_image_display(0, 0, 480, 272, TestBuffer+(FULL_SIZE*animation_frame), 0);
			nand_image_display(2, 4, 10, 10, TestBuffer+(FULL_SIZE*230), 0);
			nand_image_display(4, 8, 10, 10, TestBuffer+(NAND_SIZE*230), 0);
			nand_image_display(6, 12, 10, 10, TestBuffer+(NAND_SIZE*230), 0);
			nand_image_display(8, 16, 10, 10, TestBuffer+(NAND_SIZE*230), 0);
			nand_image_display(10, 20, 10, 10, TestBuffer+(NAND_SIZE*230), 0);
			nand_image_display(12, 24, 10, 10, TestBuffer+(NAND_SIZE*230), 0);
			nand_image_display(14, 28, 10, 10, TestBuffer+(NAND_SIZE*230), 1);
			
			//logo_image_display(TestBuffer+(NAND_SIZE*animation_frame));
			if(animation_frame >= 229)
				animation_frame=0;	
			else
				animation_frame+=1;
		}*/			

		/*	animation_frame=0;		
		kkk=0;
		AT91F_NandFlash_Init(); 
		read_image_from_nand((unsigned char*)&TestBuffer, NAND_SIZE*80, NAND_SIZE*0);
		//AT91F_Memory_Dump((char *)&TestBuffer, 128);
		while(animation_frame < 80)
		{
			switch(kkk)
			{
			case 100:
				logo_image_display(TestBuffer+(NAND_SIZE*animation_frame));
				///change_framebuffer_base (TestBuffer+(NAND_SIZE*animation_frame));
				animation_frame+=1;
				break;
			}
			if(kkk > 200)	kkk=0;
			else	kkk++;
		}

		animation_frame=0;
		kkk=0;
		AT91F_NandFlash_Init(); 
		read_image_from_nand((unsigned char*)&TestBuffer2, NAND_SIZE*80, NAND_SIZE*80);
		//AT91F_Memory_Dump((char *)&TestBuffer, 128);
		while(animation_frame < 80)
		{
			switch(kkk)
			{
			case 100:
				logo_image_display(TestBuffer2+(NAND_SIZE*animation_frame));
				///change_framebuffer_base (TestBuffer2+(NAND_SIZE*animation_frame));
				animation_frame+=1;
				break;
			}
			if(kkk > 200)	kkk=0;
			else	kkk++;
		}

		animation_frame=0;		
		kkk=0;
		AT91F_NandFlash_Init(); 
		read_image_from_nand((unsigned char*)&TestBuffer3, NAND_SIZE*70, NAND_SIZE*160);
		//AT91F_Memory_Dump((char *)&TestBuffer, 128);
		while(animation_frame < 70)
		{
			switch(kkk)
			{
			case 100:
				logo_image_display(TestBuffer3+(NAND_SIZE*animation_frame));
				///change_framebuffer_base (TestBuffer3+(NAND_SIZE*animation_frame));
				animation_frame+=1;
				break;
			}
			if(kkk > 200)	kkk=0;
			else	kkk++;
		}
	*/	

	}
}

void AT91F_PIT_Handler(void)
{
	unsigned int temp, temp1;
	Timer_1ms++;
	Timer_1ms_diplay++;
	
	LedStateOut_KeyCheck();
	EndodeMath();

	if(Timer_1ms_diplay >= 100)
	{
		Timer_1ms_diplay = 0;
		DisplayFlag = 1;
		
		if(ANI_MODE == ANI_MODE_ON)
		{
			change_framebuffer_base (AnimationBuffer+(FULL_SIZE*animation_frame));
			//lcd_ctrl_init (AnimationBuffer+(FULL_SIZE*animation_frame));
			if(animation_frame >=29)		animation_frame = 0;
			else	animation_frame+=1;
		}
		else if(ANI_MODE == ANI_MODE_OFF)
		{
			change_framebuffer_base (AnimationBuffer+(FULL_SIZE*animation_frame));
			//lcd_ctrl_init (AnimationBuffer+(FULL_SIZE*animation_frame));
			if(animation_frame >=21)		animation_frame = 0;
			else	animation_frame+=1;
		}
		else if(ANI_MODE == ANI_MODE_LOADING)
		{
			change_framebuffer_base (Loading+(FULL_SIZE*animation_frame));
			//lcd_ctrl_init (IconBuffer+(FULL_SIZE*animation_frame));
			if(animation_frame >=2)		animation_frame = 0;
			else	animation_frame+=1;
		}
		else
		{
		}
	}

	if(Timer_1ms >= 10)
	{
		Timer_1ms = 0;
		Timer_10ms++;
	}

	if(Timer_10ms >= 10)
	{
		Timer_10ms = 0;
		Timer_100ms++;
		LedMath();
	}

	if(Timer_100ms >= 10)
	{
		Timer_100ms = 0;
		Timer_1S++;
		Timer_1S_FLAG=1;
	}

	if(Timer_1S >= 22)
	{
		Timer_1S = 0;
		Timer_1M++;
	}

	if(Timer_1M >= 60)
	{
		Timer_1M = 0;
	}

	temp = AT91C_BASE_PITC->PITC_PIVR; /* Clear the PIT interrupt. */ 
	temp1 = AT91C_BASE_PITC->PITC_PIIR;
}

unsigned int AniOffset(unsigned int address)
{
	unsigned int temp;
	if(address%1024)
	{
		temp=address;
		address/=1024;
		address-=1;
		address*=1024;
		ani_offset=(1024-(temp-address));
		return(address);
	}
	else
	{
		ani_offset=0;
		return(address);
	}
}

void LedMath(void)
{
	static unsigned char kkk=0;
	/*if(KEY_STATE.SW1)
	{
		KEY_STATE.SW1=0;
		switch(kkk)
		{
		case 0:
			LED_STATE.LED13_R = 1;
			LED_STATE.LED19_L = 0;
			break;
		case 1:
			LED_STATE.LED13_R = 0;
			LED_STATE.LED19_L = 1;
			break;
		case 2:
			LED_STATE.LED13_R = 1;
			LED_STATE.LED19_L = 1;
			break;
		case 3:
			LED_STATE.LED13_R = 0;
			LED_STATE.LED19_L = 0;
			kkk=0;
			break;
		default:
			break;
		}
		kkk+=1;
	}*/		
	/*if(KEY_STATE.SW2)
	{
		KEY_STATE.SW2=0;
		if(LED_STATE.L1)
			LED_STATE.L1 = 0;
		else
			LED_STATE.L1 = 1;
	}*/
	/*if(KEY_STATE.SW3)
	{
		KEY_STATE.SW3=0;
		if(LED_STATE.L2)
			LED_STATE.L2 = 0;
		else
			LED_STATE.L2 = 1;
	}
	if(KEY_STATE.SW4)
	{
		KEY_STATE.SW4=0;
		if(LED_STATE.L3)
			LED_STATE.L3 = 0;
		else
			LED_STATE.L3 = 1;
	}
	if(KEY_STATE.SW5)
	{
		KEY_STATE.SW5=0;
		if(LED_STATE.L4)
			LED_STATE.L4 = 0;
		else
			LED_STATE.L4 = 1;
	}
	if(KEY_STATE.SWITCH1)
	{
		KEY_STATE.SWITCH1=0;
		if(LED_STATE.LED4)
			LED_STATE.LED4 = 0;
		else
			LED_STATE.LED4 = 1;
	}*/
		
	if(KEY_STATE.SWITCH2)
	{
		KEY_STATE.SWITCH2=0;
		if(LED_STATE.LED5)
			LED_STATE.LED5 = 0;
		else
			LED_STATE.LED5 = 1;
	}		
	if(KEY_STATE.SWITCH3)
	{
		KEY_STATE.SWITCH3=0;
		if(LED_STATE.LED6)
			LED_STATE.LED6 = 0;
		else
			LED_STATE.LED6 = 1;
	}		
	if(KEY_STATE.SWITCH4)
	{
		KEY_STATE.SWITCH4=0;
		if(LED_STATE.LED7)
			LED_STATE.LED7 = 0;
		else
			LED_STATE.LED7 = 1;
	}		
	if(KEY_STATE.SWITCH5)
	{
		KEY_STATE.SWITCH5=0;
		if(LED_STATE.LED8)
			LED_STATE.LED8 = 0;
		else
			LED_STATE.LED8 = 1;
	}		
	if(KEY_STATE.SWITCH6)
	{
		KEY_STATE.SWITCH6=0;
		if(LED_STATE.LED9)
			LED_STATE.LED9 = 0;
		else
			LED_STATE.LED9 = 1;
	}		

	if(KEY_STATE.ENCODEPA5)
	{
		KEY_STATE.ENCODEPA5=0;
	
		if(KEY_STATE.ENCODE_DIRECTION)
		{
			TestDisplay+=1;
			if(LED_STATE.LED10_1)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=1;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED14_2)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=1;
				LED_STATE.LED20_4=0;	
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED17_3)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=1;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED20_4)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=1;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED11_5)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=1;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED15_6)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=1;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED18_7)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=1;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED12_8)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=1;
			}
			else if(LED_STATE.LED16_9)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else
			{
				LED_STATE.LED10_1=1;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
		}
		else
		{
			TestDisplay-=1;
			if(LED_STATE.LED10_1)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED14_2)
			{
				LED_STATE.LED10_1=1;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED17_3)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=1;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}	
			else if(LED_STATE.LED20_4)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=1;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED11_5)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=1;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED15_6)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=1;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED18_7)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=1;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED12_8)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=1;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=0;
			}
			else if(LED_STATE.LED16_9)
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=1;
				LED_STATE.LED16_9=0;
			}
			else
			{
				LED_STATE.LED10_1=0;
				LED_STATE.LED14_2=0;
				LED_STATE.LED17_3=0;
				LED_STATE.LED20_4=0;
				LED_STATE.LED11_5=0;
				LED_STATE.LED15_6=0;
				LED_STATE.LED18_7=0;
				LED_STATE.LED12_8=0;
				LED_STATE.LED16_9=1;
			}
		}
	}	
}

void EndodeMath(void)
{
	if(!AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA5))			// ENCODEPA5 KEY CHECK 			
	{
		if(!KEY_STATE.ENCODEPA5_FLAG)
		{
			KEY_STATE.ENCODEPA5_FLAG = 1;
			KEY_STATE.ENCODEPA5 = 1;
			if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA12))			// ENCODEPA12 KEY CHECK 			
				KEY_STATE.ENCODE_DIRECTION = 1;		//RIGHT
			else
				KEY_STATE.ENCODE_DIRECTION = 0;		//LEFT
		}
	}
	else
	{
		KEY_STATE.ENCODEPA5_FLAG = 0;
	}
}

void LedStateOut_KeyCheck(void)
{
	static unsigned char step=0;
	switch(step)
	{
	case 0:
		step+=1;
		if(LED_STATE.L1)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15 | AT91C_PIO_PA29));				// L1, L5 on	
		}
		if(LED_STATE.L2)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA14 | AT91C_PIO_PA29));				// L2, L6 on	
		}
		if(LED_STATE.LED19_L)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA19 | AT91C_PIO_PA28));				// LED19_L on	
		}
		break;
	case 1:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15 | AT91C_PIO_PA29));				// L1, L5 off
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA14 | AT91C_PIO_PA29));				// L2, L6 off	
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA19 | AT91C_PIO_PA28));				// LED19_L off	
		if(LED_STATE.L3)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15 | AT91C_PIO_PA28));				// L3, L7 on	
		}
		if(LED_STATE.L4)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA14 | AT91C_PIO_PA28));				// L4, L8 on	
		}
		break;
	case 2:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15 | AT91C_PIO_PA28));				// L3, L7 off
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA14 | AT91C_PIO_PA28));				// L4, L8 off
		if(LED_STATE.LED6)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA17 | AT91C_PIO_PA27));				// LED6 on
		}
		if(LED_STATE.LED9)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA17 | AT91C_PIO_PA26));				// LED9 on
		}
		break;
	case 3:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA17 | AT91C_PIO_PA27));				// LED6 on
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA17 | AT91C_PIO_PA26));				// LED9 on
		if(LED_STATE.LED5)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA16 | AT91C_PIO_PA27));				// LED5 on
		}
		if(LED_STATE.LED8)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA16 | AT91C_PIO_PA26));				// LED8 on
		}
		break;
	case 4:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA16 | AT91C_PIO_PA27));				// LED5 on
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA16 | AT91C_PIO_PA26));				// LED8 on
		if(LED_STATE.LED4)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15 | AT91C_PIO_PA27));				// LED4 on
		}
		if(LED_STATE.LED7)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15 | AT91C_PIO_PA26));				// LED7 on
		}
		break;
	case 5:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15 | AT91C_PIO_PA27));				// LED4 on
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15 | AT91C_PIO_PA26));				// LED7 on
		if(LED_STATE.LED10_1)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA21 | AT91C_PIO_PA26));				// LED10_1 on	
		}
		if(LED_STATE.LED14_2)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA21 | AT91C_PIO_PA27));				// LED14_2 on	
		}
		break;
	case 6:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA21 | AT91C_PIO_PA26));				// LED10_1 off	
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA21 | AT91C_PIO_PA27));				// LED14_2 off	
		if(LED_STATE.LED11_5)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA20 | AT91C_PIO_PA26));				// LED11_5 on	
		}
		if(LED_STATE.LED15_6)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA20 | AT91C_PIO_PA27));				// LED15_6 on	
		}
		break;
	case 7:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA20 | AT91C_PIO_PA26));				// LED11_5 off	
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA20 | AT91C_PIO_PA27));				// LED15_6 off	
		if(LED_STATE.LED12_8)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA19 | AT91C_PIO_PA26));				// LED12_8 on	
		}
		if(LED_STATE.LED16_9)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA19 | AT91C_PIO_PA27));				// LED16_9 on	
		}
		break;
	case 8:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA19 | AT91C_PIO_PA26));				// LED12_8 off	
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA19 | AT91C_PIO_PA27));				// LED16_9 off	
		if(LED_STATE.LED13_R)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA18 | AT91C_PIO_PA26));				// LED13_R on	
		}
		break;
	case 9:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA18 | AT91C_PIO_PA26));				// LED13_R off	
		if(LED_STATE.LED17_3)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA21 | AT91C_PIO_PA28));				// LED17_3 on	
		}
		if(LED_STATE.LED20_4)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA21 | AT91C_PIO_PA29));				// LED20_4 on	
		}
		break;
	case 10:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA21 | AT91C_PIO_PA28));				// LED17_3 off	
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA21 | AT91C_PIO_PA29));				// LED20_4 off	
		if(LED_STATE.LED18_7)
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA20 | AT91C_PIO_PA28));				// LED18_7 on	
		}
		break;
	case 11:
		step+=1;
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA20 | AT91C_PIO_PA28));				// LED18_7 off	
		AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA20));				// SW2, SW3 KEY CHECK READY			
		break;
	case 12:
		step+=1;
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA24))			// SW2 KEY CHECK 			
		{	
			if(!KEY_STATE.SW2_FLAG)			
			{
				KEY_STATE.SW2_FLAG = 1;
				KEY_STATE.SW2 = 1;
			}
		}
		else
		{
			KEY_STATE.SW2_FLAG = 0;
		}
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA23))			// SW3 KEY CHECK 			
		{
			if(!KEY_STATE.SW3_FLAG)
			{
				KEY_STATE.SW3_FLAG = 1;
				KEY_STATE.SW3 = 1;
			}
		}
		else
		{
			KEY_STATE.SW3_FLAG = 0;
		}
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA20));			// SW2, SW3 KEY CHECK FREE				
		AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA19));				// SW4, SW5 KEY CHECK READY		
		break;
	case 13:
		step+=1;
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA24))			// SW4 KEY CHECK 			
		{
			if(!KEY_STATE.SW4_FLAG)
			{
				KEY_STATE.SW4_FLAG = 1;
				KEY_STATE.SW4 = 1;
			}
		}
		else
		{
			KEY_STATE.SW4_FLAG = 0;
		}
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA23))			// SW5 KEY CHECK 			
		{
			if(!KEY_STATE.SW5_FLAG)
			{
				KEY_STATE.SW5_FLAG = 1;
				KEY_STATE.SW5 = 1;
			}
		}	
		else
		{
			KEY_STATE.SW5_FLAG = 0;
		}
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA19));			// SW4, SW5 KEY CHECK FREE							
		AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA17));				// SW1 KEY CHECK READY		
		break;
	case 14:
		step+=1;
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA24))			// SW1 KEY CHECK 			
		{
			if(!KEY_STATE.SW1_FLAG)
			{
				KEY_STATE.SW1_FLAG = 1;
				KEY_STATE.SW1 = 1;
			}
		}
		else
		{
			KEY_STATE.SW1_FLAG = 0;
		}
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA17));			// SW1 KEY CHECK FREE		
		AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA16));				// SWITCH3, SWITCH6 KEY CHECK READY		
		break;
	case 15:
		step+=1;
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA24))			// SWITCH3 KEY CHECK 			
		{
			if(!KEY_STATE.SWITCH3_FLAG)
			{
				KEY_STATE.SWITCH3_FLAG = 1;
				KEY_STATE.SWITCH3 = 1;
			}
		}
		else
		{
			KEY_STATE.SWITCH3_FLAG = 0;
		}
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA23))			// SWITCH6 KEY CHECK 			
		{
			if(!KEY_STATE.SWITCH6_FLAG)
			{
				KEY_STATE.SWITCH6_FLAG = 1;
				KEY_STATE.SWITCH6 = 1;
			}
		}
		else
		{
			KEY_STATE.SWITCH6_FLAG = 0;
		}
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA16));			// SWITCH3, SWITCH6 KEY CHECK FREE		
		AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15));				// SWITCH2, SWITCH5 KEY CHECK READY		
		break;
	case 16:
		step+=1;
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA24))			// SWITCH2 KEY CHECK 			
		{
			if(!KEY_STATE.SWITCH2_FLAG)
			{
				KEY_STATE.SWITCH2_FLAG = 1;
				KEY_STATE.SWITCH2 = 1;
			}
		}
		else
		{
			KEY_STATE.SWITCH2_FLAG = 0;
		}
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA23))			// SWITCH5 KEY CHECK 			
		{
			if(!KEY_STATE.SWITCH5_FLAG)
			{
				KEY_STATE.SWITCH5_FLAG = 1;
				KEY_STATE.SWITCH5 = 1;
			}
		}
		else
		{
			KEY_STATE.SWITCH5_FLAG = 0;
		}		
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA15));			// SWITCH2, SWITCH5 KEY CHECK FREE		
		AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA14));				// SWITCH1, SWITCH4 KEY CHECK READY		
		break;
	case 17:
		step+=1;
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA24))			// SWITCH1 KEY CHECK 			
		{
			if(!KEY_STATE.SWITCH1_FLAG)
			{
				KEY_STATE.SWITCH1_FLAG = 1;
				KEY_STATE.SWITCH1 = 1;
			}
		}
		else
		{
			KEY_STATE.SWITCH1_FLAG = 0;
		}
		if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA23))			// SWITCH4 KEY CHECK 			
		{
			if(!KEY_STATE.SWITCH4_FLAG)
			{
				KEY_STATE.SWITCH4_FLAG = 1;
				KEY_STATE.SWITCH4 = 1;
			}
		}
		else
		{
			KEY_STATE.SWITCH4_FLAG = 0;
		}		
		AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA14));			// SWITCH1, SWITCH4 KEY CHECK FREE		
		step=0;
		break;
	default:
		step=0;
		break;
	}
}


void Delay(unsigned int DelayValue)
{
	unsigned int i, j;
	for(i=0; i<DelayValue; i++)
	{
		for(j=0; j<1000; j++)	{}
	}
}

void change_framebuffer_base(void *lcdbase)
{

	unsigned long value;

//	AT91F_LCDC_TFT_CfgPIO();

	/* Enable HCLOCK in PMC */
	AT91_SYS->PMC_SCER = AT91C_PMC_HCK1;

	/* Init controller field in panel_info */
	panel_info.controller.lcdc = (AT91PS_LCDC) AT91C_BASE_LCDC;
	panel_info.controller.frame_buffer = (unsigned long) lcdbase;

	/* Disable all interrupts */
	panel_info.controller.lcdc->LCDC_IDR = ~0UL;
	// Set contrast
	value = AT91C_LCDC_PS_DIVIDEDBYEIGHT | AT91C_LCDC_POL_POSITIVEPULSE | AT91C_LCDC_ENA_PWMGEMENABLED;
	panel_info.controller.lcdc->LCDC_CTRSTCON = value;
	panel_info.controller.lcdc->LCDC_CTRSTVAL = 0xDA; 


	panel_info.controller.lcdc->LCDC_BA1 = (unsigned int)lcdbase;

}

void read_image_from_nand(unsigned char* TestBuffer, unsigned int TestBufferSize, unsigned long start_address)
{
	
	unsigned int i;
	char tmp[54];
	u_char	buffer[1024];
	unsigned int image1_offset, image2_offset;
	unsigned int width, height, size, page_total;
	unsigned int offset=0;

	//NAND READ SPARE AREA
	nand_read_page(0,buffer,NAND_BAB_OFF,NAND_OOB);
	if(buffer[NAND_BAB_OFF-1] != GOOD_BLOCK){
	}

	page_total = (TestBufferSize) / NANDFLASH_PAGESIZE;
	for(i=0; i<=page_total; i++)
	{
		offset = i*NANDFLASH_PAGESIZE;
		nand_read_page(offset+(start_address),TestBuffer+offset,NANDFLASH_PAGESIZE,NAND_DATA);
	}
}

void AT91F_GPIO_Handler(void)
{
/*	if(AT91F_PIO_IsInterruptSet(AT91C_BASE_PIOA, AT91C_PIO_PA5))
	{
			KEY_STATE.ENCODEPA5 = 1;
			if(AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, AT91C_PIO_PA12))			// ENCODEPA12 KEY CHECK 			
				KEY_STATE.ENCODE_DIRECTION = 1;		//RIGHT
			else
				KEY_STATE.ENCODE_DIRECTION = 0;		//LEFT
	}*/
}

void AT91F_Configure_Timer(void)
{
	AT91S_AIC	*pAIC;

	pAIC = AT91C_BASE_AIC;
	
 	AT91F_PITC_CfgPMC();	

	//Interrupt Handler
	AT91F_AIC_ConfigureIt(
		AT91C_BASE_AIC, 
		AT91C_ID_SYS, 
		AT91C_AIC_PRIOR_HIGHEST, 
		AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED, 
		AT91F_PIT_ASM_Handler);

	// Interrupt Enable
	AT91F_AIC_EnableIt(AT91C_BASE_AIC, AT91C_ID_SYS); 

 	// Interrupt Clear
	AT91F_AIC_ClearIt(AT91C_BASE_AIC, AT91C_ID_SYS);
	pAIC->AIC_EOICR = 1;  // end of interrupt 
 

 
	//PIT Interrupt Init - base time microseconds
	//AT91F_PITInit(AT91C_BASE_PITC, 10000, 40);
	AT91F_PITInit(AT91C_BASE_PITC, 10000, 10);		// 1ms
	//AT91F_PITInit(AT91C_BASE_PITC, 10000, 100);	// 10ms
	AT91F_PITEnableInt(AT91C_BASE_PITC);
}

void AT91F_Configure_GPIO (void)
{
	AT91S_AIC	*pAIC;

	pAIC = AT91C_BASE_AIC;	
	AT91F_PIOA_CfgPMC();

	// GPIO Enable
	AT91F_PIO_Enable(AT91C_BASE_PIOA, (AT91C_PIO_PA14 | AT91C_PIO_PA15 | AT91C_PIO_PA26 | AT91C_PIO_PA27 | AT91C_PIO_PA28| AT91C_PIO_PA29));
	AT91F_PIO_Enable(AT91C_BASE_PIOA, (AT91C_PIO_PA27 | AT91C_PIO_PA26 | AT91C_PIO_PA17 | AT91C_PIO_PA16 | AT91C_PIO_PA15));
	AT91F_PIO_Enable(AT91C_BASE_PIOA, (AT91C_PIO_PA18 | AT91C_PIO_PA19 | AT91C_PIO_PA20 | AT91C_PIO_PA21 | AT91C_PIO_PA26 | AT91C_PIO_PA27 | AT91C_PIO_PA28 | AT91C_PIO_PA29));
	AT91F_PIO_Enable(AT91C_BASE_PIOA, (AT91C_PIO_PA23 | AT91C_PIO_PA24));
	AT91F_PIO_Enable(AT91C_BASE_PIOA, (AT91C_PIO_PA5 | AT91C_PIO_PA12));
	AT91F_PIO_Enable(AT91C_BASE_PIOA, (AT91C_PIO_PA30 | AT91C_PIO_PA31));

	// GPIO Init
	AT91F_PIO_CfgPullup(AT91C_BASE_PIOA, (AT91C_PIO_PA14 | AT91C_PIO_PA15 | AT91C_PIO_PA26 | AT91C_PIO_PA27 | AT91C_PIO_PA28| AT91C_PIO_PA29));
	AT91F_PIO_CfgPullup(AT91C_BASE_PIOA, (AT91C_PIO_PA27 | AT91C_PIO_PA26 | AT91C_PIO_PA17 | AT91C_PIO_PA16 | AT91C_PIO_PA15));
	AT91F_PIO_CfgPullup(AT91C_BASE_PIOA, (AT91C_PIO_PA18 | AT91C_PIO_PA19 | AT91C_PIO_PA20 | AT91C_PIO_PA21 | AT91C_PIO_PA26 | AT91C_PIO_PA27 | AT91C_PIO_PA28 | AT91C_PIO_PA29));
	AT91F_PIO_CfgPullup(AT91C_BASE_PIOA, (AT91C_PIO_PA5 | AT91C_PIO_PA12));
	AT91F_PIO_CfgPullup(AT91C_BASE_PIOA, (AT91C_PIO_PA30 | AT91C_PIO_PA31));
	
	AT91F_PIO_CfgInput(AT91C_BASE_PIOA, (AT91C_PIO_PA23 | AT91C_PIO_PA24));
	AT91F_PIO_CfgInput(AT91C_BASE_PIOA, (AT91C_PIO_PA5 | AT91C_PIO_PA12));

	AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA14 | AT91C_PIO_PA15 | AT91C_PIO_PA26 | AT91C_PIO_PA27 | AT91C_PIO_PA28| AT91C_PIO_PA29));
	AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA27 | AT91C_PIO_PA26 | AT91C_PIO_PA17 | AT91C_PIO_PA16 | AT91C_PIO_PA15));
	AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA18 | AT91C_PIO_PA19 | AT91C_PIO_PA20 | AT91C_PIO_PA21 | AT91C_PIO_PA26 | AT91C_PIO_PA27 | AT91C_PIO_PA28 | AT91C_PIO_PA29));
	AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA30 | AT91C_PIO_PA31));

	AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA14 | AT91C_PIO_PA15 | AT91C_PIO_PA26 | AT91C_PIO_PA27 | AT91C_PIO_PA28| AT91C_PIO_PA29));
	AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA27 | AT91C_PIO_PA26 | AT91C_PIO_PA17 | AT91C_PIO_PA16 | AT91C_PIO_PA15));
	AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA18 | AT91C_PIO_PA19 | AT91C_PIO_PA20 | AT91C_PIO_PA21 | AT91C_PIO_PA26 | AT91C_PIO_PA27 | AT91C_PIO_PA28 | AT91C_PIO_PA29));
	AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, (AT91C_PIO_PA30 | AT91C_PIO_PA31));

	/*AT91F_PIO_InterruptEnable(AT91C_BASE_PIOA, (AT91C_PIO_PA5));
	AT91F_PIO_InputFilterEnable(AT91C_BASE_PIOA, (AT91C_PIO_PA5));
	
	//Initialize Interrupt Controller
	AT91F_AIC_ConfigureIt (
		AT91C_BASE_AIC,                        // AIC base address
		AT91C_ID_PIOA,                         // System peripheral ID
		AT91C_AIC_PRIOR_HIGHEST,               // Max priority
		AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE,
		AT91F_GPIO_ASM_Handler );

	AT91F_PIO_GetInterruptStatus(AT91C_BASE_PIOA);

	AT91F_AIC_EnableIt(AT91C_BASE_AIC, AT91C_ID_PIOA);

	pAIC->AIC_EOICR = 1;  // end of interrupt */
}

static void AT91F_LCDC_TFT_CfgPIO (void)
{
#if 0
  AT91_SYS->PIOB_PDR = (AT91C_PB1_LCDHSYNC | AT91C_PB2_LCDDOTCK | AT91C_PB3_LCDDEN | AT91C_PB4_LCDCC |
			AT91C_PB7_LCDD2 | AT91C_PB8_LCDD3 | AT91C_PB9_LCDD4 |
			AT91C_PB10_LCDD5 | AT91C_PB11_LCDD6 | AT91C_PB12_LCDD7 |
			AT91C_PB15_LCDD10 | AT91C_PB16_LCDD11 | AT91C_PB17_LCDD12 |
			AT91C_PB18_LCDD13 | AT91C_PB19_LCDD14 | AT91C_PB20_LCDD15 |
			AT91C_PB23_LCDD18 | AT91C_PB24_LCDD19 | AT91C_PB25_LCDD20 |
			AT91C_PB26_LCDD21 | AT91C_PB27_LCDD22 | AT91C_PB28_LCDD23);

  AT91_SYS->PIOB_BSR = (AT91C_PB23_LCDD18 | AT91C_PB24_LCDD19 | AT91C_PB25_LCDD20 |
			AT91C_PB26_LCDD21 | AT91C_PB27_LCDD22 | AT91C_PB28_LCDD23);
#else
	AT91_SYS->PIOB_PDR = (AT91C_PB0_LCDVSYNC | AT91C_PB1_LCDHSYNC | AT91C_PB2_LCDDOTCK | AT91C_PB3_LCDDEN | AT91C_PB4_LCDCC |
			AT91C_PB5_LCDD0 | AT91C_PB6_LCDD1 |
			AT91C_PB7_LCDD2 | AT91C_PB8_LCDD3 | AT91C_PB9_LCDD4 |
			AT91C_PB10_LCDD5 | AT91C_PB11_LCDD6 | AT91C_PB12_LCDD7 |
			AT91C_PB13_LCDD8 | AT91C_PB14_LCDD9 |
			AT91C_PB15_LCDD10 | AT91C_PB16_LCDD11 | AT91C_PB17_LCDD12 |
			AT91C_PB18_LCDD13 | AT91C_PB19_LCDD14 | AT91C_PB20_LCDD15 |
			AT91C_PB21_LCDD16 | AT91C_PB22_LCDD17 |
			AT91C_PB23_LCDD18 | AT91C_PB24_LCDD19 | AT91C_PB25_LCDD20 |
			AT91C_PB26_LCDD21 | AT91C_PB27_LCDD22 | AT91C_PB28_LCDD23);

//  AT91_SYS->PIOB_BSR = (AT91C_PB3_LCDDEN | AT91C_PB21_LCDD16 | AT91C_PB22_LCDD17 | AT91C_PB23_LCDD18 | AT91C_PB24_LCDD19 | AT91C_PB25_LCDD20 |
//			AT91C_PB26_LCDD21 | AT91C_PB27_LCDD22 | AT91C_PB28_LCDD23);
  AT91_SYS->PIOB_BSR = (AT91C_PB21_LCDD16 | AT91C_PB22_LCDD17 | AT91C_PB23_LCDD18 | AT91C_PB24_LCDD19 | AT91C_PB25_LCDD20 |
			AT91C_PB26_LCDD21 | AT91C_PB27_LCDD22 | AT91C_PB28_LCDD23);
#endif
}
 
static void lcd_init_lut(void)
{
	unsigned int i;
	for(i =0; i <256; i++) {
		panel_info.controller.lcdc->LCDC_LUT_ENTRY[i] = 
		   ((rgbPalette[i].red) >> 3 |
		   ((rgbPalette[i].green & 0xf8) << 2) |
		   ((rgbPalette[i].blue & 0xf8) << 7));
	}
}

void lcd_ctrl_init (void *lcdbase)
{

	unsigned long value;

	AT91F_LCDC_TFT_CfgPIO();

	/* Enable HCLOCK in PMC */
	AT91_SYS->PMC_SCER = AT91C_PMC_HCK1;

	/* Init controller field in panel_info */
	panel_info.controller.lcdc = (AT91PS_LCDC) AT91C_BASE_LCDC;
	panel_info.controller.frame_buffer = (unsigned long) lcdbase;

	/* Turn off the LCD controller and the DMA controller */
	panel_info.controller.lcdc->LCDC_PWRCON = 0x0C;
	panel_info.controller.lcdc->LCDC_DMACON = 0;
	
	// Reset LCDC DMA
	panel_info.controller.lcdc->LCDC_DMACON = AT91C_LCDC_DMARST;

	/* ...set frame size and burst length = 8 words (?) */
//#if 0	
	value = ((unsigned int)panel_info.vl_row * (unsigned int)panel_info.vl_col * (unsigned int)NBITS(panel_info.vl_bpix)) / 32;
//#else // km192 - 20070425	
//	value = ((unsigned int)panel_info.vl_row * (unsigned int)panel_info.vl_col * 24) / 32;
	//value |= ((SIDSAFB_DMA_BURST_LEN - 1) << 24);
//#endif	

	panel_info.controller.lcdc->LCDC_FRMCFG = value;

	/* Set pixel clock */
	value = AT91C_MASTER_CLOCK_FOR_I2S / panel_info.vl_pixclock;

	if (AT91C_MASTER_CLOCK_FOR_I2S % panel_info.vl_pixclock)
	  value++;

	value = (value / 2) - 1;

	if (!value) {
		panel_info.controller.lcdc->LCDC_LCDCON1 = AT91C_LCDC_BYPASS;
	} else
		panel_info.controller.lcdc->LCDC_LCDCON1 = value << 12;

	/* Initialize control register 2 */
	if (panel_info.vl_tft)
	  value = ( AT91C_LCDC_MEMOR_LITTLEIND | AT91C_LCDC_DISTYPE_TFT | AT91C_LCDC_CLKMOD);
	else
	  value = ( AT91C_LCDC_MEMOR_LITTLEIND | AT91C_LCDC_CLKMOD);

	if (!((unsigned int)panel_info.vl_hsp & FB_SYNC_HOR_HIGH_ACT))
		value |= 1 << 10;	/* INVLINE */
	if (!((unsigned int)panel_info.vl_vsp & FB_SYNC_VERT_HIGH_ACT))
		value |= 1 << 9;	/* INVFRAME */

	value |= (unsigned int)(panel_info.vl_bpix << 5);	
	
	panel_info.controller.lcdc->LCDC_LCDCON2 = value;
//	lcd_init_lut();	

	/* Vertical timing */
	value = (unsigned int)((panel_info.vl_vpw - 1) << 16);
	value |= (unsigned int)(panel_info.vl_bfw << 8);
	value |= (unsigned int)panel_info.vl_efw;
	panel_info.controller.lcdc->LCDC_TIM1 = value;

	/* Horizontal timing */
	value = (unsigned int)((panel_info.vl_elw - 1) << 21);
	value |= (unsigned int)((panel_info.vl_hpw - 1) << 8);
	value |= (unsigned int)(panel_info.vl_blw - 1);

	panel_info.controller.lcdc->LCDC_TIM2 = value;

	value = (unsigned int)((panel_info.vl_col - 1) << 21);
	value |= (unsigned int)(panel_info.vl_row - 1);

	panel_info.controller.lcdc->LCDC_LCDFRCFG = value;

	/* FIFO Threshold: Use formula from data sheet */
	value = SIDSAFB_FIFO_SIZE - (2 * SIDSAFB_DMA_BURST_LEN + 3);
	//value = 512;
	panel_info.controller.lcdc->LCDC_FIFO = value;

	/* Toggle LCD_MODE every frame */
	value = 0;
	panel_info.controller.lcdc->LCDC_MVAL = value;

	/* Disable all interrupts */
	panel_info.controller.lcdc->LCDC_IDR = ~0UL;
	// Set contrast
	value = AT91C_LCDC_PS_DIVIDEDBYEIGHT | AT91C_LCDC_POL_POSITIVEPULSE | AT91C_LCDC_ENA_PWMGEMENABLED;
	panel_info.controller.lcdc->LCDC_CTRSTCON = value;
	panel_info.controller.lcdc->LCDC_CTRSTVAL = 0xDA; 


	panel_info.controller.lcdc->LCDC_BA1 = (unsigned int)lcdbase;
	//panel_info.controller.lcdc->LCDC_FRMCFG = (15 << 24) + ((unsigned int)panel_info.vl_col * (unsigned int)panel_info.vl_row * (unsigned int)NBITS(panel_info.vl_bpix)) / 32;
	panel_info.controller.lcdc->LCDC_DMACON    = AT91C_LCDC_DMAEN;
	panel_info.controller.lcdc->LCDC_PWRCON    = AT91C_LCDC_PWR | 0x0c;
}

void TestAction(void)
{
	nand_image_display(0, 0, 480, 235, AnimationBuffer+(BACK_SIZE*animation_frame), 0);
	if(animation_frame >= 29)
		animation_frame=0;	
	else
	animation_frame+=1;

	switch(TestDisplay)
	{
	case 10:
		nand_image_display(0, 0, 480, 272, SDRAM_00_course_a_dumy_480x272, 0);
		break;
	case 11:
		nand_image_display(0, 0, 480, 234, SDRAM_00_course_bg_480x234, 0);
		break;
	case 12:
		nand_image_display(0, 0, 329, 83, SDRAM_00_course_eng_all_329x83, 0);
		break;
	case 13:
		nand_image_display(0, 0, 326, 71, SDRAM_00_course_eng_bul_326x71, 0);
		break;
	case 14:
		nand_image_display(0, 0, 419, 71, SDRAM_00_course_eng_cot_419x71, 0);
		break;
	case 15:
		nand_image_display(0, 0, 253, 71, SDRAM_00_course_eng_del_253x71, 0);
		break;
	case 16:
		nand_image_display(0, 0, 459, 71, SDRAM_00_course_eng_han_459x71, 0);
		break;
	case 17:
		nand_image_display(0, 0, 312, 71, SDRAM_00_course_eng_per_312x71, 0);
		break;
	case 18:
		nand_image_display(0, 0, 244, 71, SDRAM_00_course_eng_san_244x71, 0);
		break;
	case 19:
		nand_image_display(0, 0, 308, 71, SDRAM_00_course_eng_spe_308_71, 0);
		break;
	case 20:
		nand_image_display(0, 0, 364, 71, SDRAM_00_course_eng_ste_364x71, 0);
		break;
	case 21:
		nand_image_display(0, 0, 267, 71, SDRAM_00_course_eng_tub_267x71, 0);
		break;
	case 22:
		nand_image_display(0, 0, 395, 85, SDRAM_00_course_esp_all_395x85, 0);
		break;
	case 23:
		nand_image_display(0, 0, 442, 71, SDRAM_00_course_esp_bul_442x71, 0);
		break;
	case 24:
		nand_image_display(0, 0, 447, 71, SDRAM_00_course_esp_cot_447x71, 0);
		break;
	case 25:
		nand_image_display(0, 0, 228, 71, SDRAM_00_course_esp_del_228x71, 0);
		break;
	case 26:
		nand_image_display(0, 0, 441, 71, SDRAM_00_course_esp_han_441x71, 0);
		break;
	case 27:
		nand_image_display(0, 0, 377, 71, SDRAM_00_course_esp_per_377x71, 0);
		break;
	case 28:
		nand_image_display(0, 0, 254, 71, SDRAM_00_course_esp_san_254x71, 0);
		break;
	case 29:
		nand_image_display(0, 0, 334, 71, SDRAM_00_course_esp_spe_334x71, 0);
		break;
	case 30:
		nand_image_display(0, 0, 460, 71, SDRAM_00_course_esp_ste_460x71, 0);
		break;
	case 31:
		nand_image_display(0, 0, 397, 71, SDRAM_00_course_esp_tub_397x71, 0);
		break;
	case 32:
		nand_image_display(0, 0, 417, 85, SDRAM_00_course_fra_all_417x85, 0);
		break;
	case 33:
		nand_image_display(0, 0, 275, 71, SDRAM_00_course_fra_bul_275x71, 0);
		break;
	case 34:
		nand_image_display(0, 0, 387, 71, SDRAM_00_course_fra_cot_387x71, 0);
		break;
	case 35:
		nand_image_display(0, 0, 223, 71, SDRAM_00_course_fra_del_223x71, 0);
		break;
	case 36:
		nand_image_display(0, 0, 424, 71, SDRAM_00_course_fra_han_424_71, 0);
		break;
	case 37:
		nand_image_display(0, 0, 341, 71, SDRAM_00_course_fra_per_341x71, 0);
		break;
	case 38:
		nand_image_display(0, 0, 254, 71, SDRAM_00_course_fra_san_254x71, 0);
		break;
	case 39:
		nand_image_display(0, 0, 368, 71, SDRAM_00_course_fra_spe_368x71, 0);
		break;
	case 40:
		nand_image_display(0, 0, 419, 71, SDRAM_00_course_fra_ste_419x71, 0);
		break;
	case 41:
		nand_image_display(0, 0, 279, 71, SDRAM_00_course_fra_tub_279x71, 0);
		break;
	case 42:
		nand_image_display(0, 0, 480, 272, SDRAM_00_select_eng_480x272, 0);
		break;
	case 43:
		nand_image_display(0, 0, 295, 125, SDRAM_00_select_fre_295x125, 0);
		break;
	case 44:
		nand_image_display(0, 0, 295, 125, SDRAM_00_select_spa_295x125, 0);
		break;
	case 45:
		nand_image_display(0, 0, 165, 29, SDRAM_01_Detail_1_eng_col_s_165x29, 0);
		break;
	case 46:
		nand_image_display(0, 0, 165, 29, SDRAM_01_Detail_1_eng_ext_s_165x29, 0);	
		break;
	case 47:
		nand_image_display(0, 0, 165, 29, SDRAM_01_Detail_1_eng_hot_s_165x29, 0);
		break;
	case 48:
		nand_image_display(0, 0, 165, 29, SDRAM_01_Detail_1_eng_war_2_s_165x29, 0);		
		break;
	case 49:
		nand_image_display(0, 0, 165, 29, SDRAM_01_Detail_1_eng_war_s_165x29, 0);
		break;
	case 50:
		nand_image_display(0, 0, 127, 29, SDRAM_01_Detail_2_eng_ext_s_127x29, 0);	
		break;
	case 51:
		nand_image_display(0, 0, 127, 29, SDRAM_01_Detail_2_eng_hig_s_127x29, 0);	
		break;
	case 52:
		nand_image_display(0, 0, 127, 29, SDRAM_01_Detail_2_eng_low_s_127x29, 0);
		break;
	case 53:
		nand_image_display(0, 0, 127, 29, SDRAM_01_Detail_2_eng_mei_s_127x29, 0);
		break;
	case 54:
		nand_image_display(0, 0, 127, 29, SDRAM_01_Detail_2_eng_nos_s_127x29, 0);
		break;
	case 55:
		nand_image_display(0, 0, 97, 29, SDRAM_01_Detail_3_eng_hea_s_97x29, 0);
		break;
	case 56:
		nand_image_display(0, 0, 97, 29, SDRAM_01_Detail_3_eng_lig_s_97x29, 0);
		break;
	case 57:
		nand_image_display(0, 0, 97, 29, SDRAM_01_Detail_3_eng_nor_s_97x29, 0);
		break;
	case 58:
		nand_image_display(0, 0, 83, 29, SDRAM_01_Detail_4_eng_hig_s_83x29, 0);
		break;
	case 59:
		nand_image_display(0, 0, 83, 29, SDRAM_01_Detail_4_eng_low_s_83x29, 0);	
		break;
	case 60:
		nand_image_display(0, 0, 83, 29, SDRAM_01_Detail_4_eng_off_s_83x29, 0);
		break;
	case 61:
		nand_image_display(0, 0, 480, 39, SDRAM_01_Detail_bg_480x39, 0);
		break;
	case 62:
		nand_image_display(0, 0, 13, 15, SDRAM_01_Detail_che_13x15, 0);
		break;

	case 63:
		nand_image_display(0, 0, 114, 22, SDRAM_05_Error_3_04_fra_cod_114x22, 0);
		break;
	case 64:
		nand_image_display(0, 0, 254, 24, SDRAM_05_Error_4_07_spa_2_254x24, 0);
		break;
	case 65:
		nand_image_display(0, 0, 325, 52, SDRAM_07_end_spa_325x52, 0);
		break;
	case 66:
		nand_image_display(0, 0, 221, 77, SDRAM_08_ani_rinse_fra_25_221x77, 0);
		break;
	case 67:
		nand_image_display(0, 0, 221, 77, SDRAM_08_ani_rinse_fra_30_221x77, 0);
		break;
	case 68:
		nand_image_display(0, 0, 259, 71, SDRAM_08_ani_spin_fra_30_259x71, 0);	
		break;
	case 69:
		nand_image_display(0, 0, 366, 71, SDRAM_08_ani_refresh_fra_30_366x71, 0);
		break;
	case 70:
		nand_image_display(0, 0, 470, 71, SDRAM_08_ani_refresh_esp_30_470x71, 0);
		break;
	case 71:
		nand_image_display(0, 0, 280, 71, SDRAM_08_ani_tub_fra_30_280x71, 0);
		break;
	case 72:
		nand_image_display(0, 0, 250, 71, SDRAM_08_ani_wash_fra_30_250x71, 0);
		break;
	case 73:
		nand_image_display(0, 0, 411, 41, SDRAM_09_set_tub_set_fra_02_411x41, 0);
		break;

	default:
		break;
	}	
	TestScreenNumber();
}

void TestScreenNumber(void)
{
	unsigned int digit100=0;
	unsigned int digit10=0;
	unsigned int digit1=0;

	digit100=TestDisplay/100;
	digit10=(TestDisplay%100)/10;
	digit1=(TestDisplay%100)%10;
	
	switch(digit100)
	{
		case 0:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_0_15x26, 0);	
			break;
		case 1:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_1_15x26, 0);	
			break;
		case 2:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_2_15x26, 0);	
			break;
		case 3:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_3_15x26, 0);	
			break;
		case 4:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_4_15x26, 0);	
			break;
		case 5:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_5_15x26, 0);	
			break;
		case 6:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_6_15x26, 0);	
			break;
		case 7:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_7_15x26, 0);	
			break;
		case 8:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_8_15x26, 0);	
			break;
		case 9:
			nand_image_display(400, 240, 15, 26, SDRAM_01_Timelft_9_15x26, 0);	
			break;
		default:
			break;
	}
	switch(digit10)
	{
		case 0:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_0_15x26, 0);	
			break;
		case 1:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_1_15x26, 0);	
			break;
		case 2:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_2_15x26, 0);	
			break;
		case 3:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_3_15x26, 0);	
			break;
		case 4:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_4_15x26, 0);	
			break;
		case 5:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_5_15x26, 0);	
			break;
		case 6:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_6_15x26, 0);	
			break;
		case 7:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_7_15x26, 0);	
			break;
		case 8:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_8_15x26, 0);	
			break;
		case 9:
			nand_image_display(420, 240, 15, 26, SDRAM_01_Timelft_9_15x26, 0);	
			break;
		default:
			break;
	}
	switch(digit1)
	{
		case 0:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_0_15x26, 1);	
			break;
		case 1:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_1_15x26, 1);	
			break;
		case 2:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_2_15x26, 1);	
			break;
		case 3:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_3_15x26, 1);	
			break;
		case 4:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_4_15x26, 1);	
			break;
		case 5:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_5_15x26, 1);	
			break;
		case 6:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_6_15x26, 1);	
			break;
		case 7:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_7_15x26, 1);	
			break;
		case 8:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_8_15x26, 1);	
			break;
		case 9:
			nand_image_display(440, 240, 15, 26, SDRAM_01_Timelft_9_15x26, 1);	
			break;
		default:
			break;
	}
}