#include <types.h>
#include <console.h>
#include <string.h>

EXTERN UINT32 CLIBstrlen(CONST CHAR *a) ;
EXTERN INT32 CLIBstrcmp(CONST CHAR *a, CONST CHAR *b);
EXTERN CHAR *CLIBstrtok(CHAR *s1, CONST CHAR *s2);

struct command {
  CHAR name[20];
  CHAR alias[10];
  CHAR help_args[40];
  CHAR help_text[100];
  VOID (*action_rtn)(VOID);
};

VOID print_version(VOID);
VOID memorydump(VOID);
VOID memorymodify(VOID);
VOID flash_operation( VOID );

EXTERN VOID ldownload(VOID);
EXTERN VOID xdownload(VOID);
EXTERN VOID zdownload(VOID);
EXTERN VOID rdownload(VOID);
EXTERN VOID gdownload(VOID);
EXTERN VOID run_JBOSN(VOID);

//CHAR history[10][128];


STATIC struct command commands[] = {
	{"ver", "v",
	" ",
	"version", //     "Print Board & BootManager & OS version", 
	&print_version  },
#ifdef USE_NET
	{"ldownload", "ldn",
	"<Address>",
	"download memory content by network (TFTP)", 
	&ldownload},                       //ConsoleMemoryListAction
#endif
#ifdef USE_ZMODEM
	{"zdownload", "zdn",
	"<Address>",
	"download memory content by serial (ZMODEM)", 
	&zdownload},                       //ConsoleMemoryListAction
#endif

#ifdef USE_XMODEM
	{"xdownload", "xdn",
	"<Address>",
	"download memory content by serial (XMODEM)", 
	&xdownload},                       //ConsoleMemoryListAction
#endif

	{"memorydump", "md",
	"<Addr> [Len] (HEX)", //"<Address> [length]",
	"dump memory contents", 
	&memorydump},                       //ConsoleMemoryListAction

	{"memorymodify", "mm",
	"<Addr> [Data]",//"<Address> [data]",
	"Modify memory content", 
	&memorymodify},                         //ConsoleMemoryModifyAction
#ifdef USE_FLASH_NOR
	{"flash_operation", "flash",
	"[operation] [Sblock] [Eblock] (DEC)",
	"Block size : 256KB",//"Flash", 
	&flash_operation},
#endif
	{"rdownload", "rdn",
	"<addr>",
	"download memory content from rom", 
	&rdownload},

#ifdef USE_GDN
	{"gdownload", "gdn",
	"<addr>",
	"download memory content from gziped rom image", 
	&gdownload},
#endif

	{"branch", "bl",
	"<addr>",
	"branch to image", 
	&run_JBOSN},

};

#define NUM_FMU_COMMANDS (sizeof(commands) / sizeof(struct command))

#define MAX_TOKENS 10
CHAR *tokens[MAX_TOKENS];
INT32 ntokens = 0;

INT32	E_ston;

INT32 atoh_one(CHAR inchar)
{
	if ( (inchar>='0') && (inchar<='9') )		return inchar-'0';
	else if ( (inchar>='a') && (inchar<='f') )	return inchar-'a'+10;
	else if ( (inchar>='A') && (inchar<='F') )	return inchar-'A'+10;
	else{ E_ston = TRUE;	return -1;}

}
INT32 stoh(CHAR *instr)
{
	INT32	rv=0;
	INT32	dw1;
	
	while(*instr)
	{
		dw1= (INT32)atoh_one(*instr++);
		if(dw1 < 0)
			return -1;
		rv=rv*16+dw1;
	}
	
	return rv;

}

INT32 atod_one(CHAR inchar)
{
	if ( (inchar>='0') && (inchar<='9') )	return inchar-'0';
	else{ E_ston = TRUE;	return -1;}
	
}

INT32 stod(CHAR *instr)
{
	INT32	rv=0;
	INT32	dw1;
	
	while(*instr)
	{
		dw1 = (INT32)atod_one(*instr++);
		if(dw1 < 0)
			return -1;
			
		rv=rv*10+dw1;
	}
	
	return rv;

}

INT32	ston(CHAR *s)
{
	INT32	dw1;
	
	if( s[0] == '0')
		if( (s[1] == 'x') || (s[1] == 'X') )
		{
			E_ston = FALSE;
			dw1 = stoh(s+2);
			return dw1;
		}
		
	E_ston = FALSE;
	dw1 = stod(s);
	return dw1;
}


CHAR *__PULL()
{
	CHAR *value = NULL;
	
	if (ntokens > 0) 
	{
		INT32 i;
	
		value = tokens[0];
		for (i = 1; i < ntokens; i++) 
			tokens[i-1] = tokens[i];

		ntokens--;
	}
	return value;
}

#define __ADD(value) tokens[ntokens++] = value


STATIC INT32 partial_compare(CHAR *token, CHAR *command) 
{
	INT32 len ;
	INT32 i ;
	
	if (CLIBstrlen(command) < CLIBstrlen(token)) 
		return FALSE ;
		
	len = CLIBstrlen(token) ;
	
	for (i = 0 ; i < len ; i++)  
		if (*token++ != *command++) return FALSE ;
	
	return TRUE ;
	
}


STATIC VOID parse_command(VOID)
{
	CHAR *token;
	INT32 i;
	INT32 found = 0;
	INT32 index = -1 ;
	
	/* now work out what the command was from the first token */
	token = __PULL();

	if (token != NULL) 
	{
		/* first, look for an absolute match */
		for (i = 0; i < NUM_FMU_COMMANDS; i++) 
		{
			if ( !CLIBstrcmp(token, commands[i].name) || !CLIBstrcmp(token, commands[i].alias) ) 
			{
				(commands[i].action_rtn)();
				return ;
			}
		}
		/* now, look for a partial absolute match */
		for (i = 0; i < NUM_FMU_COMMANDS; i++) 
		{
			if (partial_compare(token, commands[i].name) == TRUE ||
				partial_compare(token, commands[i].alias) == TRUE) 
			{
				found++ ;
				index = i ;
			}
		}
		if (found == 0) 
			console_printf("\n command not found\n");
		else if (found > 1) 
			console_printf("\n wrong command\n");
		else 
			(commands[index].action_rtn)();
	}
	
	return;
}

STATIC VOID do_command(CHAR *command)
{
    CHAR *token;

    /* Grab all of the command tokens */
    ntokens = 0;
    token = CLIBstrtok(command, " \t");

    while(token != NULL) 
    {
		/* treat ';' as end of line, new command */
		if (CLIBstrcmp(token, ";") == 0) 
		{
			parse_command();
			ntokens = 0;
		}
		else
		{
			__ADD(token);
		}
		token = CLIBstrtok(NULL, " \t");
    }
    /* now that we've found it, parse it. */
    parse_command();

}


VOID PrintHelp(VOID)
{
	INT32 index;

	console_printf("===========================================================================\n");

	for( index = 0; index < NUM_FMU_COMMANDS; index++)
	{
		console_printf("%s :",commands[index].name );
		console_printf(" %s",commands[index].alias );
		console_printf(" %s",commands[index].help_args );
		console_printf(" %s\n",commands[index].help_text );
	}

	console_printf("===========================================================================\n");

}
//#define _FOR_DEV_
EXTERN INT32 getcommand(CHAR *);
//CHAR history[10][128];
VOID do_console()
{
	INT32 ret;
    INT32   i = 0;

#ifdef _FOR_DEV_

#define WAIT_LOOP   0xFF00000

    do
    {
        INT32   Ret;

        Ret = ser_getc();

        if( (Ret == 'p')||(Ret == 'p') )
            break;

        if( ++i > WAIT_LOOP )
            break;
    }while(1);

    if( i  > WAIT_LOOP)
	{
	    do_command("r");
	    do_command("b");
    }

	console_printf("\n");

	do
	{
		CHAR command[128];

		PrintHelp();
		console_printf("Command> ");

		ret = getcommand(command);
	
		if(ret == 0)
			continue;

		if(command[0] == 'q')
			break;		
		do_command(command);
	}while(1);

#else

    do_command("r");
    do_command("b");

#endif

	return;
}


