#include <types.h>
#include <console.h>

#include <macros.h>

#ifdef USE_FLASH_NOR

extern CHAR *__PULL();
extern INT32	E_ston;

EXTERN INT32 ston(CHAR *s);
EXTERN BYTE CLIBisprintable(BYTE ch);


/*
 *******************************************************************
 * Flash Operation
 *******************************************************************
 */
#define FLASH_BASE_ADDR	(0x00000000)
#define	STRATA_BLOCK_SIZE	(0x20000)
#define	STRATA_BLOCK_SIZE_32	(0x40000)

#define	STRATA_CMD_CLEAR_GSTATUS	0x5050
#define	STRATA_CMD_READ_GSTATUS		0x7070
#define	STRATA_CMD_WRITE_WORD		0x4040	// Tell flash to program
#define	STRATA_CMD_READ_ARRAY		0xFFFF	// Read(normal) mode
#define	STRATA_CMD_BLOCK_ERASE1		0x2020	// Start erase
#define	STRATA_CMD_BLOCK_ERASE2		0xD0D0	// Confirm erase
#define	STRATA_SR_READY			0x0080	// Ready for use status

#define	STRATA_CMD_CLEAR_GSTATUS_32	0x50505050
#define	STRATA_CMD_READ_GSTATUS_32	0x70707070
#define	STRATA_CMD_WRITE_WORD_32	0x40404040	// Tell flash to program
#define	STRATA_CMD_READ_ARRAY_32	0xFFFFFFFF	// Read(normal) mode
#define	STRATA_CMD_BLOCK_ERASE1_32	0x20202020	// Start erase
#define	STRATA_CMD_BLOCK_ERASE2_32	0xD0D0D0D0	// Confirm erase
#define	STRATA_SR_READY_32		0x00800080	// Ready for use status

#define STRATA_CMD_READ_IDENTIFIER	0x9090
#define STRATA_CMD_LOCK_0		0x6060
#define STRATA_CMD_LOCK_1		0x0101
#define STRATA_CMD_UNLOCK_0		0x6060
#define STRATA_CMD_UNLOCK_1		0xD0D0

#define STRATA_CMD_READ_IDENTIFIER_32	0x90909090
#define STRATA_CMD_LOCK_0_32		0x60606060
#define STRATA_CMD_LOCK_1_32		0x01010101
#define STRATA_CMD_UNLOCK_0_32		0x60606060
#define STRATA_CMD_UNLOCK_1_32		0xD0D0D0D0

#define	FLASH_CLEAR_GSTATUS(dwBlockAddr)	WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_CLEAR_GSTATUS)
#define	FLASH_READ_GSTATUS(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_READ_GSTATUS)
#define	FLASH_WRITE_WORD(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_WRITE_WORD)
#define	FLASH_READ_ARRAY(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_READ_ARRAY)
#define	FLASH_BLOCK_ERASE1(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_BLOCK_ERASE1)
#define	FLASH_BLOCK_ERASE2(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_BLOCK_ERASE2)
#define	FLASH_SR_READY(dwBlockAddr)		READ_REGISTER_USHORT(dwBlockAddr)

#define	FLASH_CLEAR_GSTATUS_32(dwBlockAddr)	WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_CLEAR_GSTATUS_32)
#define	FLASH_READ_GSTATUS_32(dwBlockAddr)	WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_READ_GSTATUS_32)
#define	FLASH_WRITE_WORD_32(dwBlockAddr)	WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_WRITE_WORD_32)
#define	FLASH_READ_ARRAY_32(dwBlockAddr)	WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_READ_ARRAY_32)
#define	FLASH_BLOCK_ERASE1_32(dwBlockAddr)	WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_BLOCK_ERASE1_32)
#define	FLASH_BLOCK_ERASE2_32(dwBlockAddr)	WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_BLOCK_ERASE2_32)
#define	FLASH_SR_READY_32(dwBlockAddr)		READ_REGISTER_ULONG(dwBlockAddr)

#define	FLASH_READ_IDENTIFIER(dwBlockAddr)	WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_READ_IDENTIFIER)
#define	FLASH_LOCK_0(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_LOCK_0)
#define	FLASH_LOCK_1(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_LOCK_1)
#define	FLASH_UNLOCK_0(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_UNLOCK_0)
#define	FLASH_UNLOCK_1(dwBlockAddr)		WRITE_REGISTER_USHORT(dwBlockAddr,(UINT16)STRATA_CMD_UNLOCK_1)

#define	FLASH_READ_IDENTIFIER_32(dwBlockAddr)	WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_READ_IDENTIFIER_32)
#define	FLASH_LOCK_0_32(dwBlockAddr)		WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_LOCK_0_32)
#define	FLASH_LOCK_1_32(dwBlockAddr)		WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_LOCK_1_32)
#define	FLASH_UNLOCK_0_32(dwBlockAddr)		WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_UNLOCK_0_32)
#define	FLASH_UNLOCK_1_32(dwBlockAddr)		WRITE_REGISTER_ULONG(dwBlockAddr,(UINT32)STRATA_CMD_UNLOCK_1_32)


//=================================== 32 Ʈ ==========================================

static UINT32 flash_check_status_32( UINT32 dwBlockAddr)
{
	UINT32 iTimeOut = 0;
	
	while ( (FLASH_SR_READY_32(dwBlockAddr))!=STRATA_SR_READY_32 )
	{
		iTimeOut++;
		if( iTimeOut > 0xFFFFF)
		{
			console_printf("\n Timed out.\n");
			return FALSE;
		}	
	}
	
	return TRUE;
}

static UINT32 flash_erase_block_32(INT32 iBlock) //ULONG FlashEraseIBlock(ULONG dwBlock)
{
	UINT32	dwBlockAddr;

	console_printf(" %d :",iBlock);

	dwBlockAddr  =  FLASH_BASE_ADDR + (iBlock << 18);	// 256K / Block

	FLASH_CLEAR_GSTATUS_32(dwBlockAddr);

	// Get Global Status
	FLASH_READ_GSTATUS_32(dwBlockAddr);

	if( !flash_check_status_32(dwBlockAddr) )
		return 0;

	// Attempt to Erase the Block
	FLASH_BLOCK_ERASE1_32(dwBlockAddr);
	FLASH_BLOCK_ERASE2_32(dwBlockAddr);

	if( !flash_check_status_32(dwBlockAddr) )
		return 0;

	FLASH_READ_ARRAY_32(dwBlockAddr);	// Set Back to read array mode

//	if( !flash_check_status(dwBlockAddr) )
//		return 0;

	console_printf(" is erased.\n" );

	return 0;

}

static UINT32 flash_lock_block_32( INT32 iBlock)
{
	UINT32	dwBlockAddr;

	console_printf(" %d :",iBlock);

	dwBlockAddr  =  FLASH_BASE_ADDR + (iBlock << 18);	// 256K / Block

	FLASH_CLEAR_GSTATUS_32(dwBlockAddr);

	FLASH_LOCK_0_32(dwBlockAddr);
	FLASH_LOCK_1_32(dwBlockAddr);
	if( !flash_check_status_32(dwBlockAddr) )
		return 0;
	
	FLASH_READ_ARRAY_32(dwBlockAddr);	// Set Back to read array mode

	console_printf(" is locked.\n" );
	
	return 0;
}

static UINT32 flash_unlock_block_32( INT32 iBlock)
{
	UINT32	dwBlockAddr;

	console_printf(" %d :",iBlock);

	dwBlockAddr  =  FLASH_BASE_ADDR + (iBlock << 18);	// 256K / Block

	FLASH_CLEAR_GSTATUS_32(dwBlockAddr);

	FLASH_UNLOCK_0_32(dwBlockAddr);
	FLASH_UNLOCK_1_32(dwBlockAddr);
	if( !flash_check_status_32(dwBlockAddr) )
		return 0;
	
	FLASH_READ_ARRAY_32(dwBlockAddr);	// Set Back to read array mode
	
	console_printf(" is unlocked.\n" );

	return 0;
}

static UINT32 flash_program_block_32(UINT32 iBlock, CHAR *pData) 
{
	UINT32	dwBlockAddr;
	INT32 i;

    console_printf("pData = %x\n",(UINT32)pData);
	console_printf(" %d :",iBlock);

	// should be erased
	flash_erase_block_32(iBlock);
	
	dwBlockAddr  =  FLASH_BASE_ADDR + (iBlock << 18);	// 256K / Block
    console_printf("dwBlockAddr = %x\n",(UINT32)dwBlockAddr);

	FLASH_CLEAR_GSTATUS_32(dwBlockAddr);

	// Get Global Status
	FLASH_READ_GSTATUS_32(dwBlockAddr);

	if( !flash_check_status_32(dwBlockAddr) )
		return 0;


	for( i = 0; i < STRATA_BLOCK_SIZE_32; i+=4)
	{
		FLASH_WRITE_WORD_32(dwBlockAddr+i);						// Tell Flash to program..
		WRITE_REGISTER_ULONG(dwBlockAddr+i,*(UINT32*)(pData+i));					// Copy it to Flash
		if( !flash_check_status_32(dwBlockAddr+i) )
			return 0;
	}

	FLASH_READ_ARRAY_32(dwBlockAddr);	// Set Back to read array mode


	console_printf(" is programed.\n" );

	return 0;

}

UINT32 flash_lock_32( INT32 iStart, INT32 iEnd)
{
	for( ; iStart <= iEnd; iStart++)
		flash_lock_block_32(iStart);

	return 0;
}
UINT32 flash_unlock_32( INT32 iStart, INT32 iEnd)
{
	for( ; iStart <= iEnd; iStart++)
		flash_unlock_block_32(iStart);

	return 0;
}
	
UINT32 flash_erase_32( INT32 iStart, INT32 iEnd)
{
	for( ; iStart <= iEnd; iStart++)
		flash_erase_block_32(iStart);

	return 0;
}

UINT32 flash_program_32( UINT32 iTarget, UINT32 iSource, UINT32 iSize)
{
	UINT32 iStart, iEnd;
	
	if( iTarget + iSize > 0x2000000)	//16M Flash 
	{
		console_printf("The paramenter is wrong\n");
		return 1;
	}
	
	iStart = (iTarget / STRATA_BLOCK_SIZE_32);
	iEnd = (iTarget + iSize + STRATA_BLOCK_SIZE_32-1)/STRATA_BLOCK_SIZE_32;

	for( ; iStart < iEnd; iStart++, iSource += STRATA_BLOCK_SIZE_32)
		flash_program_block_32(iStart, (CHAR *)iSource);

	return 0;
}


//====================================== 16 Ʈ ==========================================

static UINT32 flash_check_status( UINT32 dwBlockAddr)
{
	UINT32 iTimeOut = 0;
	
	while ( (FLASH_SR_READY(dwBlockAddr))!=STRATA_SR_READY )
	{
		iTimeOut++;
		if( iTimeOut > 0xFFFFF)
		{
			console_printf("\n Timed out.\n");
			return FALSE;
		}	
	}
	
	return TRUE;
}

static UINT32 flash_erase_block(INT32 iBlock) //ULONG FlashEraseIBlock(ULONG dwBlock)
{
	UINT32	dwBlockAddr;

RETRY_ERASE:
	console_printf(" %D :",iBlock);

	dwBlockAddr  =  FLASH_BASE_ADDR + (iBlock << 17);	// 128K / Block

	FLASH_CLEAR_GSTATUS(dwBlockAddr);

	// Get Global Status
	FLASH_READ_GSTATUS(dwBlockAddr);

	if( !flash_check_status(dwBlockAddr) )
	{
		console_printf(" status read fail : retry ");
		goto RETRY_ERASE;
		return 0;
	}

	// Attempt to Erase the Block
	FLASH_BLOCK_ERASE1(dwBlockAddr);
	FLASH_BLOCK_ERASE2(dwBlockAddr);

	if( !flash_check_status(dwBlockAddr) )
	{
		console_printf(" erase fail : retry ");
		goto RETRY_ERASE;
		return 0;
	}

	FLASH_READ_ARRAY(dwBlockAddr);	// Set Back to read array mode

//	if( !flash_check_status(dwBlockAddr) )
//		return 0;

	console_printf(" is erased.\n" );

	return 0;

}

static UINT32 flash_lock_block( INT32 iBlock)
{
	UINT32	dwBlockAddr;

	console_printf(" %d :",iBlock);

	dwBlockAddr  =  FLASH_BASE_ADDR + (iBlock << 17);	// 128K / Block

	FLASH_CLEAR_GSTATUS(dwBlockAddr);

	FLASH_LOCK_0(dwBlockAddr);
	FLASH_LOCK_1(dwBlockAddr);
	if( !flash_check_status(dwBlockAddr) )
		return 0;
	
	FLASH_READ_ARRAY(dwBlockAddr);	// Set Back to read array mode

	console_printf(" is locked.\n" );
	
	return 0;
}

static UINT32 flash_unlock_block( INT32 iBlock)
{
	UINT32	dwBlockAddr;

	console_printf(" %D :",iBlock);

	dwBlockAddr  =  FLASH_BASE_ADDR + (iBlock << 17);	// 128K / Block

	FLASH_CLEAR_GSTATUS(dwBlockAddr);

	FLASH_UNLOCK_0(dwBlockAddr);
	FLASH_UNLOCK_1(dwBlockAddr);
	if( !flash_check_status(dwBlockAddr) )
		return 0;
	
	FLASH_READ_ARRAY(dwBlockAddr);	// Set Back to read array mode
	
	console_printf(" is unlocked.\n" );

	return 0;
}

static UINT32 flash_program_block(UINT32 iBlock, CHAR *pData) 
{
	UINT32	dwBlockAddr;
	INT32 i;

    console_printf("pData = %X\n",(UINT32)pData);
RETRY_PROGRAM:
	console_printf(" %D :",iBlock);

	// should be erased
	flash_erase_block(iBlock);
	
	dwBlockAddr  =  FLASH_BASE_ADDR + (iBlock << 17);	// 128K / Block
    console_printf("dwBlockAddr = %X\n",(UINT32)dwBlockAddr);

	FLASH_CLEAR_GSTATUS(dwBlockAddr);

	// Get Global Status
	FLASH_READ_GSTATUS(dwBlockAddr);

	if( !flash_check_status(dwBlockAddr) )
	{
	    console_printf("status read fail : retry ");
	    goto RETRY_PROGRAM;
		return 0;
    }

	for( i = 0; i < STRATA_BLOCK_SIZE; i+=2)
	{
		FLASH_WRITE_WORD(dwBlockAddr+i);						// Tell Flash to program..
		WRITE_REGISTER_USHORT(dwBlockAddr+i,*(UINT16*)(pData+i));					// Copy it to Flash
		if( !flash_check_status(dwBlockAddr+i) )
    	{
    	    console_printf("program fail : retry ");
    	    goto RETRY_PROGRAM;
    		return 0;
        }
	}

	FLASH_READ_ARRAY(dwBlockAddr);	// Set Back to read array mode

	for( i = 0; i < STRATA_BLOCK_SIZE; i+=2)
	{
        if( *(PUINT16)(pData+i) != *(PUINT16)(dwBlockAddr + i) )
        {
    	    console_printf("program fail : retry ");
    	    goto RETRY_PROGRAM;
    		return 0;
        }
    }

	console_printf(" is programed.\n" );

	return 0;

}
UINT32 flash_lock( INT32 iStart, INT32 iEnd)
{
	for( ; iStart <= iEnd; iStart++)
		flash_lock_block(iStart);

	return 0;
}
UINT32 flash_unlock( INT32 iStart, INT32 iEnd)
{
	for( ; iStart <= iEnd; iStart++)
		flash_unlock_block(iStart);

	return 0;
}
	
UINT32 flash_erase( INT32 iStart, INT32 iEnd)
{
	for( ; iStart <= iEnd; iStart++)
		flash_erase_block(iStart);

	return 0;
}

UINT32 flash_program( UINT32 iTarget, UINT32 iSource, UINT32 iSize)
{
	UINT32 iStart, iEnd;
	
	if( iTarget + iSize > 0x1000000)	//16M Flash 
	{
		console_printf("The paramenter is wrong\n");
		return 1;
	}
	
	iStart = (iTarget / STRATA_BLOCK_SIZE);
	iEnd = (iTarget + iSize + STRATA_BLOCK_SIZE-1)/STRATA_BLOCK_SIZE;

	for( ; iStart < iEnd; iStart++, iSource += STRATA_BLOCK_SIZE)
		flash_program_block(iStart, (CHAR *)iSource);

	return 0;
}

//================================================================================

VOID flash_operation( VOID )
{
	CHAR	*token;
	CHAR	op;
	
	
	/*
	 *  Get all the arguments.
	 */
	/* operation supplied? */
	token = __PULL() ;
	if ((token == NULL))
	{
		console_printf("\nERROR: you must supply an operation\n");
		return ;
	}
	if( token[1]!=0)
	{
		console_printf("\nERROR: incorrect operation\n");
		return;
	}
	op = *token;


	switch(op)
	{
	
	case 'e':
	case 'E':
	{
		INT32	iStart, iEnd;
		
		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an start block\n");
			return ;
		}
		iStart = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}

		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an start block\n");
			return ;
		}
		iEnd = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}
	
        if( op == 'e')
		    flash_erase(iStart, iEnd);
		else
		    flash_erase_32(iStart, iEnd);
		
		break;
	}
	case 'l':
	case 'L':
	{
		INT32	iStart, iEnd;

		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an start block\n");
			return ;
		}
		iStart = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}

		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an start block\n");
			return ;
		}
		iEnd = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}
	
        if( op == 'l')
    		flash_lock(iStart, iEnd);
        else
	    	flash_lock_32(iStart, iEnd);
		
		break;
	}
	case 'u':
	case 'U':
	{
		INT32	iStart, iEnd;

		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an start block\n");
			return ;
		}
		iStart = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}

		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an start block\n");
			return ;
		}
		iEnd = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}
	
        if( op == 'u')
    		flash_unlock(iStart, iEnd);
		else
    		flash_unlock_32(iStart, iEnd);

		break;
	}
	case 'p':
	case 'P':
	{
		UINT32	iSource, iTarget, iSize;

		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an target address\n");
			return ;
		}
		iTarget = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}

		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an source address\n");
			return ;
		}
		iSource = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}
	
		token = __PULL() ;
		if ((token == NULL))
		{
			console_printf("\nERROR: you must supply an size\n");
			return ;
		}
		iSize = ston(token);
		if( E_ston )
		{
			console_printf("\nwrong parameter.\n") ;
			return;
		}

        if( op == 'p')
		    flash_program(iTarget, iSource, iSize);
		else
		    flash_program_32(iTarget, iSource, iSize);
		    
		break;
	}
	default :
		console_printf("\nERROR: incorrect operation\n");
		
	}

	return;
}
#endif 