#include <types.h>
#include <net/netdev.h>
#include <net/ip.h>
#include <net/if_ether.h>
#include <net/time.h>

#include <string.h>

UINT32	cached_ip;
STATIC BYTE	cached_hw[8];

struct arphdr {
	UINT16	ar_hrd;		/* format of hardware address	*/
	UINT16	ar_pro;		/* format of protocol address	*/
	BYTE	ar_hln;		/* length of hardware address	*/
	BYTE	ar_pln;		/* length of protocol address	*/
	UINT16	ar_op;		/* ARP opcode (command)		*/
	BYTE	ar_sha[6];	/* sender h/w address		*/
	BYTE	ar_sip[4];	/* sender ip address		*/
	BYTE	ar_tha[6];	/* target h/w address		*/
	BYTE	ar_tip[4];	/* target ip address		*/
};

/*
 * Send an ARP request to resolve an IP address
 */
STATIC INT32 arp_send(struct netdev *nd, UINT32 ip_addr)
{
	struct arphdr ar;
	struct buflist bl, *blp;

	bl.data = &ar;
	bl.size = sizeof(ar);
	bl.next = NULL;

	memzero(&ar, sizeof(ar));

	ar.ar_hrd = htons(1);		/* ARPHRD_ETHER */
	ar.ar_pro = htons(ETH_P_IP);
	ar.ar_hln = nd->hw_addr_len;
	ar.ar_pln = sizeof(ip_addr);
	ar.ar_op  = htons(1);		/* ARPOP_REQUEST */
	memcpy(ar.ar_sha, nd->hw_addr, nd->hw_addr_len);
	memcpy(ar.ar_sip, &nd->ip_addr, sizeof(nd->ip_addr));
	memcpy(ar.ar_tip, &ip_addr, sizeof(ip_addr));

//console_printf(" sip = %x, ar.sip = %x.\n", nd->ip_addr, *(UINT32*)ar.ar_sip);
//console_printf(" tip = %x, ar.tip = %x.\n", ip_addr, *(UINT32*)ar.ar_tip);


	blp = nd->trans->header(nd, nd->hw_broadcast, ETH_P_ARP, &bl);

//console_printf(" sip = %x, ar.sip = %x.\n", nd->ip_addr, *(UINT32*)ar.ar_sip);
//console_printf(" tip = %x, ar.tip = %x.\n", ip_addr, *(UINT32*)ar.ar_tip);

//{
//	INT32 i;
//	console_printf(" arp_send : hw_addr = ");
//	for( i=0; i < 6;i++)
//		console_printf("%x.",nd->hw_addr[i]);
//	console_printf("\n");
//
//	console_printf(" arp_send : hw_broadcase = ");
//	for( i=0; i < 6;i++)
//		console_printf("%x.",nd->hw_broadcast[i]);
//	console_printf("\n");
//
//	console_printf(" size = %x\n",nd->hw_addr_len);
//}
	return blp ? nd->trans->send(nd, blp) : -1;
}

/*
 * Check ARP reply is valid for the specified IP address
 */
STATIC INT32 arp_reply_valid(struct netdev *nd, UINT32 ip_addr, struct arphdr *ar)
{
	INT32 valid;

	valid = ar->ar_hrd == htons(1);			/* ARPHRD_ETHER */
	valid = valid && ar->ar_pro == htons(ETH_P_IP);
	valid = valid && ar->ar_hln == nd->hw_addr_len;
	valid = valid && ar->ar_pln == sizeof(ip_addr);
	valid = valid && ar->ar_op == htons(2);		/* ARPOP_REPLY */
	valid = valid && memeq(ar->ar_tha, nd->hw_addr, nd->hw_addr_len);
	valid = valid && memeq(ar->ar_tip, &nd->ip_addr, sizeof(nd->ip_addr));
	valid = valid && memeq(ar->ar_sip, &ip_addr, sizeof(ip_addr));

	return valid;
}

/*
 * Get an Ether address for an IP address
 */
INT32 arp_lookup(struct netdev *nd, UINT32 addr, BYTE *eth_addr)
{
	INT32 retries = 15, timeout = 25;

//console_printf("arp lookup : %x\n", addr);

	while(addr != cached_ip && retries) {
		char buffer[1024];
		struct arphdr *ar = (struct arphdr *)buffer;
		INT32 bytes;
		INT32 next_try;

		arp_send(nd, addr);

		centisecs = getcentisec();
		next_try = centisecs + timeout;
		timeout = timeout * 2;
		if (timeout > 500)
			timeout = 500;

		do {
			bytes = nd->trans->recv(nd, ETH_P_ARP, buffer);
//if(bytes>0)console_printf(" arp_lookup : bytes = %d , ar = %d\n",bytes,sizeof(*ar) );
			if (bytes >= sizeof(*ar) && arp_reply_valid(nd, addr, ar))
				break;
			bytes = 0;
//if(bytes>0)console_printf(" arp_lookup :arp_replay_valid : bytes = %x\n",bytes);
		centisecs = getcentisec();
		} while (centisecs < next_try);

		retries -= 1;

		if (!bytes)
			continue;

		cached_ip = addr;
		memcpy(cached_hw, ar->ar_sha, ar->ar_hln);
	}

	if (addr == cached_ip) {
		memcpy(eth_addr, cached_hw, nd->hw_addr_len);
		return 0;
	}

//console_printf("arp lookup : end %x , %x\n", addr, *eth_addr);

	return 1;
}
