#include <types.h>
#include <net/netdev.h>
#include <net/checksum.h>
#include <net/buflist.h>
#include <net/ip.h>
#include <net/if_ether.h>

#include <string.h>
#include <console.h>

EXTERN INT32 arp_lookup(struct netdev *nd, UINT32 addr, BYTE *eth_addr);


STATIC UINT32 ip_pkt_no;

CHAR *in_ntoa(UINT32 addr)
{
	STATIC CHAR buffer[16];
/*
	sprintf(buffer, "%ld.%ld.%ld.%ld",
		addr & 255,
		(addr >> 8) & 255,
		(addr >> 16) & 255,
		(addr >> 24) & 255);
*/
{
    INT32 i,j;
    CHAR h;
    CHAR b;
    CHAR value;

    for( i=0,j=0; j < 4; j++)
    {
        value = (addr>>j) & 0xFF;
    
        b = 0;
        h = value/100;
        if( h != 0)
        {
            buffer[i++] = '0'+ h;
            value -= 100*h;
            b = 1;
        }
    
        h = value/10;
        if( ( b == 0) && ( value != 0) )
        {
            buffer[i++] = '0'+ h;
            value -= 10*h;
            b = 1;
        }
        else
        {
            buffer[i++] = '0'+ h;
            value -= h*10;
        }
        
        buffer[i++] = '0'+ h;

        buffer[i++] = '.';
    }

    buffer[i-1] = 0;
}

	return buffer;
}

STATIC UINT16 ip_check(struct iphdr *ip)
{
	UINT32 check, length = ip->ip_ihl * 4;

	check = ~checksum(ip, length, 0);

	if (check == 0)
		check = -1;

	return htons(check);
}

INT32 ip_send(struct netdev *nd, INT32 protocol, UINT32 from, UINT32 to, struct buflist *data)
{
	BYTE hwto[6], *dest;
	struct buflist bl, *blp;
	struct iphdr ip;
	INT32 size = 0;

	for (blp = data; blp; blp = blp->next)
		size += blp->size;

	ip.ip_ver    = 4;
	ip.ip_ihl    = 5;
	ip.ip_tos    = 0;
	ip.ip_len    = htons(sizeof(ip) + size);
	ip.ip_id     = htons(ip_pkt_no);
	ip.ip_frag   = 0;
	ip.ip_ttl    = 0x40;
	ip.ip_proto  = protocol;
	ip.ip_check  = 0;
	ip.ip_source = from;
	ip.ip_dest   = to;
	ip.ip_check  = ip_check(&ip);
console_printf("\n-------------------------------ip.ip_check  %x\n",ip.ip_check );	

	ip_pkt_no += 1;

	bl.data = &ip;
	bl.size = sizeof(ip);
	bl.next = data;

	if (to == INADDR_ANY)
		dest = nd->hw_broadcast;
	else if (!arp_lookup(nd, to, hwto))
		dest = hwto;
	else
		return -1;

//console_printf("ip_send 0\n");	
	blp = nd->trans->header(nd, dest, ETH_P_IP, &bl);

//console_printf("ip_send 1: %x\n", blp);	
	return blp ? nd->trans->send(nd, blp) : -1;
}

INT32 ip_recv(struct netdev *nd, INT32 protocol, UINT32 from, UINT32 to, BYTE *buffer)
{
	BYTE ip_buffer[1584];
	struct iphdr *ip = (struct iphdr *)ip_buffer;
	INT32 bytes, hdrlen;

	bytes = nd->trans->recv(nd, ETH_P_IP, ip_buffer);
	console_printf("ip_recv0  %d\n", bytes);

	if (bytes == 0)
		return 0;

	if (ip->ip_ver != 4 || ip->ip_ihl < 5) {
		console_printf("wrong IP version/header length\n");
		return 0;
	}

	hdrlen = ip->ip_ihl * 4;

	if (checksum(ip, hdrlen, 0) != 0xffff) {
		console_printf("bad IP checksum\n");
		return 0;
	}

	if (ip->ip_proto != protocol) {
		console_printf("wrong protocol %x\n",ip->ip_proto);
		return 0;
	}

	if (from != INADDR_ANY && from != ip->ip_source) {
		console_printf("wrong source address\n");
		return 0;
	}

	if (to != INADDR_ANY && to != ip->ip_dest) {
		console_printf("wrong destination address\n");
		return 0;
	}

	bytes -= hdrlen;

	console_printf("ip_recv %d\n", bytes);

	memcpy(buffer, ip_buffer + hdrlen, bytes);

	return bytes;
}
