#include <types.h>
#include <net/netdev.h>
#include <net/bootdev.h>
#include <net/ip.h>
#include <net/udp.h>
#include <net/time.h>

#include <string.h>
#include <console.h>



#define CONF_TIMEOUT_MINOR_BASE	10
#define CONF_TIMEOUT_MAJOR_BASE	500
#define CONF_TIMEOUT_MULT	5/4
#define CONF_TIMEOUT_MAX	1500
#define CONF_RETRIES		10

#define TFTP_RRQ		1
#define TFTP_DATA		3
#define TFTP_ACK		4
#define TFTP_ERROR		5

#define TFTP_STATE_RRQ		0
#define TFTP_STATE_ACK		1
#define TFTP_STATE_COMPLETE	2

struct tftp_rrq {
	UINT16	cmd;
	BYTE	filename[100];
};

struct tftp_ack {
	UINT16	cmd;
	UINT16	block;
};

struct tftp_data {
	UINT16	cmd;
	UINT16	block;
	BYTE	data[512];
};

EXTERN struct downloadconfig	dnconfig;
EXTERN struct netdev	*selected_net_devs;
STATIC struct sin	 tftp_me, tftp_serv;

STATIC INT32 tftp_send_rrq(VOID)
{
	struct tftp_rrq rrq;
	struct buflist bl;

	tftp_me.sin_addr = dnconfig.my_ip;
	tftp_me.sin_port = htons(0x8000);

	tftp_serv.sin_addr = dnconfig.server_ip;
	tftp_serv.sin_port = htons(15306); //htons(0x45);

	memzero(&rrq, sizeof(rrq));
	rrq.cmd = htons(TFTP_RRQ);

//	sprintf(rrq.filename, "%s%c%s", dnconfig.filename, 0, "octet");
{
    INT32 i;
    for( i = 0; ;i++)
    {
        if( dnconfig.filename[i] == 0 )
            break;

        rrq.filename[i] = dnconfig.filename[i];
    }
    
    rrq.filename[i] = 0;
    rrq.filename[i+1] = 'o';
    rrq.filename[i+2] = 'c';
    rrq.filename[i+3] = 't';
    rrq.filename[i+4] = 'e';
    rrq.filename[i+5] = 't';
}

	bl.data = &rrq;
	bl.size = 102;
	bl.next = NULL;

	return udp_send(selected_net_devs, &tftp_me, &tftp_serv, &bl);
}

STATIC INT32 tftp_send_ack(INT32 block)
{
	struct tftp_ack ack;
	struct buflist bl;

	ack.cmd   = htons(TFTP_ACK);
	ack.block = htons(block);

	bl.data = &ack;
	bl.size = 102;
	bl.next = NULL;

	return udp_send(selected_net_devs, &tftp_me, &tftp_serv, &bl);
}

STATIC INT32 tftp_recv_data(INT32 block, VOID *buffer)
{
	struct tftp_data data;
	INT32 bytes, recvd_block;

	if (block == 1)
		tftp_serv.sin_port = 0;

	bytes = udp_recv(selected_net_devs, &tftp_serv, &tftp_me, &data, sizeof(data));

	if (!bytes)
		return 0;

	if (data.cmd != htons(TFTP_DATA))
		return 0;

	recvd_block = htons(data.block);

	if (recvd_block <= block)
		tftp_send_ack(recvd_block);

	if (recvd_block != block)
		return 0;

	memcpy(buffer, data.data, bytes - 4);

	return bytes - 4;
}

#define TWIDDLE_IDX(x)	  (((x) >> 16) & 3)
#define TWIDDLE_UPDATE(x) (((x) & 0xffff) == 0)

INT32 do_tftp(VOID)
{
	EXTERN UINT32	cached_ip;

	STATIC CHAR twiddle[] = { '|', '/', '-', '\\' };
	INT32 timeout, targ, retries, error, block = 1, bytes = 0;
	INT32 total_bytes = 0;
	BYTE *data = (BYTE *)dnconfig.addr;

	console_printf("TFTPing %s... ", dnconfig.filename);

	timeout = CONF_TIMEOUT_MAJOR_BASE;
	retries = CONF_RETRIES;

	cached_ip = 0;

	do {
//console_printf(" do_tftp, %d\n", block);
		if (block == 1)
			error = tftp_send_rrq();
		else
			error = tftp_send_ack(block);

		if (error)
			break;

		centisecs = getcentisec();
		targ = centisecs + timeout;

		while ( (centisecs=getcentisec()) < targ) {
			bytes = tftp_recv_data(block, data + total_bytes);
			if (bytes) {
				retries = CONF_RETRIES;
				timeout = CONF_TIMEOUT_MINOR_BASE;

				centisecs = getcentisec();
				targ = centisecs + timeout;
				if (TWIDDLE_UPDATE(total_bytes))
					console_printf("%c\010", twiddle[TWIDDLE_IDX(total_bytes)]);
				total_bytes += bytes;
				if (bytes == 512)
					block ++;
				else
					break;
			}
		}

		if (timeout == CONF_TIMEOUT_MINOR_BASE)
			timeout = CONF_TIMEOUT_MAJOR_BASE;

		if (bytes)
			break;

		if (!--retries) {
			console_printf("\010 timed out\n");
			error = 1;
			break;
		}

		timeout = timeout * CONF_TIMEOUT_MULT;
		if (timeout > CONF_TIMEOUT_MAX)
			timeout = CONF_TIMEOUT_MAX;
	} while (1);

	if (error) {
		console_printf("\010 Error\n");
		return 1;
	}

	console_printf("\010 Ok - %dKB\n", total_bytes / 1024);

	return 0;
}
