#ifndef __DEV_DRVLIB_H__
#define __DEV_DRVLIB_H__

#include <types.h>
#include <message.h>

#include <mach/gpio.h>


EXTERN int pio_setup (const struct pio_desc *pio_desc, int nb_pio);
EXTERN int pio_get_value(unsigned pin);
EXTERN VOID pio_set_value(unsigned pin, int value);


 /** DMA Transfer Descriptor as well as Linked List Item */
 typedef struct _HDMA_TRANSFER_DESC_
 {
     UINT32      SrcAddr;        /**< Source buffer address */
     UINT32      DstAddr;        /**< Destination buffer address */
     UINT32      CtrlA;          /**< Control A register settings */
     UINT32      CtrlB;          /**< Control B register settings */
     UINT32      DscAddr;        /**< Next descriptor address */
 } HDMA_TRANSFER_DESC;
 
/*----------------------------------------------------------------------------
 *        Consts
 *----------------------------------------------------------------------------*/
#define DMAD_TRANSFER_MEMORY  0xFF   /**< DMA transfer from or to memory */
#define DMAD_ALLOC_FAILED     0xFFFF /**< Channel allocate failed */

#define DMAD_TRANSFER_TX      0
#define DMAD_TRANSFER_RX      1

/** DMA status or return code */
enum
{
    DMAD_OK = 0,        /**< Operation is sucessful */
    DMAD_BUSY,          /**< Channel occupied or transfer not finished */
    DMAD_PARTIAL_DONE,  /**< Partial of LLI finished (on each buffer done) */
    DMAD_ERROR,         /**< Operation failed */
    DMAD_CANCELED       /**< Operation canceled */
};

/** DMA transfer callback */
typedef void (*fnDmadTransferCallback)(UINT32 Status, PVOID pArg);

BYTE DMAIF_Get_ChannelNumber (BYTE bDmac,
                              BYTE bPeriphID, 
                              BYTE bTransfer);

EXTERN VOID DMAD_Initialize( void );
EXTERN UINT32 DMAD_Handler( UINT32 nChannel );
EXTERN UINT32 DMAD_AllocateChannel( BYTE bSrcID, 
                             BYTE bDstID,
                             BYTE bPollingMode );
EXTERN UINT32 DMAD_SetCallback( UINT32 nChannel,
                         fnDmadTransferCallback fCallback,
                         PVOID pArg );
EXTERN UINT32 DMAD_ConfigurePIP( UINT32 nChannel,
                           UINT32 dwSrcPIP, 
                           UINT32 dwDstPIP );
EXTERN UINT32 DMAD_PrepareChannel( UINT32 nChannel,
                             UINT32 dwCfg );
EXTERN UINT32 DMAD_IsTransferDone( UINT32 nChannel );
EXTERN VOID DMAD_ClearAuto( UINT32 nChannel );
EXTERN UINT32 DMAD_StartTransfer( UINT32 nChannel );
EXTERN UINT32 DMAD_StartTransfers( BYTE bDmac, UINT32 bmChannels );
EXTERN UINT32 DMAD_StopTransfer( UINT32 nChannel );
EXTERN UINT32 DMAD_PrepareSingleTransfer( UINT32 nChannel,
                                    HDMA_TRANSFER_DESC *pXfrDesc );
EXTERN UINT32 DMAD_PrepareMultiTransfer( UINT32 nChannel,
                                   HDMA_TRANSFER_DESC *pXfrDesc );


/* for cache */
extern unsigned int CP15_ReadID(void);
extern unsigned int CP15_ReadControl(void);
extern void         CP15_WriteControl(unsigned int value);
extern void         CP15_WriteTTB(unsigned int value);
extern void         CP15_WriteDomainAccessControl(unsigned int value);

extern void CP15_InvalidateIcacheInnerSharable(void);
extern void CP15_InvalidateBTBinnerSharable(void);
extern void CP15_InvalidateIcache(void);
extern void CP15_InvalidateIcacheByMva(void);
extern void CP15_FlushBTB(void);
extern void CP15_FlushBTBbyMva(void);
extern void CP15_InvalidateDcacheLineByMva(void);
extern void CP15_InvalidateDcacheLineBySetWay(void);
extern void CP15_CleanDCacheByMva(void);
extern void CP15_CleanDCacheBySetWay(void);
extern void CP15_CleanDCacheMva(void);
extern void CP15_CleanInvalidateDcacheLineByMva(void);
extern void CP15_CleanInvalidateDcacheLine(void);

extern void CP15_coherent_dcache_for_dma (UINT32 startAddr, UINT32 endAddr );
extern void CP15_invalidate_dcache_for_dma (UINT32 startAddr, UINT32 endAddr );
extern void CP15_clean_dcache_for_dma (UINT32 startAddr, UINT32 endAddr );
extern void CP15_flush_dcache_for_dma (UINT32 startAddr, UINT32 endAddr );
extern void CP15_flush_kern_dcache_for_dma (UINT32 startAddr, UINT32 size );                                   
                                   
#endif //__DEV_DRVLIB_H__

