
#ifndef __NAND_DRV_H__
#define __NAND_DRV_H__



#define IOCTRL_DEVICE_NAND_READ     (128|(MESSAGE_DEVICE<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_DEVICE_NAND_WRITE    (129|(MESSAGE_DEVICE<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_DEVICE_NAND_ERASE    (130|(MESSAGE_DEVICE<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_DEVICE_NAND_GET_INFO (131|(MESSAGE_DEVICE<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_DEVICE_NAND_READ_STREAM    (132|(MESSAGE_DEVICE<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))



typedef struct 
{
    PBYTE       pBuf;
    PBYTE       pOob;
    UINT32      Addr;
}NAND_DRV_DATA, *NAND_DRV_PDATA;


typedef struct 
{
    PBYTE       pBuf;
    UINT32      Len;
    UINT32      Addr;
}NAND_DRV_DATA_STREAM, *NAND_DRV_PDATA_STREAM;

typedef struct 
{
    UINT32      fErase;
    UINT32      Addr;
}NAND_DRV_ERASE, *NAND_DRV_PERASE;


#define NAND_MODE_DATA              (1<<0)
#define NAND_MODE_SPARE             (1<<1)


typedef struct _NAND_FLASH_INFO_
{
    UINT16      MafId;
    UINT16      DevId;

    UINT32      PageShift;
    UINT32      BlockShift;

    UINT32      MaskOfPage;
    UINT32      MaskOfBlock;
    
    UINT32      nBytePerPage;
    UINT32      nBytePerSpare;
    UINT32      nBytePerBlock;
    UINT32      nBytePerChip;

    UINT32      nPagePerBlock;
    //UINT32      nBytePerBlock;

    UINT32      nBlockPerChip;
    

} NAND_FLASH_INFO, *NAND_FLASH_PINFO;



#define POOL_ON_READYBUSY           0x01                        //!< Pool on ReadyBusy PIO
#define INTERRUPT_ON_READYBUSY      0x02                        //!< Interrupt on ReadyBusy PIO
#define POOL_ON_STATUS              0x04                        //!< Pool on Status byte



// Standard NAND flash commands
#define NAND_CMD_READ0          0
#define NAND_CMD_READ1          1
#define NAND_CMD_RNDOUT         5
#define NAND_CMD_PAGEPROG       0x10
#define NAND_CMD_READOOB        0x50
#define NAND_CMD_ERASE1         0x60
#define NAND_CMD_STATUS         0x70
#define NAND_CMD_STATUS_MULTI   0x71
#define NAND_CMD_SEQIN          0x80
#define NAND_CMD_RNDIN          0x85
#define NAND_CMD_READID         0x90
#define NAND_CMD_ERASE2         0xd0
#define NAND_CMD_RESET          0xff

// extended commands for large page devices
#define NAND_CMD_READSTART      0x30
#define NAND_CMD_RNDOUTSTART    0xE0
#define NAND_CMD_CACHEDPROG     0x15


/* Status bits */
#define NAND_STATUS_FAIL        0x01
#define NAND_STATUS_FAIL_N1     0x02
#define NAND_STATUS_TRUE_READY  0x20
#define NAND_STATUS_READY       0x40
#define NAND_STATUS_WP          0x80





EXTERN HANDLE   NAND_IoControlInit( VOID );
EXTERN BOOL     NAND_IoControlGetInfo( NAND_FLASH_PINFO pNandFlashInfo );
EXTERN BOOL     NAND_IoControlRead( UINT32 nPage, PBYTE pBuf, UINT32 nBuf, PBYTE pOob, UINT32 nOob);
EXTERN BOOL     NAND_IoControlWrite( UINT32 nPage, PBYTE pBuf, UINT32 nBuf, PBYTE pOob, UINT32 nOob);
EXTERN BOOL     NAND_IoControlErase( UINT32 nPage);



#endif /*__NAND_DRV_H__*/

