/*    $ dooroos: hal_interrupt.h, v 0.1 2002/12/17 $    */

/*
 * Copyright (C) 2002    iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: hal_interrupt.h
 *
 * Purpose: interrupt relative definition
 *
 */
#ifndef    __DOOROOS_HAL_INTERRUPT_H_
#define    __DOOROOS_HAL_INTERRUPT_H_

#include <types.h>
#include <time.h>
#include <hal.h>
#include <kernel.h>
//------------------------------------------------------------------------------
// Physical Interrupt Number 
//------------------------------------------------------------------------------
// *****************************************************************************
//               PERIPHERAL ID DEFINITIONS FOR AT91SAM9GX5
// *****************************************************************************
//This define can be conformed in SAMA5D3X.h
#define AT91C_ID_FIQ             0    /* Advanced Interrupt Controller (FIQ) */
#define AT91C_ID_SYS             1    /* System Controller */
#define SAMA5D3_ID_DBGU          2    /* debug Unit (usually no special interrupt line) */
#define SAMA5D3_ID_PIT           3    /* PIT */
#define SAMA5D3_ID_WDT           4    /* WDT */
#define SAMA5D3_ID_HSMC          5    /* Static Memory Controller */
#define SAMA5D3_ID_PIOA          6    /* PIOA */
#define SAMA5D3_ID_PIOB          7    /* PIOB */
#define SAMA5D3_ID_PIOC          8    /* PIOC */
#define SAMA5D3_ID_PIOD          9    /* PIOD */
#define SAMA5D3_ID_PIOE         10    /* PIOE */
#define SAMA5D3_ID_SMD          11    /* SMD */
#define SAMA5D3_ID_USART0       12    /* USART0 */
#define SAMA5D3_ID_USART1       13    /* USART1 */
#define SAMA5D3_ID_USART2       14    /* USART2 */
#define SAMA5D3_ID_USART3       15    /* USART3 */
#define SAMA5D3_ID_UART0        16    /* UART0 */
#define SAMA5D3_ID_UART1        17    /* UART1 */
#define SAMA5D3_ID_TWI0         18    /* Two-Wire Interface 0 */
#define SAMA5D3_ID_TWI1         19    /* Two-Wire Interface 1 */
#define SAMA5D3_ID_TWI2         20    /* Two-Wire Interface 2 */
#define SAMA5D3_ID_HSMCI0       21    /* MCI */
#define SAMA5D3_ID_HSMCI1       22    /* MCI */
#define SAMA5D3_ID_HSMCI2       23    /* MCI */
#define SAMA5D3_ID_SPI0         24    /* Serial Peripheral Interface 0 */
#define SAMA5D3_ID_SPI1         25    /* Serial Peripheral Interface 1 */
#define SAMA5D3_ID_TC0          26    /* Timer Counter 0 (ch. 0, 1, 2) */
#define SAMA5D3_ID_TC1          27    /* Timer Counter 1 (ch. 0, 1, 2) */
#define SAMA5D3_ID_PWM          28    /* Pulse Width Modulation Controller (PWM) */
#define SAMA5D3_ID_ADC          29    /* Touch Screen ADC Controller (ADC)*/
#define SAMA5D3_ID_DMA0         30    /* DMA Controller 0 */
#define SAMA5D3_ID_DMA1         31    /* DMA Controller 1 */
#define SAMA5D3_ID_UHPHS        32    /* USB Host High Speed */
#define SAMA5D3_ID_UDPHS        33    /* USB Device High Speed */
#define SAMA5D3_ID_GMAC         34    /* Gigabit Ethernet MAC */
#define SAMA5D3_ID_EMAC         35    /* Ethernet MAC */
#define SAMA5D3_ID_LCDC         36    /* LCD Controller */
#define SAMA5D3_ID_ISI          37    /* Image Sensor Interface */
#define SAMA5D3_ID_SSC0         38    /* Synchronous Serial Controller 0 */
#define SAMA5D3_ID_SSC1         39    /* Synchronous Serial Controller 1 */
#define SAMA5D3_ID_CAN0         40    /* CAN Controller 0 */
#define SAMA5D3_ID_CAN1         41    /* CAN Controller 1 */
#define SAMA5D3_ID_SHA          42    /* Secure Hash Algorithm (SHA) */
#define SAMA5D3_ID_AES          43    /* Advanced Encryption Standard (AES) */
#define SAMA5D3_ID_TDES         44    /* Triple Data EncryptionStandard (TDES) */
#define SAMA5D3_ID_TRNG         45    /* True Random Number Generator (TRNG) */
#define SAMA5D3_ID_ARM          46    /* Performance Monitor Unit (ARM) */
#define SAMA5D3_ID_IRQ          47    /* Advanced Interrupt Controller (IRQ) */
#define SAMA5D3_ID_FUSE         48    /* Fuse Controller (FUSE) */
#define SAMA5D3_ID_MPDDRC	    49    /* MPDDR controller (MPDDRC) */

#define IRQ_PHY_MAX_COUNT              50


//------------------------------------------------------------------------------
// GPIO Virtual Interrupt Number 32*5
//------------------------------------------------------------------------------
#define IRQ_VIR_GPIO_START          IRQ_PHY_MAX_COUNT
#define IRQ_VIR_GPIO_END            (IRQ_VIR_GPIO_START + 32*5)    // Group: A,B,C,D,E

#define IRQ_VIR_GPIO_A_START        (IRQ_VIR_GPIO_START + 32*0)
#define IRQ_VIR_GPIO_B_START        (IRQ_VIR_GPIO_START + 32*1)
#define IRQ_VIR_GPIO_C_START        (IRQ_VIR_GPIO_START + 32*2)
#define IRQ_VIR_GPIO_D_START        (IRQ_VIR_GPIO_START + 32*3)
#define IRQ_VIR_GPIO_E_START        (IRQ_VIR_GPIO_START + 32*4)

//------------------------------------------------------------------------------
// DMA Virtual Interrupt Number 
//------------------------------------------------------------------------------
#define IRQ_VIR_DMA_START           IRQ_VIR_GPIO_END
#define IRQ_VIR_DMA_END             (IRQ_VIR_DMA_START + 8 + 8)

#define IRQ_VIR_DMA0_START          (IRQ_VIR_DMA_START + 0)
#define IRQ_VIR_DMA1_START          (IRQ_VIR_DMA_START + 8)

//------------------------------------------------------------------------------
// MAX(Physical+Virtual) Interrupt Number
//------------------------------------------------------------------------------
#define IRQ_MAX_COUNT               IRQ_VIR_DMA_END




#define LOGINTR_UNDEFINED           ((DWORD)-1)
#define LOGINTR_MIN                 0

#define LOGINTR_AIC_FIRST           1 //First AIC's irq
#define LOGINTR_AIC_LAST            31 //Last AIC's irq

/// define the system used interrupt number
#define     SYSINTR_NOP             (0)
#define     SYSINTR_TICK            (1)

#define SYSINTR_RESERVED        2
#define SYSINTR_MAX_DEVICES     64
#define SYSINTR_MAXIMUM         (SYSINTR_RESERVED + SYSINTR_MAX_DEVICES)

//#define IOCTRL_HAL_USER                     ( 128|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_HAL_SYSINTR_REQUEST          ( 128|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_HAL_SYSINTR_RELEASE          ( 129|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))

typedef struct _SYSINTR_REQUEST_
{
    UINT16      nHalIrq;
    UINT16      BitMap;
    UINT32      Flag;
}SYSINTR_REQUEST, *SYSINTR_PREQUEST;

typedef struct _JB_INTRRUPT_
{
    UINT16      nSysIntr;
    UINT16      BitMap;
}JB_INTRRUPT, *JB_PINTRRUPT;


#define SYSINTR_REQ_FLAG_STATIC          (1<<0)      
#define SYSINTR_REQ_FLAG_FORCE_STATIC    (1<<1)
#define SYSINTR_REQ_FLAG_TRANSLATE       (1<<2)
#define SYSINTR_REQ_FLAG_MULTIPLE        (1<<3)
//#define SYSINTR_REQ_FLAG_DYNAMIC         (1<<3)
//#define SYSINTR_REQ_FLAG_UNDEFINED       (1<<4)
#endif // __DOOROOSHAL_INTERRUPT_H_
