#include <types.h>

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3XBD.h>


#define	OEM_DEBUG_READ_NODATA	(-1)
#define	OEM_DEBUG_COM_ERROR		(-2)

#define DBGU_BAUDRATE	(115200)
#define AT91C_US_ASYNC_MODE ( DBGU_MR_CHMODE_NORM + \
                        US_MR_NBSTOP_1_BIT + \
                        DBGU_MR_PAR_NONE + \
                        US_MR_CHRL_8_BIT + \
                        US_MR_USCLKS_SCK )

/// pointer to the Debug Serial controller
STATIC AT91PS_DBGU g_pDBGU = (AT91PS_DBGU)AT91C_BASE_DBGU;


/*****************************************************************************
* HAL_DebugWriteByte 
*
* NOTE: This function MUST NOT use any global variables and interrupt!!!!!!
*/
PUBLIC VOID HAL_DebugWriteByte( BYTE Ch ) 
{

    EXTERN BOOL bDebug;
    if(!bDebug)
        return ;

    
	if (g_pDBGU == NULL)
		return;

	while (!(g_pDBGU->DBGU_SR & DBGU_SR_TXRDY))
	{
		//do nothing. wait for the controller to be ready
	}
	g_pDBGU->DBGU_THR = Ch;
	while (!(g_pDBGU->DBGU_SR & DBGU_SR_TXRDY))
	{
		//do nothing. wait for the controller to be ready
	}
}



/*****************************************************************************
* HAL_DebugReadByte 
*
* NOTE: This function MUST NOT use any global variables and interrupt!!!!!!
*/

PUBLIC INT32 HAL_DebugReadByte( VOID )
{
	INT32   data;

	data=OEM_DEBUG_READ_NODATA;

	if (g_pDBGU == NULL)
	{
		return data;
	}

	if (g_pDBGU->DBGU_SR & DBGU_SR_RXRDY) //Did we receive at least one byte ?
	{
		data = g_pDBGU->DBGU_RHR; //If so return its value
	}

	return data; //If not so, return the no data flag
}


PUBLIC VOID 
ConsoleInit( VOID ) 
{
	UINT32              dwMasterClock;  
	AT91PS_PIO pPioA = (AT91PS_PIO) AT91C_BASE_PIOA;

    g_pDBGU = (AT91PS_DBGU)AT91C_BASE_DBGU;

	//dwMasterClock = AT91SAM926x_GetMasterClock(FALSE);
	dwMasterClock = BOARD_MCK;

	//
	// Init the DBGU
	//
	// Configure PIO controllers to periph mode	   	
	pPioA->PIO_PDR = (AT91C_PB12_DRXD | AT91C_PB13_DTXD);
	pPioA->PIO_ASR = (AT91C_PB12_DRXD | AT91C_PB13_DTXD);

	//Configure USART
	
	//* Disable interrupts
    g_pDBGU->DBGU_IDR = (unsigned int) -1;

    //* Reset receiver and transmitter
    g_pDBGU->DBGU_CR = DBGU_CR_RSTRX | DBGU_CR_RSTTX | DBGU_CR_RXDIS | DBGU_CR_TXDIS ;

	//* Define the baud rate divisor register
	{
		unsigned int baud_value = ((dwMasterClock*10)/(DBGU_BAUDRATE * 16));
		if ((baud_value % 10) >= 5)
			baud_value = (baud_value / 10) + 1;
		else
			baud_value /= 10;
		 g_pDBGU->DBGU_BRGR = baud_value;
	}

	//* Write the Timeguard Register
//??	g_pDBGU->US_TTGR = 0;

    //* Clear Transmit and Receive Counters

//    //* Disable the RX and TX PDC transfer requests
//	pPDC->PDC_PTCR = AT91C_PDC_RXTDIS;
//	pPDC->PDC_PTCR = AT91C_PDC_TXTDIS;
//
//	//* Reset all Counter register Next buffer first
//	pPDC->PDC_TNPR = 0;
//	pPDC->PDC_TNCR = 0;
//	pPDC->PDC_RPR = 0;
//	pPDC->PDC_RCR = 0;
//	pPDC->PDC_RPR = 0;
//	pPDC->PDC_RCR = 0;
//	pPDC->PDC_TPR = 0;
//	pPDC->PDC_TCR = 0;
//
//    //* Enable the RX and TX PDC transfer requests
//	pPDC->PDC_PTCR = AT91C_PDC_RXTEN;
//	pPDC->PDC_PTCR = AT91C_PDC_TXTEN;

    //* Define the USART mode
    g_pDBGU->DBGU_MR= AT91C_US_ASYNC_MODE  ;

	// Enable Transmitter
	g_pDBGU->DBGU_CR = DBGU_CR_RXEN;
	g_pDBGU->DBGU_CR = DBGU_CR_TXEN;

    return;
}


// this function is for easy use in platform... 
// not required in HAL
PUBLIC VOID 
ConsoleWrite( PBYTE p_Str) 
{

    // Send message to serial port
    while (*p_Str) 
	{
		if(*p_Str == '\n')	HAL_DebugWriteByte((BYTE)('\r'));

		HAL_DebugWriteByte((BYTE)*p_Str++);
	}		
}



//PUBLIC INT32 
//ConsoleRead( PBYTE p_Buffer, INT32 n)
//{
//	INT32   Ret;
//	INT32   i=0;	
//	for(; n > 0; n--)
//	{
//		do
//		{
//			Ret = ser_getc();
//			if( ( Ret == OEM_DEBUG_COM_ERROR) )
//				return i;
//		}while(Ret < 0);
//		
//		*p_Buffer++ = (BYTE)Ret;
//		i++;
//	}
//	
//	return i;
//}
//
//PUBLIC VOID 
//ConsolePutChar( BYTE Ch ) 
//{
//
//    ser_putch(Ch);
//	
//	if (Ch=='\n') ConsolePutChar('\r');
//
//}
//
//PUBLIC VOID 
//ConsoleWrite( PBYTE p_Str) 
//{
//    // Send message to serial port
//    while (*p_Str) 
//	{
//		if(*p_Str == '\n')	ser_putch((BYTE)('\r'));
//
//		ser_putch((BYTE)*p_Str++);
//	}		
//}

