/*    $ JBOSN: locore.s, v 0.1 2002/12/17 $    */

/*
 * Copyright (C) 2002    iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

#if 1
/* This defines the area where the MMU descriptors are stored.                  */
/* Somehow if the stack is too big, the two will overlap and the sytem'll trap  */
#define SAMA5D3X_BASE_MMU_CFG              (0x20004000)	

#define SAMA5D3X_BASE_IBOOT                0x00000000
#define SAMA5D3X_BASE_IROM                 0x00100000
#define SAMA5D3X_BASE_NFC_RAM              0x00200000
#define SAMA5D3X_BASE_SRAM                 0x00300000
#define SAMA5D3X_BASE_SMD                  0x00400000
#define SAMA5D3X_BASE_UDPHS_RAM            0x00500000
#define SAMA5D3X_BASE_UHP_OHCI             0x00600000
#define SAMA5D3X_BASE_UHP_EHCI             0x00700000
#define SAMA5D3X_BASE_AXI_MATRIX           0x00800000
#define SAMA5D3X_BASE_DAP                  0x00900000

#define SAMA5D3X_BASE_EBI0                 0x10000000
#define SAMA5D3X_BASE_DDR 	               0x20000000
#define SAMA5D3X_BASE_EBI1                 0x40000000
#define SAMA5D3X_BASE_EBI2                 0x50000000
#define SAMA5D3X_BASE_EBI3                 0x60000000
#define SAMA5D3X_BASE_NFC                  0x70000000
#define SAMA5D3X_BASE_UNDEF                0x80000000
#define SAMA5D3X_BASE_REG                  0xF0000000

#define SAMA5D3X_VIRT_EBI0                 0xD0000000
#define SAMA5D3X_VIRT_IBOOT                0xE0000000
#define SAMA5D3X_VIRT_IROM                 0xE0100000
#define SAMA5D3X_VIRT_NFC_RAM              0xE0200000
#define SAMA5D3X_VIRT_SRAM                 0xE0300000
#define SAMA5D3X_VIRT_SMD                  0xE0400000
#define SAMA5D3X_VIRT_UDPHS_RAM            0xE0500000
#define SAMA5D3X_VIRT_UHP_OHCI             0xE0600000
#define SAMA5D3X_VIRT_UHP_EHCI             0xE0700000
#define SAMA5D3X_VIRT_AXI_MATRIX           0xE0800000
#define SAMA5D3X_VIRT_DAP                  0xE0900000

#endif

/*
 * Module Name: locore.s
 *
 * Purpose: start the kernel and initialize memory and relocation the J-BOSN Operating System..
 *
 */

/* **** CPWAIT ******************************************************** */
/* * Params: $Rd      : temporary read register                         */
/* *                                                                    */
/* * Returns: nothing                                                   */
/* *                                                                    */
/* * Effects: corrupts $Rd                                              */
/* *                                                                    */
/* *  This macro is used to wait for coprocessor operations to complete */
/* *                                                                    */
/* Ex:                                                                  */
/*                                                                      */
/*    CPWAIT r0                                                         */
/*                                                                      */
/* ******************************************************************** */

#if 0
    .macro CPWAIT,  Rd
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
    .endm
#else
    .macro CPWAIT,  Rd
        mrc     p15, 0, \Rd, c2, c0, 0       @ arbitrary read of CP15
        mov     \Rd, \Rd                     @ wait for it (foward dependency)
        sub     pc, pc, #4                   @ branch to next instruction
    .endm
#endif

        .section ".text.init",#alloc,#execinstr

    .word       _text
    .word       _end

  .globl stext
stext:

    .align    0

    add        pc, pc, #(vector_reset - . - 8)
    ldr        pc, [pc, #Lundefined - . - 8]
    ldr        pc, [pc, #Lswi - . - 8]
    ldr        pc, [pc, #Labort_prefetch - . - 8]
    ldr        pc, [pc, #Labort_data - . - 8]
    ldr        pc, [pc, #Lempty - . - 8]
    ldr        pc, [pc, #Lirq - . - 8]
    ldr        pc, [pc, #Lfiq - . - 8]

Lundefined:
    .word    0x3FFFE004
Lswi:
    .word    0x3FFFE008
Labort_prefetch:
    .word    0x3FFFE00C
Labort_data:
    .word    0x3FFFE010
Lempty:
    .word    0x3FFFE014
Lirq:
    .word    0x3FFFE018
Lfiq:
    .word    0x3FFFE01C



    .global vector_reset
vector_reset:

#if 0
    /* Disable the MMU, caches, and write-buffer and flush. */
    ldr     r0, =0x2043             @ enable access to all coprocessors
    mcr     p15, 0, r0, c15, c1, 0  @
    CPWAIT  r0                      @

    ldr     r0, =0x00000078         @ get a zero to turn things off (must write bits[6:3] as 1s)
    mcr     p15, 0, r0, c1, c0, 0   @ turn off MMU, I&D caches, and write buffer 
    CPWAIT  r0                      @

    ldr     r0, =0x00000000         @ get a zero to turn things off
    mcr     p15, 0, r0, c8, c7, 0   @ flush (invalidate) I/D TLBs
    mcr     p15, 0, r0, c7, c7, 0   @ flush (invalidate) I/D caches
    mcr     p15, 0, r0, c7, c10, 4  @ drain the write buffer
    nop                             @
    nop                             @
    nop                             @

    mvn     r0, #0                  @ grant manager access to all domains
    mcr     p15, 0, r0, c3, c0, 0   @

    /* Icache enable and asynchronous clocking mode */
	@mrc     p15,    0,r6,c1,c0,0
    @orr     r6,     r6,#0xC0000000
    @orr     r6,     r6,#0x1000
	@mcr     p15,    0,r6,c1,c0,0


    @b   VirtualStart

setupMMU :



    /* build the MMU table */

    /* ******************************************************************/
    /*  Load RVA of MMUAddressTable[]                                   */
    /*                                                                  */

    add     r0, pc, #MMUAddressTable-(.+8)      @ (r0) = MMUAddressTable phys addr
    mov     r11, r0                             @ (r11) = &MemoryMap (save pointer)


    b   BUILDTTB


MMUAddressTable:
    /*----------------------------------------------------------------------------------------*/
    /*       virutal                     phisical                    size(MB)       Cachable  */
    /*----------------------------------------------------------------------------------------*/

    .word   SAMA5D3X_BASE_IBOOT,      SAMA5D3X_BASE_DDR,        0x20000000,     1
    .word   SAMA5D3X_BASE_DDR ,       SAMA5D3X_BASE_DDR,   	    0x20000000,     0
    .word   SAMA5D3X_BASE_EBI1,       SAMA5D3X_BASE_EBI1,       0x10000000,     0
    .word   SAMA5D3X_BASE_EBI2,       SAMA5D3X_BASE_EBI2,       0x10000000,     0
    .word   SAMA5D3X_BASE_EBI3,       SAMA5D3X_BASE_EBI3,       0x10000000,     0
    .word   SAMA5D3X_BASE_NFC,        SAMA5D3X_BASE_NFC,        0x10000000,     0
    .word   SAMA5D3X_BASE_UNDEF,      SAMA5D3X_BASE_UNDEF,      0x50000000,     0

    .word   SAMA5D3X_VIRT_EBI0,       SAMA5D3X_BASE_EBI0,       0x10000000,     0
    .word   SAMA5D3X_VIRT_IBOOT,      SAMA5D3X_BASE_IBOOT,   	0x00100000,     1
    .word   SAMA5D3X_VIRT_IROM,       SAMA5D3X_BASE_IROM,   	0x00100000,     1
    .word   SAMA5D3X_VIRT_NFC_RAM,    SAMA5D3X_BASE_NFC_RAM,    0x00100000,     0
    .word   SAMA5D3X_VIRT_SRAM,       SAMA5D3X_BASE_SRAM,    	0x00100000,     1
    .word   SAMA5D3X_VIRT_SMD,        SAMA5D3X_BASE_SMD,    	0x00100000,     0
    .word   SAMA5D3X_VIRT_UDPHS_RAM,  SAMA5D3X_BASE_UDPHS_RAM,  0x00100000,     0
    .word   SAMA5D3X_VIRT_UHP_OHCI,   SAMA5D3X_BASE_UHP_OHCI,   0x00100000,     0
    .word   SAMA5D3X_VIRT_UHP_EHCI,   SAMA5D3X_BASE_UHP_EHCI,   0x00100000,     0
    .word   SAMA5D3X_VIRT_AXI_MATRIX, SAMA5D3X_BASE_AXI_MATRIX, 0x00100000,     0
    .word   SAMA5D3X_VIRT_DAP,        SAMA5D3X_BASE_DAP,        0x0F700000,     0

    .word   SAMA5D3X_BASE_REG,        SAMA5D3X_BASE_REG,     	0x10000000,     0
    .word   0x00000000,                  0x00000000,                0         ,     0


BUILDTTB:
    /*  ============================================================================== */
    /*    Fill in first-level Section descriptors to create 1MB mapped regions         */
    /* from the contents of the MemoryMap array. (MMUAddressTable[])                   */
    /*                                                                                 */
    /*    (r9) = TTB                                                                   */
    /*    (r11) = ptr to MemoryMap array                                               */
        

    /*  Set TTB */
    ldr     r9, =SAMA5D3X_BASE_MMU_CFG     @ Physical address of the first-level table
    ldr     r0, =0xFFFFC000                     @
    and     r9, r9, r0                          @ Mask off TTB[31:0] (must be 0's).
    mcr     p15, 0, r9, c2, c0, 0               @ Set the TTB.

@    ldr     r9, =0x30000000
@    ldr     r1, =0x00010000
@    orr     r9, r9, r1                      @ r9 = 0x30010000
@@    mcr     p15, 0, r9, c2, c0, 0           @ set TTB

    /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
    /* ~~~~~~~~~~ MAP CACHEABLE. BUFFERABLE SECTION DESCRIPTORS ~~~~~~~~~~~~~~~~ */
    /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

    mov     r1, r11                @ (r1) = ptr to MemoryMap array

    /* r1 = pointer to MemoryMap array */
    /* r9 = TTB                        */

20:
    mov     r0, #0x0E               @ (r0) = Section (1MB) Descriptor@ C=B=1.  Write-Back, Read-Allocate
    orr     r0, r0, #0x400          @ set AP

    /*                                                  */
    /* start crunching through the MMUAddressTable[]    */
    /*                                                  */
    /* r2 temporarily holds MMUAddressTable[VA]         */
    /* r3 temporarily holds MMUAddressTable[PHY]        */
    /* r4 temporarily holds MMUAddressTable[#MB]        */
    /* r5 cachable                                      */
25:
    ldr     r2, [r1], #4            @ (r2) = virtual, Cached address to map Bank at
    ldr     r3, [r1], #4            @ (r3) = physical address to map from
    ldr     r4, [r1], #4            @ (r4) = num MB to map
    ldr     r5, [r1], #4            @ (r5) = cachable

    mov     r4, r4, lsr #20         @ convert to MB unit

    cmp     r5, #0x0
    biceq   r0, r0, #0x0C           @ clear cachable & bufferable bits in Descriptor (clear C&B bits)

    cmp     r4, #0                  @ End of table?
    beq     29f

    /*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
    /* (r2) holds the &descriptor                                                      */
    /* (r0) holds the actual section descriptor                                        */
        
    /* Create Descriptor Address */
    ldr     r6, =0xFF000000
    orr     r6, r6, #0x00F00000     @ r6 = 0xFFF00000
    and     r2, r2, r6              @ only VA[31:20] are valid
    add     r2, r9, r2, LSR #18     @ build the descriptor address:  r2 = (TTB[31:14} | VA[31:20] >> 18)

    /* Create the Descriptor */
    and     r3, r3, r6              @ only PA[31:20] are valid for the descriptor@ rest will be static
    orr     r0, r3, r0              @ build the descriptor: r0 = (PA[31:20] | the rest of the descriptor)

    /* Store the descriptor at the proper (phy) address */
28:
    str     r0, [r2], #4
    add     r0, r0, #0x00100000     @ (r0) = section descriptor for the next 1MB mapping (just add 1MB)
        
    sub     r4, r4, #1              @ Decrement number of MB left 
    cmp     r4, #0
    bne     28b                     @ Map next MB

    b       20b                     @ Get and process next table element                

29:



ACTIVATEMMU:
    /* The 1st Level Section Descriptors are setup. Initialize the MMU and turn it on. */
    mov     r1, #3
    mcr     p15, 0, r1, c3, c0, 0   @ Set up access to domain 0.
    mcr     p15, 0, r0, c8, c7, 0   @ Flush the instruction and data TLBs.
    mcr     p15, 0, r1, c7, c10, 4  @ Drain the write and fill buffers.

    mov     r1, #0x78               @ Bits [6:3] must be written as 1's.
    orr     r1, r1, #0x1            @ Enable MMU.
    orr     r1, r1, #0x1000         @ Enable IC.
    orr     r1, r1, #0x0800         @ Enable BTB.
    orr     r1, r1, #0x4            @ Enable DC.

    ldr     r2, =VirtualStart       @ Get virtual address of 'VirtualStart' label.
    @cmp     r2, #0                  @ Make sure no stall on "mov pc,r2" below.
	@beq		STALL2

    /* Enable the MMU. */
    mcr     p15, 0, r1, c1, c0, 0   @ MMU ON:  All memory accesses are now virtual.

    /* Jump to the virtual address of the 'VirtualStart' label. */
    mov     pc, r2
    nop
    nop
    nop

    /*=========================================================================*/
    /* The MMU and caches are now enabled and we're running in a virtual       */
    /* address space.                                                          */

VirtualStart:
#endif



setupMemory:
    add     r0, pc, #(Lstart - . - 8)
    ldmia   r0, {r0, r1, r2, r3}    /* load the memory usage. */
                                    /* r0 : _etext
                                       r1 : _sdata
                                       r2 : _edata
                                       r3 : _end
                                     */

@    /* copy the data section */
@1:
@    sub     r4, r2, r1
@    ldr     r5, [r0], #4
@    str     r5, [r1], #4
@    subs    r4, r4, #4
@    bgt     1b

    /* zero the bss section */
    sub     r4, r3, r2
    mov     r5, #0

10:
    str     r5, [r2], #4
    subs    r4, r4, #4
    bgt     10b


    @mov     sp, #0x20000000     /* temporary stack */
    @add     sp, sp, #0x00800000 /* temporary stack */
    mov     sp,  #0x06000000 /* temporary stack */

    
    mov     r5, r3              /* for keeping _end */

    bl      PanicInit

    mov     r0, r5
    bl      KernelLoad
    mov     r5, r0              /* the return value is the start address of the nanokernel */
 
    add     r4, pc, #(LNanoKernelStart - . - 8)
    ldmia   r4, {r0, r1, r2, r3, r4}    /* load the nanokernel start parameters */
                                        /*
                                         * r0 : nanokernel hardware adaptation data pointer
                                         * r1 : Idle thread configuration pointer
                                         * r2 : micorkernel servers(threads) configuration pointer
                                         * r3 : nanokernel stack pointer
                                         * r4 : nanokernel start pointer
                                         */

@    /* copy the micro kernel exeption table */
@    mov     r5, #0
@    @mov     r4, #0x3FFFE000    @nano-kernel start address
@    mov     r8, r4
@    mov     r6, #16
@1:
@    ldr     r7, [r8], #4
@    str     r7, [r5], #4
@    subs    r6, r6, #1
@    @tst     r6, #0
@    bne     1b
    
    mov     lr, pc
@    mov     pc, r4
    mov     pc, r5
    /* Never gets here */

    
    b    .

LNanoKernelStart:
        .word   HalBoot                  /* nanokernel hardware adaptation data pointer */
        .word   IdleConfig               /* Idle thread configuration */
        .word   KernelConfig             /* microkernel/microkernel servers(threads) configuration */
        .word   0x00004000               /* nanokernel stack pointer */

@Lendofram:
@    .word   _endofram

Lstart:
    .word   _etext
    .word   _sdata
    .word   _edata
    .word   _end



/* End of locore.S */
