/*	$ JBOSN : hal_main.c, v 0.1 10-12-03 15:13 $	*/

/*
 * Copyright (C) 2003	iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: hal_main.c
 *
 * Purpose: JBOSN Operating System Configuration and start .
 *
 */

#include <types.h>
#include <fs.h>
#include <thread.h>
#include <time.h>
#include <hal.h>
#include <nano_boot.h>

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3Xbd.h>
#include <mach/gpio.h>
#include <mach/hal_interrupt.h>

#include <lib/string.h>
#include <hal_panic.h>

//==========================================================================
typedef struct _HAL_DEVICE_ID_HW_
{
    HAL_DEVICE_ID       HalDeviceId;
    BYTE                String[48];
}HAL_DEVICE_ID_HW;


STATIC HAL_DEVICE_ID_HW HalDeviceIdHw = 
{
    {sizeof(HAL_DEVICE_ID_HW), sizeof(HAL_DEVICE_ID), 33, sizeof(HAL_DEVICE_ID) + 33+1, 13},
    "AT91SAMA5D3x dooroos.realtime BSP IBOSN SYSTEMS"
};

#define IOCTRL_HAL_WATCHDOG_MASK        ( 249|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_HAL_WATCHDOG_SET         ( 250|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_HAL_WATCHDOG_GET         ( 251|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))

//#define IOCTRL_HAL_BL                    ( 254|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
//#define IOCTRL_HAL_UHCPOWER             ( 252|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_HAL_DEBUGMSG             ( 253|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
#define IOCTRL_HAL_RESTART              ( 254|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))

//==========================================================================


/*
  Name: HAL_InterruptInit
  Description: ref. hal_interrupt.c
*/
EXTERN VOID HAL_InterruptInit( VOID );



STATIC VOID
HAL_MiscInit( VOID )
{
	AT91C_BASE_PMC->PMC_PCER0 = 1 << SAMA5D3_ID_PIOA;
	AT91C_BASE_PMC->PMC_PCER0 = 1 << SAMA5D3_ID_PIOB;
	AT91C_BASE_PMC->PMC_PCER0 = 1 << SAMA5D3_ID_PIOC;
	AT91C_BASE_PMC->PMC_PCER0 = 1 << SAMA5D3_ID_PIOD;
	AT91C_BASE_PMC->PMC_PCER0 = 1 << SAMA5D3_ID_PIOE;

    //AT91F_PIO_CfgOutput( AT91C_BASE_PIOD,   AT91C_UHC_SLOT1_EN );
    //AT91F_PIO_CfgInput( AT91C_BASE_PIOB,    AT91C_UHC_SLOT1_FLG );
    //AT91F_PIO_SetOutput( AT91C_BASE_PIOD,   AT91C_UHC_SLOT1_EN );
	//AT91F_PIO_ClearOutput( AT91C_BASE_PIOD, AT91C_UHC_SLOT1_EN );
    return;
}

/*
  Name: HAL_Init()
  Copyright: 
  Author: jung byoung oh
  Date: 10-12-03 15:19
  Description: initialize hardware
*/
STATIC VOID
HAL_Init( VOID )
{
    HAL_MiscInit( );
    HAL_InterruptInit( );


    return;
}

EXTERN JB_FONT_EXPORT   TTF_FontExport;
PUBLIC UINT32 WatchDog_mask=0;
PUBLIC UINT32 WatchDog_mask_set=0; 

STATIC UINT32
HAL_Control( UINT32 uCmd, PVOID pInBuf, UINT32 nInBufSize, PVOID pOutBuf, UINT32 nOutBufSize )
{
    UINT32  vRet = 0;
    switch( uCmd )
    {
		case IOCTRL_HAL_GET_DEVICE_INFO:
			break;

		case IOCTRL_HAL_GET_DEVICE_ID:
			if( nOutBufSize > sizeof(HAL_DEVICE_ID_HW) )
				nOutBufSize = sizeof(HAL_DEVICE_ID_HW);
			
			MemCpy( pOutBuf, &HalDeviceIdHw, nOutBufSize );
			vRet = nOutBufSize;
			break;
		case IOCTRL_HAL_SYSINTR_REQUEST:
		{
			extern BOOL OALIntrRequestSysIntr( SYSINTR_PREQUEST pSysIntrReq, PUINT32 pnSysIntr );

			if( pInBuf == NULL ||
				nInBufSize != sizeof(SYSINTR_REQUEST) ||
				pOutBuf == NULL /*||
				nOutBufSize != sizeof(UINT32)*/ )
				break;
//DEBUGMSG(1,"%s: nOutBufSize : %d\n",__FUNCTION__, nOutBufSize);

			vRet = OALIntrRequestSysIntr( (SYSINTR_PREQUEST)pInBuf, (PUINT32)pOutBuf);
			//DEBUGMSG(1,"%s: vRet : %d\n",__FUNCTION__, vRet);
			break;
		}

			
		case IOCTRL_HAL_SYSINTR_RELEASE:
		{
			extern BOOL OALIntrReleaseSysIntr( UINT32 nSysIntr );
			vRet = OALIntrReleaseSysIntr((UINT32)pInBuf);
			break;
		}
			
		case IOCTRL_HAL_PROCESSOR_INFO:
			break;

#if 0
        case IOCTRL_HAL_BL:
        {
            EXTERN UINT32 nPwmValue;
            nPwmValue = *(UINT32 *)pInBuf;
kprintf("%s(%d): nPwmValue = %d\n", __FUNCTION__,__LINE__, nPwmValue );
            break;
        }
#endif
        case IOCTRL_HAL_WATCHDOG_MASK:
            WatchDog_mask |= *(UINT32*)pInBuf;
            //kprintf("WatchDog Mask=%X\n",App_mask);
            break;

        case IOCTRL_HAL_WATCHDOG_SET:
            WatchDog_mask_set |= *(UINT32*)pInBuf;
            //kprintf("WatchDog Set=%X\n",App_mask_set);
            break;

        case IOCTRL_HAL_WATCHDOG_GET:
        {
            //DEBUGMSG(1,"%s: IOCTRL_HAL_SYSINTR_REQUEST == PHYIRQ_PDMA0 : %x %x\n",__FUNCTION__, pInBuf, pOutBuf);
            //DEBUGMSG(1,"%s: IOCTRL_HAL_SYSINTR_REQUEST == PHYIRQ_PDMA0 : %x %x\n",__FUNCTION__, *(PUINT32)pInBuf, *(PUINT32)pOutBuf);
            //DEBUGMSG(1,"%s: IOCTRL_HAL_SYSINTR_REQUEST == PHYIRQ_PDMA0 : %d %d\n",__FUNCTION__, nInBufSize, nOutBufSize);
            vRet = WatchDog_mask_set;
            //DEBUGMSG(1,"%s: vRet : %d\n",__FUNCTION__, vRet);
            break;
        }

 
		case IOCTRL_HAL_REBOOT:
		{
            // reset the cpu
            AT91PS_RSTC pRstc = AT91C_BASE_RSTC;
            
            pRstc->RSTC_RCR = (0xA5<<24) | (AT91C_RSTC_PROCRST|AT91C_RSTC_PERRST|AT91C_RSTC_EXTRST);

#if 0
	#define SCCTRL		(SPEAR3XX_ICM3_SYS_CTRL_BASE + 0x00)
	#define SCSYSSTAT	(SPEAR3XX_ICM3_SYS_CTRL_BASE + 0x04)

		/* Going into slow mode before resetting SOC */
		*(PUINT32)SCCTRL = 0x02;

		/* Writing any value to the system status register will reset the SOC */	
		*(PUINT32)SCSYSSTAT = 0x00;

		/* system will restart */
		while(1);
#endif
		break;
		}

		case IOCTRL_HAL_DEBUGMSG:
        {
            EXTERN BOOL bDebug;
            bDebug = *(BOOL *)pInBuf;

            break;
            
        }
#if 0
        case IOCTRL_HAL_UHCPOWER:
        {
            UINT32 OnFlag = *(UINT32 *)pInBuf;
            if( OnFlag & (1<<0) )
                AT91F_PIO_SetOutput( AT91C_BASE_PIOB, (1<<11) );      // power on
            else
                AT91F_PIO_ClearOutput( AT91C_BASE_PIOB, (1<<11) );    // power off

            if( OnFlag & (1<<1) )
                AT91F_PIO_SetOutput( AT91C_BASE_PIOB, (1<<13) );      // power on
            else
                AT91F_PIO_ClearOutput( AT91C_BASE_PIOB, (1<<13) );    // power off

            break;
        }
#endif

		case IOCTRL_HAL_RESTART:
        {
            PSHARED_MEMORY  pSharedMemory = (PSHARED_MEMORY)SHARED_MEMORY_BASE;
            pSharedMemory->BootMode = WARM_BOOTMODE;

            AT91PS_AIC  pAIC = (AT91PS_AIC)AT91C_BASE_AIC;
            AT91PS_PMC  pPMC = (AT91PS_PMC)AT91C_BASE_PMC;
            pPMC->PMC_PCDR0 =   
                        //(1<<AT91C_ID_FIQ)            |
                        //(1<<AT91C_ID_SYS)            |
                        //(1<<SAMA5D3_ID_DBGU)         |
                        (1<<SAMA5D3_ID_PIT)          |
                        (1<<SAMA5D3_ID_WDT)          |
                        //(1<<SAMA5D3_ID_HSMC)         |
                        (1<<SAMA5D3_ID_PIOA)         |
                        (1<<SAMA5D3_ID_PIOB)         |
                        (1<<SAMA5D3_ID_PIOC)         |
                        (1<<SAMA5D3_ID_PIOD)         |
                        (1<<SAMA5D3_ID_PIOE)         |
                        (1<<SAMA5D3_ID_SMD)          |
                        (1<<SAMA5D3_ID_USART0)       |
                        (1<<SAMA5D3_ID_USART1)       |
                        (1<<SAMA5D3_ID_USART2)       |
                        (1<<SAMA5D3_ID_USART3)       |
                        (1<<SAMA5D3_ID_UART0)        |
                        (1<<SAMA5D3_ID_UART1)        |
                        (1<<SAMA5D3_ID_TWI0)         |
                        (1<<SAMA5D3_ID_TWI1)         |
                        (1<<SAMA5D3_ID_TWI2)         |
                        (1<<SAMA5D3_ID_HSMCI0)       |
                        (1<<SAMA5D3_ID_HSMCI1)       |
                        (1<<SAMA5D3_ID_HSMCI2)       |
                        //(1<<SAMA5D3_ID_SPI0)         |
                        (1<<SAMA5D3_ID_SPI1)         |
                        (1<<SAMA5D3_ID_TC0)          |
                        (1<<SAMA5D3_ID_TC1)          |
                        (1<<SAMA5D3_ID_PWM)          |
                        (1<<SAMA5D3_ID_ADC)          |
                        (1<<SAMA5D3_ID_DMA0)         |
                        (1<<SAMA5D3_ID_DMA1)         |
                        0;
            pPMC->PMC_PCDR1 = 
                        (1<<(SAMA5D3_ID_UHPHS - 32))        |
                        (1<<(SAMA5D3_ID_UDPHS - 32))        |
                        //(1<<(SAMA5D3_ID_GMAC - 32))         |
                        //(1<<(SAMA5D3_ID_EMAC - 32))         |
                        //(1<<(SAMA5D3_ID_LCDC - 32))         |
                        (1<<(SAMA5D3_ID_ISI - 32))          |
                        (1<<(SAMA5D3_ID_SSC0 - 32))         |
                        (1<<(SAMA5D3_ID_SSC1 - 32))         |
                        (1<<(SAMA5D3_ID_CAN0 - 32))         |
                        (1<<(SAMA5D3_ID_CAN1 - 32))         |
                        //(1<<(SAMA5D3_ID_SHA - 32))          |
                        //(1<<(SAMA5D3_ID_AES - 32))          |
                        //(1<<(SAMA5D3_ID_TDES - 32))         |
                        //(1<<(SAMA5D3_ID_TRNG - 32))         |
                        //(1<<(SAMA5D3_ID_ARM - 32))          |
                        //(1<<(SAMA5D3_ID_IRQ - 32))          |
                        //(1<<(SAMA5D3_ID_FUSE - 32))         |
                        //(1<<(SAMA5D3_ID_MPDDRC - 32))	    
                        0;

            int i;
            for (i = 0; i < IRQ_PHY_MAX_COUNT; i++) 
            {
                pAIC->AIC_SSR = i;

                /* Disable the interrupt first */
                pAIC->AIC_IDCR = AIC_IDCR_INTD;

                /* Configure mode and handler */
                pAIC->AIC_SVR = i;
                pAIC->AIC_SMR = AIC_SMR_PRIOR(0) | AIC_SMR_SRCTYPE_INT_LEVEL_SENSITIVE;

                /* Clear interrupt */
                pAIC->AIC_ICCR = AIC_ICCR_INTCLR;
                //pAIC->AIC_IMR = AIC_IMR_INTM;
            }
            WatchDog_mask = 0;

            extern void vector_reset(void);
            vector_reset();
            break;
        }
    }

    return vRet;

}

STATIC VOID
HAL_main( VOID )
{
#if 0
    HANDLE  hThread;

    kprintf("%s(%d): \n",__FUNCTION__,__LINE__);
    EXTERN THREADCONFIG NND_DevThread;
    hThread = ThreadCreate( &NND_DevThread );

    kprintf("%s(%d): hThread = %x\n",__FUNCTION__,__LINE__, hThread );
#endif

    return;
}


EXTERN UINT32 _endofram;
EXTERN INT32    app_main(VOID);

NKERNEL_BOOT    HalBoot = 
{
    40,
    (UINT32)(-1),
    (UINT32)&_endofram,
//    0x70400000,
//    0x72000000,
    HAL_Init,
    HAL_ClockInit,
    HAL_InterruptEnable,
    HAL_InterruptDisable,
    HAL_InterruptDone,
    HAL_InterruptHandler,
    HAL_TimerInterruptHandler,
    HAL_SetRealTime,
    HAL_GetRealTime,
    HAL_Control,
    HAL_PowerOff,
    HAL_DebugWriteByte,
    HAL_DebugReadByte,
    HAL_main,
    app_main
};

