/*	$ JBOSN : hal_time.c, v 0.1 10-12-03 15:12 $	*/

/*
 * Copyright (C) 2003	iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: hal_time.c
 *
 * Purpose: HAL basic initialization of time related hardware
 *
 */

#include <types.h>
#include <time.h>
#include <mach/hal_time.h>
#include <mach/hal_interrupt.h>

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3XBD.h>

#include <hal_panic.h>

//#define _USE_RTT_       1

struct _REAL_SYSTEM_TIME_
{
    union
    {
        SYSTEMTIME  SystemTime;
        UINT64      SystemTime64;
    }u;
};

STATIC struct _REAL_SYSTEM_TIME_ uSystemTime;


/*
  Name: MsCount
  Description:  ref. time.c in kernel_thread
                tick counter
*/
//EXTERN UINT64	MsCount;


PUBLIC UINT32           gnCountPerPeriod;

/*
  Name: HAL_ClockInit()
  Copyright: 
  Author: jung boung oh
  Date: 10-12-03 15:02
  Description: initialize hardware specific clock controller.
*/

PUBLIC VOID
HAL_ClockInit( VOID )
{
    AT91PS_PITC     pPITC =     (AT91PS_PITC)AT91C_BASE_PITC;
    AT91PS_AIC      pAIC =      (AT91PS_AIC)AT91C_BASE_AIC;
	//AT91PS_RTC      pRTC =      (AT91PS_RTC)AT91C_BASE_RTC;
	AT91PS_PMC      pPMC = (AT91PS_PMC)AT91C_BASE_PMC;


	UINT32  nMCKdiv16PerMSec, nCountPerPeriod;

//DEBUGMSG(1,"HAL_ClockInit started : period %d\r\n", TIMER_PERIOD);
//DEBUGMSG(1,"HAL_ClockInit started : pPITC = %x\r\n", pPITC);

    
    pPMC->PMC_PCER0 = (1 << ( SAMA5D3_ID_PIT));

    // MCK/16 per MSec
    //nMCKdiv16PerMSec = (BOARD_MCK / 1000) / 16;
    //gnCountPerPeriod = TIMER_PERIOD * nMCKdiv16PerMSec;
	//DEBUG2(1,"HAL_ClockInit started : nMCKdiv16PerMSec %d\r\n", nMCKdiv16PerMSec);
	//DEBUG2(1,"HAL_ClockInit started : gnCountPerPeriod %d\r\n", gnCountPerPeriod);

    gnCountPerPeriod = (TIMER_PERIOD * (BOARD_MCK/1000) + 8) >> 4;
    //gnCountPerPeriod = (TIMER_PERIOD * (BOARD_MCK/1000000) + 8) >> 4;
//DEBUGMSG(1,"HAL_ClockInit started : gnCountPerPeriod %d\r\n", gnCountPerPeriod);

	// Enable interrupt for the timer
	pPITC->PITC_PIMR = AT91C_PITC_PITEN | AT91C_PITC_PITIEN | (gnCountPerPeriod);	//enable interrupt and enable timer    

	// Enable the PIT IRQ
    //pAIC->AIC_IECR = (1<<SAMA5D3_ID_PIT);
    pAIC->AIC_SSR  = SAMA5D3_ID_PIT;
    pAIC->AIC_IECR = AIC_IECR_INTEN;
    //pAIC->AIC_IMR = AIC_IMR_INTM;

    // start RTC...
#if defined(_USE_RTT_)
	pRTTC->RTTC_RTMR = AT91C_RTTC_RTTRST | 0x8000;     // for 1Hz

	pRTTC->RTTC_RTMR &= ~(AT91C_RTTC_ALMIEN | AT91C_RTTC_RTTINCIEN); 
#endif
    uSystemTime.u.SystemTime64 = 0;
	// Enable the SYS IRQ
    // Enable System Tick interrupt
	//HAL_InterruptEnable( SYSINTR_TICK );
    OALIntrRequestSystemTimer(SAMA5D3_ID_PIT);
    HAL_InterruptEnable( SYSINTR_TICK );

//    DEBUG1(1,"HAL_ClockInit ended\n");

#if 0
for(;;)
{
    DEBUGMSG(1,"HAL_ClockInit started : pAIC->AIC_IVR   = %x\r\n", pAIC->AIC_IVR);
    DEBUGMSG(1,"HAL_ClockInit started : pPITC->PITC_PIVR = %x\r\n", pPITC->PITC_PIVR);
    DEBUGMSG(1,"HAL_ClockInit started : pAIC->AIC_IPR0   = %x\r\n", pAIC->AIC_IPR0);
    DEBUGMSG(1,"HAL_ClockInit started : pAIC->AIC_IPR1   = %x\r\n", pAIC->AIC_IPR1);
    DEBUGMSG(1,"HAL_ClockInit started : pAIC->AIC_IPR2   = %x\r\n", pAIC->AIC_IPR2);
    DEBUGMSG(1,"HAL_ClockInit started : pAIC->AIC_IPR3   = %x\r\n", pAIC->AIC_IPR3);
}
#endif
    return;
}

/*
  Name: HAL_TimerInterruptHandler
  Copyright: 
  Author: jung byoung oh
  Date: 10-12-03 15:04
  Description: tick timer processing handler.
*/

PUBLIC VOID
HAL_TimerInterruptHandler( PUINT64 pMsCount )
{
    REGISTER UINT32 nElapsedTime;
    AT91PS_PITC     pPITC = (AT91PS_PITC)AT91C_BASE_PITC;

    UINT32 dwPIVR = pPITC->PITC_PIVR;

    nElapsedTime = dwPIVR >> 20;
    nElapsedTime = TIMER_PERIOD*nElapsedTime;

    *pMsCount += nElapsedTime;

//DEBUGMSG(1,"HAL_TimerInterruptHandler\n");

#if 0
static int it = 0 ;
it++;
if( !(it % 100) )
kprintf("pMsCount = %p: %x\n", pMsCount, (UINT32)*pMsCount );
#endif

#if !defined(_USE_RTT_)
    uSystemTime.u.SystemTime64 += nElapsedTime;
#endif

    return;
}


PUBLIC VOID
HAL_GetRealTime( PSYSTEMTIME pSystemTime )
{
#if defined(_USE_RTT_)
    UINT32  RtcVal;
	struct _REAL_SYSTEM_TIME_  sSystemTime;
	AT91PS_RTTC     pRTTC =    (AT91PS_RTTC)AT91C_BASE_RTTC;

	RtcVal = pRTTC->RTTC_RTVR;

    sSystemTime.u.SystemTime64 = (UINT64)RtcVal * 1000;
    sSystemTime.u.SystemTime64 += uSystemTime.u.SystemTime64;

#endif

    *pSystemTime = uSystemTime.u.SystemTime;
    //p_SystemTime->Sec = READ_REGISTER_ULONG(RTC_RTCDR);


    return;
}



PUBLIC VOID
HAL_SetRealTime( PSYSTEMTIME pSystemTime )
{

    uSystemTime.u.SystemTime = *pSystemTime;
    //WRITE_REGISTER_ULONG(RTC_RTCDR, p_SystemTime->Sec);

	
#if defined(_USE_RTT_)
{
	AT91PS_RTTC     pRTTC =    (AT91PS_RTTC)AT91C_BASE_RTTC;
	//restart the RTT
	pRTTC->RTTC_RTMR = AT91C_RTTC_RTTRST | 0x8000 ; // reset and 1Hz
}
#endif

    
    return;
}
