//-----------------------------------------------------------------------------
//! \addtogroup	OAL
//! @{
//!
//  All rights reserved ADENEO SAS 2005
//!
//-----------------------------------------------------------------------------
//! \file		pll.c
//!
//! \brief		PLL management for the AT91SAM926x processors
//!
//! \if subversion
//!   $URL: http://centaure/svn/interne-ce_atmel_common/TAGS/V150-Nadia1-19-Sept-2006/AT91SAM926x/KERNEL/PLL/pll.c $
//!   $Author: vbelloir $
//!   $Revision: 313 $
//!   $Date: 2006-09-14 13:37:56 +0200 (jeu., 14 sept. 2006) $
//! \endif
//-----------------------------------------------------------------------------

#include <types.h>

#include <mach/SAMA5D3X.h>

#include <panic.h>

#if 0

#define PLL_FREQUENCY_RANGE_LIMIT   195000000


static void ComputeMulAndDiv(DWORD dwSrcFreqInHz,DWORD dwDestFreqInHz,DWORD *pMul,DWORD *pDiv)
{
	DWORD dwMul,dwDiv;
	DWORD dwDestFreqInKHz = dwDestFreqInHz / 1000;
	DWORD dwSrcFreqInKHz = dwSrcFreqInHz / 1000;
	
	dwDiv = 255 * 2;	
	do
	{
		dwDiv /= 2;
		dwMul = (dwDestFreqInKHz * dwDiv) / dwSrcFreqInKHz;
		
	} while (dwMul > 2048);
	
	*pMul = dwMul;
	*pDiv = dwDiv;
}

DWORD GetMainClockFreq(AT91PS_PMC pPMC)
{
	if (pPMC->PMC_MCFR & AT91C_CKGR_MAINRDY)
	{		
		return ((pPMC->PMC_MCFR & AT91C_CKGR_MAINF) * (SLOW_CLOCK_FREQUENCY)) / 16;
	}
	else
	{
		return 0;
	}
}

DWORD GetPLLAFreq(AT91PS_PMC pPMC)
{
    DWORD dwMainClk;
    DWORD dwMUL,dwDIV;

	
	dwMainClk = GetMainClockFreq(pPMC);
    dwMUL = (pPMC->PMC_PLLAR & AT91C_CKGR_MULA) >> 16;
    dwDIV = (pPMC->PMC_PLLAR & AT91C_CKGR_DIVA) >> 0;
    
    if (dwDIV == 0)
        return 0;

    return (dwMainClk / dwDIV) * (dwMUL+1);
}

DWORD GetPLLBFreq(AT91PS_PMC pPMC)
{

    DWORD dwMainClk;
    DWORD dwMUL,dwDIV;

	dwMainClk = GetMainClockFreq(pPMC);

    dwMUL = (pPMC->PMC_PLLBR & AT91C_CKGR_MULB) >> 16;
    dwDIV = (pPMC->PMC_PLLBR & AT91C_CKGR_DIVB) >> 0;
    
    if (dwDIV == 0)
        return 0;

    return (dwMainClk  / dwDIV) * (dwMUL+1);
}

//------------------------------------------------------------------------------
//
//  Function:     GetPLLAClock
//
//  This function is called by the kernel to retreive the value of the PLLA clock in Hz
//

DWORD AT91SAM926x_GetPLLAClock(BOOL bUseShadowedValue)
{
#ifdef IS_MISTRAL
	return FALSE;  	
#else
	
    static DWORD dwPLLAClock = 0xFFFFFFFF;

    if ((dwPLLAClock == 0xFFFFFFFF) || (!bUseShadowedValue))
    {        
        	    
		AT91PS_PMC pPMC;
		pPMC = (AT91PS_PMC)AT91C_BASE_PMC;
		dwPLLAClock = GetPLLAFreq(pPMC);
    }           
	DEBUG2(1,"PLLA Clock is %d Hz\r\n",dwPLLAClock);
    return dwPLLAClock; 
#endif
}

//------------------------------------------------------------------------------
//
//  Function:     GetPLLBClock
//
//  This function is called by the kernel to retreive the value of the PLLB clock in Hz
//

DWORD AT91SAM926x_GetPLLBClock(BOOL bUseShadowedValue)
{
#ifdef IS_MISTRAL
	return FALSE;  	
#else
    static DWORD dwPLLBClock = 0xFFFFFFFF;

    if ((dwPLLBClock == 0xFFFFFFFF) || (!bUseShadowedValue))
    {        
        	    
		AT91PS_PMC pPMC;
		pPMC = (AT91PS_PMC)AT91C_BASE_PMC;
		dwPLLBClock = GetPLLBFreq(pPMC);
    }           
	DEBUG2(1,"PLLB Clock is %d Hz\r\n",dwPLLBClock);
    return dwPLLBClock;
#endif
}

//------------------------------------------------------------------------------
//
//  Function:     GetProcessorClock
//
//  This function is called by the kernel to retreive the value of the Processor clock in Hz
//

DWORD AT91SAM926x_GetProcessorClock(BOOL bUseShadowedValue)
{
#ifdef IS_MISTRAL
	return FALSE;  	
#else
	  static DWORD dwProcessorClock = 0xFFFFFFFF;

    if ((dwProcessorClock == 0xFFFFFFFF) || (!bUseShadowedValue))
    {        
		DWORD dwInputFreq;	    
		AT91PS_PMC pPMC;
		pPMC = (AT91PS_PMC)AT91C_BASE_PMC;
		switch (pPMC->PMC_MCKR & AT91C_PMC_CSS)
		{
			case AT91C_PMC_CSS_SLOW_CLK: dwInputFreq = SLOW_CLOCK_FREQUENCY; break;
			case AT91C_PMC_CSS_MAIN_CLK: dwInputFreq = GetMainClockFreq(pPMC); break;
			case AT91C_PMC_CSS_PLLA_CLK: dwInputFreq = GetPLLAFreq(pPMC); break;
			case AT91C_PMC_CSS_PLLB_CLK: dwInputFreq = GetPLLBFreq(pPMC); break;
		}

        if (dwInputFreq != 0)
		{
			dwProcessorClock = dwInputFreq >> ((pPMC->PMC_MCKR & AT91C_PMC_PRES)>>2);
		}
    }           
	DEBUG2(1,"Processor Clock is %d Hz\r\n",dwProcessorClock);
    return dwProcessorClock;
#endif
}

//------------------------------------------------------------------------------
//
//  Function:     GetMasterClock
//
//  This function is called by the kernel to retreive the value of the master clock in Hz
//

DWORD AT91SAM926x_GetMasterClock(BOOL bUseShadowedValue)
{        
#ifdef IS_MISTRAL
	return 25000000;  	
#else

    static DWORD dwMasterClock = 0xFFFFFFFF;

    if ((dwMasterClock == 0xFFFFFFFF) || (!bUseShadowedValue))
    {        
        
		DWORD temp;
		DWORD dwInputFreq;	    
		AT91PS_PMC pPMC;
		pPMC = (AT91PS_PMC)AT91C_BASE_PMC;
		switch (pPMC->PMC_MCKR & AT91C_PMC_CSS)
		{
			case AT91C_PMC_CSS_SLOW_CLK: dwInputFreq = SLOW_CLOCK_FREQUENCY; break;
			case AT91C_PMC_CSS_MAIN_CLK: dwInputFreq = GetMainClockFreq(pPMC); break;
			case AT91C_PMC_CSS_PLLA_CLK: dwInputFreq = GetPLLAFreq(pPMC); break;
			case AT91C_PMC_CSS_PLLB_CLK: dwInputFreq = GetPLLBFreq(pPMC); break;
		}

        if (dwInputFreq != 0)
		{
			temp = dwInputFreq >> ((pPMC->PMC_MCKR & AT91C_PMC_PRES)>>2);
			dwMasterClock = temp >> ((pPMC->PMC_MCKR & AT91C_PMC_MDIV) >> 8);
		}
		        
    }           
	DEBUG2(1,"Master Clock is %d Hz\r\n",dwMasterClock);
    return dwMasterClock;    
#endif
}

static UINT32 
log2(DWORD dw)
{
    DWORD log2=0;
	while (dw)
    {
	    log2++;
        dw >>= 1;
    }
	return log2-1;
}

VOID
AT91SAM926x_TurnProcessorClockOff( VOID )
{
	AT91PS_PMC pPMC;
	pPMC = (AT91PS_PMC)AT91C_BASE_PMC;
	pPMC->PMC_SCDR = AT91C_PMC_PCK;
}

DWORD AT91SAM926x_SetPLLBFreq(AT91PS_PMC pPMC,DWORD dwFreqInMHz, DWORD dwUSBDivider)
{
#ifdef IS_MISTRAL
	return 0;
#else

    DWORD dwTemp = 0;
    DWORD dwFreq,dwMainClock;  	
	DWORD dwDiv;
	DWORD dwMul;
	
	dwMainClock = GetMainClockFreq(pPMC);
	ComputeMulAndDiv(dwMainClock,dwFreqInMHz * 1000000,&dwMul,&dwDiv);
        
    dwTemp |= (dwMul << 16) | (dwDiv);    
    dwTemp |= dwUSBDivider;
        
        
    dwFreq = (dwMainClock / dwDiv) * (dwMul+1);    
    if (dwFreq < PLL_FREQUENCY_RANGE_LIMIT)
    {
        dwTemp |= AT91C_CKGR_OUTB_0;
    }
    else
    {
        dwTemp |= AT91C_CKGR_OUTB_2;
    }
    
    dwTemp |= (AT91C_CKGR_PLLBCOUNT & (0x3FF << 8)); //Maximum setup time (we don't really know the characteristics of the PLL filter implemented on the board).
    
    
    pPMC->PMC_PLLBR = dwTemp;
    
    while ((pPMC->PMC_SR & AT91C_PMC_LOCKB) == 0);    
    
    return 0;
#endif
}

#define DEFAULT_BUS_RATIO	4
#define MAX_PRESCALER		6

#define PLL_OUT_RANGE_MIN	80

BOOL AT91SAM926x_SetProcessorAndMasterClocks(	AT91PS_PMC pPMC				,
												DWORD dwProcessorClockInMHz	,
												DWORD dwBusClockRatio		)
{
#ifdef IS_MISTRAL
	return TRUE;
#else
	DWORD PLLA_CountBeforeStable;
	DWORD PLLA_Divider;
	DWORD PLLA_Multipler;
	DWORD PLLA_BusRatio;
	DWORD dwTemp;
	DWORD dwPrescaler = 0;
	
	DWORD dwTargetPLLFrequencyInMHz = dwProcessorClockInMHz;


	kprintf("Target processor Frequency is %d\r\nTarget Bus Frequency is %d\r\n",dwProcessorClockInMHz,dwProcessorClockInMHz/dwBusClockRatio);

	/* Compute the prescaler if needed */
	while (dwTargetPLLFrequencyInMHz < PLL_OUT_RANGE_MIN)
	{
		dwPrescaler++;
		if (dwPrescaler > MAX_PRESCALER)
		{
			return FALSE;
		}
		dwTargetPLLFrequencyInMHz = dwProcessorClockInMHz * (1 << dwPrescaler);
	}

	// Clip the selected bus ratio
	if ((dwBusClockRatio != 1) && (dwBusClockRatio != 2) && (dwBusClockRatio != 4))
	{
		kprintf("AT91SAM926x_SetProcessorAndMasterClocks : Invalid bus ratio (%d). Setting default (%d)\r\n",dwBusClockRatio,DEFAULT_BUS_RATIO);
		dwBusClockRatio = 4;
	}


	PLLA_CountBeforeStable	= 0x100;	
	PLLA_BusRatio			= 0;

	kprintf("2 Prescaler is %d\r\nTarget PLL A Frequency is %d MHz\r\n",dwPrescaler,dwTargetPLLFrequencyInMHz);

	{
		// Small tempo to let the Chip a bit time to settle.
		// Someow it's no good to write characters on the serial before a bit.
		volatile DWORD dwTempo = 10000;
		while (dwTempo--);
	}

	ComputeMulAndDiv(GetMainClockFreq(pPMC), dwTargetPLLFrequencyInMHz*1000000, &PLLA_Multipler, &PLLA_Divider);	


	// Switch MasterClock To SlowClock, with a prescaler of 1 and a bus divider of 2
    pPMC->PMC_MCKR = AT91C_PMC_MDIV_2 | AT91C_PMC_PRES_CLK | AT91C_PMC_CSS_SLOW_CLK;

	// Waiting that MasterClock is stable
   	while ((pPMC->PMC_SR & AT91C_PMC_MCKRDY) == 0);

	// PLLA Configuration
	dwTemp = AT91C_CKGR_SRCA | (PLLA_Multipler<<16) | (PLLA_CountBeforeStable<<8) | PLLA_Divider;

	if ((dwTargetPLLFrequencyInMHz*1000000) < PLL_FREQUENCY_RANGE_LIMIT)
    {
        dwTemp |= AT91C_CKGR_OUTA_0;
    }
    else
    {
        dwTemp |= AT91C_CKGR_OUTA_2;
    }

	pPMC->PMC_PLLAR = dwTemp;
	
	// Waiting for PLLA to be stable
	while ((pPMC->PMC_SR & AT91C_PMC_LOCKA) == 0); 

	// Selecting the right Bus Ratio
	switch (dwBusClockRatio)
	{
		case 1:
			PLLA_BusRatio = AT91C_PMC_MDIV_1;
			break;
		case 2:
			PLLA_BusRatio = AT91C_PMC_MDIV_2;
			break;
		case 4:
			PLLA_BusRatio = AT91C_PMC_MDIV_3;
			break;
		default:
			PLLA_BusRatio = AT91C_PMC_MDIV_3;
	}

	// Switch MasterClock to PLLA
	pPMC->PMC_MCKR = PLLA_BusRatio | AT91C_PMC_CSS_PLLA_CLK | (dwPrescaler << 2);

	// Waiting fo MasterClock to be locked
	while ((pPMC->PMC_SR & AT91C_PMC_MCKRDY) == 0);

    return TRUE;
#endif
}


#endif

//------------------------------------------------------------------------------
//! @}


////////////////////////////////////////////////////////////////////////////////
// End of $URL: http://centaure/svn/interne-ce_atmel_common/TAGS/V150-Nadia1-19-Sept-2006/AT91SAM926x/KERNEL/PLL/pll.c $
////////////////////////////////////////////////////////////////////////////////
//
// History : $Log$
// History : Revision 1.25  2006/04/11 15:03:40  jjhiblot
// History : fixed bug (it was previously unable to use a divider of 1)
// History :
// History : Revision 1.24  2006/03/27 08:18:32  VBelloir
// History : Delete the limitation of clock frequency
// History :
// History : Revision 1.23  2006/03/16 14:36:24  ngocel
// History : Add IOCTL to get PLLA, PLLB, and processor clock
// History :
// History : Revision 1.22  2006/03/02 16:18:08  jjhiblot
// History : typo fix
// History :
// History : Revision 1.21  2006/01/13 17:20:06  jjhiblot
// History : clean up
// History :
// History : Revision 1.20  2005/12/09 17:28:01  jjhiblot
// History : MISTRAL special PLL mgnt
// History :
// History : Revision 1.19  2005/11/18 15:18:52  jjhiblot
// History : Enhanced and fixed doxygen support
// History :
// History : Revision 1.18  2005/11/02 08:31:49  jjhiblot
// History : *** empty log message ***
// History :
// History : Revision 1.17  2005/10/27 09:05:09  jjhiblot
// History : fix compilation error
// History :
// History : Revision 1.16  2005/10/27 09:00:37  jjhiblot
// History : Added Prescaler support
// History : Fixed some bugs
// History :
// History : Revision 1.15  2005/10/17 13:50:04  lvilquin
// History : management of output frequency of PLL
// History :
// History : Revision 1.14  2005/09/20 14:55:15  dmartin
// History : *** empty log message ***
// History :
// History : Revision 1.13  2005/09/09 15:37:31  jjhiblot
// History : *** empty log message ***
// History :
// History : Revision 1.12  2005/08/02 13:00:21  dmartin
// History : *** empty log message ***
// History :
// History : Revision 1.11  2005/07/21 09:07:47  dmartin
// History : DMR : Doxy+CVS
// History :
// History : Revision 1.10  2005/07/19 08:02:36  jjhiblot
// History : Modified CVS headers to work with dOxygen
// History :
// History : Revision 1.9  2005/07/11 12:45:44  dmartin
// History : DMR : ChipSelect support extracted from PLL lib
// History :
// History : Revision 1.8  2005/07/11 08:48:10  dmartin
// History : *** empty log message ***
// History :
// History : Revision 1.7  2005/07/06 17:16:16  jjhiblot
// History : Modified CVS headers
// History :
// History : Revision 1.6  2005/07/06 14:44:20  jjhiblot
// History : Added :
// History : Watchdog support
// History : Shutdown support
// History : Real RTC support (using RTT and backup registers)
// History :
// History : Revision 1.5  2005/07/01 09:45:03  dmartin
// History : DMR :
// History : Improve PLL settings
// History : Added ChipSelect bus timing
// History :
// History : Revision 1.4  2005/06/30 12:39:07  dmartin
// History : DMR :
// History : Bug fix in RTC
// History : PLL support improved
// History : Processor Clock Selection
// History : USB Host added
// History :
// History : Revision 1.3  2005/05/27 14:57:13  jjhiblot
// History : *** empty log message ***
// History :
// History : Revision 1.2  2005/05/24 17:07:39  jjhiblot
// History : *** empty log message ***
// History :
// History : Revision 1.1  2005/05/17 08:34:12  jjhiblot
// History : -NadiaX support
// History : -AT91SAM9261EK BSP
// History :

