/*//                                                                              */
/*// Copyright (c) Microsoft Corporation.  All rights reserved.                   */
/*//                                                                              */
/*//                                                                              */
/*// Use of this source code is subject to the terms of the Microsoft end-user    */
/*// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.     */
/*// If you did not accept the terms of the EULA, you are not authorized to use   */
/*// this source code. For a copy of the EULA, please see the LICENSE.RTF on your */
/*// install media.                                                               */
/*//                                                                              */
/*//------------------------------------------------------------------------------*/
/*//                                                                              */
/*//  Header:  oal_cache.h                                                        */
/*//                                                                              */
/*//  This header file defines OAL cache module interface. The module implements  */
/*//  functions and varibles related to cache and TLB operations.                 */
/*//                                                                              */
/*//  Export for kernel/public interface:                                         */
/*//      * OEMCacheRangeFlush                                                    */
/*//      * OEMIoControl/IOCTL_HAL_GET_CACHE_INFO                                 */
/*//                                                                              */
/*//  Export for other OAL modules/protected interface:                           */
/*//      * OALCacheGlobalsInit                                                   */
/*//                                                                              */
/*//  Intenal variables/functions                                                 */
/*//      * g_oalCacheInfo                                                        */
/*//      * OALClearTLB/OALClearTLBEntry                                          */
/*//      * OALClearITLB/OALClearITLBEntry                                        */
/*//      * OALClearDTLB/OALClearDTLBEntry                                        */
/*//                                                                              */
#ifndef __OAL_CACHE_H
#define __OAL_CACHE_H


#ifdef __ASSEMBLY__

#define L1Flags		    0
#define L1ISetsPerWay   4
#define L1INumWays      8
#define L1ILineSize     12
#define L1ISize         16
#define L1DSetsPerWay   20
#define L1DNumWays      24
#define L1DLineSize     28
#define L1DSize         32

#define L2Flags         36
#define L2ISetsPerWay   40
#define L2INumWays      44
#define L2ILineSize     48
#define L2ISize         52
#define L2DSetsPerWay   56
#define L2DNumWays      60
#define L2DLineSize     64
#define L2DSize         68

#else

/* Flags for CacheSync/CacheRangeFlush */
#define CACHE_SYNC_DISCARD      0x001   /* write back & discard all cached data */
#define CACHE_SYNC_INSTRUCTIONS 0x002   /* discard all cached instructions */
#define CACHE_SYNC_WRITEBACK    0x004   /* write back but don't discard data cache*/
#define CACHE_SYNC_FLUSH_I_TLB  0x008   /* flush I-TLB */
#define CACHE_SYNC_FLUSH_D_TLB  0x010   /* flush D-TLB */
#define CACHE_SYNC_FLUSH_TLB    (CACHE_SYNC_FLUSH_I_TLB|CACHE_SYNC_FLUSH_D_TLB)    /* flush all TLB */
#define CACHE_SYNC_L2_WRITEBACK 0x020   /* write-back L2 Cache */
#define CACHE_SYNC_L2_DISCARD   0x040   /* discard L2 Cache */

#define CACHE_SYNC_ALL          0x07F   /* sync and discard everything in Cache/TLB */

#define PAGE_SIZE               0x1000

/*------------------------------------------------------------------------------ */
/*                                                                               */
/*  Structure:  OAL_CACHE_INFO                                                   */
/*                                                                               */

typedef struct {

    unsigned int L1Flags;
    unsigned int L1ISetsPerWay;
    unsigned int L1INumWays;
    unsigned int L1ILineSize;
    unsigned int L1ISize;
    unsigned int L1DSetsPerWay;
    unsigned int L1DNumWays;
    unsigned int L1DLineSize;
    unsigned int L1DSize;

    unsigned int L2Flags;
    unsigned int L2ISetsPerWay;
    unsigned int L2INumWays;
    unsigned int L2ILineSize;
    unsigned int L2ISize;
    unsigned int L2DSetsPerWay;
    unsigned int L2DNumWays;
    unsigned int L2DLineSize;
    unsigned int L2DSize;
    
} OAL_CACHE_INFO;

/*------------------------------------------------------------------------------*/
/*                                                                              */
/*  Global:  g_oalCacheInfo                                                     */
/*                                                                              */
extern OAL_CACHE_INFO g_oalCacheInfo;

/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Function:  OALIoCtlHalGetCacheInfo                                          */
/*////                                                                              */
/*////  This function implements OEMIoControl/IOCTL_HAL_GET_CACHE_INFO. It fill     */
/*////  CacheInfo structure with information about CPU/SoC cache.                   */
/*////                                                                              */
/*//BOOL OALIoCtlHalGetCacheInfo(UINT32, VOID*, UINT32, VOID*, UINT32, UINT32*);    */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Function:  OALCacheGlobalsInit                                              */
/*////                                                                              */
/*////  This function initializes globals variables which holds cache parameters.   */
/*////  It must be called before any other cache/TLB function.                      */
/*////                                                                              */
/*//VOID OALCacheGlobalsInit();                                                     */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Function:  OALFlushDCache/OALFlushDCacheLines                               */
/*////                                                                              */
/*////  This functions flush data cache (= write back to main memory modified       */
/*////  cache entries for write back cache and invalidate cache location).          */
/*////                                                                              */
/*//VOID OALFlushDCache();                                                          */
/*//VOID OALFlushDCacheLines(VOID* pAddress, UINT32 size);                          */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Function:  OALCleanDCache/OALCleanDCacheLines                               */
/*////                                                                              */
/*////  This functions clean data cache (= write back to main memory modified       */
/*////  cache entries for write back cache). It makes data in cache and main memory */
/*////  consistent.                                                                 */
/*////                                                                              */
/*//VOID OALCleanDCache();                                                          */
/*//VOID OALCleanDCacheLines(VOID* pAddress, UINT32 size);                          */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Function:  OALFlushICache/OALFlushICacheLines                               */
/*////                                                                              */
/*////  This functions flush instruction cache (= invalidate cache location).       */
/*////                                                                              */
/*//VOID OALFlushICache();                                                          */
/*//VOID OALFlushICacheLines(VOID* pAddress, UINT32 size);                          */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Function: OALClearTLB/OALClearTLBEntry                                      */
/*////                                                                              */
/*////  This functions clear full TLB or TLB entry. If system support separate      */
/*////  data and instruction TLBs both must be clear.                               */
/*////                                                                              */
/*//VOID OALClearTLB();                                                             */
/*//VOID OALClearTLBEntry(VOID*);                                                   */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Function: OALClearITLB/OALClearITLBEntry                                    */
/*////                                                                              */
/*////  This functions clear full instruction TLB or instruction TLB entry.         */
/*////                                                                              */
/*//VOID OALClearITLB();                                                            */
/*//VOID OALClearITLBEntry(VOID*);                                                  */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Function: OALClearDTLB/OALClearDTLBEntry                                    */
/*////                                                                              */
/*////  This functions clear full data TLB or data TLB entry.                       */
/*////                                                                              */
/*//VOID OALClearDTLB();                                                            */
/*//VOID OALClearDTLBEntry(VOID*);                                                  */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*//                                                                                */
/*//#if __cplusplus                                                                 */
/*//}                                                                               */
/*//#endif                                                                          */
/*//                                                                                */
/*//#else // __MIPS_ASSEMBLER                                                       */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*////                                                                              */
/*////  Defines: L1xxx/L2xxx                                                        */
/*////                                                                              */
/*////  Following defines are used in MIPS assember to access OAL_CACHE_INFO        */
/*////  structure members. It must be synchronized with OAL_CACHE_INFO definition.  */
/*////                                                                              */
/*//#define L1Flags             0                                                   */
/*//#define L1ISetsPerWay       4                                                   */
/*//#define L1INumWays          8                                                   */
/*//#define L1ILineSize         12                                                  */
/*//#define L1ISize             16                                                  */
/*//#define L1DSetsPerWay       20                                                  */
/*//#define L1DNumWays          24                                                  */
/*//#define L1DLineSize         28                                                  */
/*//#define L1DSize             32                                                  */
/*//                                                                                */
/*//#define L2Flags             36                                                  */
/*//#define L2ISetsPerWay       40                                                  */
/*//#define L2INumWays          44                                                  */
/*//#define L2ILineSize         48                                                  */
/*//#define L2ISize             52                                                  */
/*//#define L2DSetsPerWay       56                                                  */
/*//#define L2DNumWays          60                                                  */
/*//#define L2DLineSize         64                                                  */
/*//#define L2DSize             68                                                  */
/*//                                                                                */
/*////------------------------------------------------------------------------------*/
/*//                                                                                */
/*//#endif // __MIPS_ASSEMBLER                                                      */

#endif

#endif // __OAL_CACHE_H

