//-----------------------------------------------------------------------------
//! \addtogroup	OAL
//! @{
//!
//  All rights reserved ADENEO SAS 2005
//!
//-----------------------------------------------------------------------------
//! \file		ChipSelect.c
//!
//! \brief		ChipSelect management for the AT91SAM926x processors
//!
//! \if subversion
//!   $URL: http://centaure/svn/interne-ce_atmel_common/TAGS/V150-Nadia1-19-Sept-2006/AT91SAM926x/MISC/ChipSelect/ChipSelect.c $
//!   $Author: jjhiblot $
//!   $Revision: 278 $
//!   $Date: 2006-07-24 15:25:58 +0200 (lun., 24 juil. 2006) $
//! \endif
//-----------------------------------------------------------------------------


#include <types.h>
#include <mach/SAMA5D3X.h>
#include <hal_panic.h>

#define ROUND_UP_TO_NEAREST_MULTIPLE(x,y) ((((x) + (y) - 1) / (y)) * (y))


BOOL AT91SAM926x_SetChipSelectTimingIn_ns(	AT91PS_SMC pSMC			,
											DWORD dwChipSelect		,
											DWORD dwMasterClock		,
											DWORD dwNWE_SETUP		,
											DWORD dwNCS_WR_SETUP	,
											DWORD dwNRD_SETUP		,
											DWORD dwNCS_RD_SETUP	,
											DWORD dwNWE_PULSE		,										
											DWORD dwNCS_WR_PULSE	,
											DWORD dwNRD_PULSE		,
											DWORD dwNCS_RD_PULSE	,
											DWORD dwNRD_CYCLE		,
											DWORD dwNWE_CYCLE		)
{

	DWORD dwClockPeriod_ns	;

	DWORD temp				;

	DWORD dw_r_SETUP		;
	DWORD dw_r_NWE_SETUP	;
	DWORD dw_r_NCS_WR_SETUP	;
	DWORD dw_r_NRD_SETUP	;
	DWORD dw_r_NCS_RD_SETUP	;

	DWORD dw_r_PULSE		;
	DWORD dw_r_NWE_PULSE	;
	DWORD dw_r_NCS_WR_PULSE	;
	DWORD dw_r_NRD_PULSE	;
	DWORD dw_r_NCS_RD_PULSE	;

	DWORD dw_r_CYCLE		;
	DWORD dw_r_NRD_CYCLE	;
	DWORD dw_r_NWE_CYCLE	;

	// Compute the HOLD time
	DWORD dwNWE_HOLD		;
	DWORD dwNCS_WR_HOLD		;
	DWORD dwNDR_HOLD		;
	DWORD dwNCS_RD_HOLD		;

	AT91_REG* pSMC_SETUP	;
	AT91_REG* pSMC_PULSE	;	
	AT91_REG* pSMC_CYCLE	;
	AT91_REG* pSMC_CTRL		;

	

	// Compute the HOLD time
	dwNWE_HOLD		= dwNWE_CYCLE - dwNWE_SETUP		- dwNWE_PULSE;
	if ((int)dwNWE_HOLD <0)
	{
		dwNWE_HOLD = 0;
	}
	dwNCS_WR_HOLD		= dwNWE_CYCLE - dwNCS_WR_SETUP	- dwNCS_WR_PULSE;
	if ((int)dwNCS_WR_HOLD <0)
	{
		dwNCS_WR_HOLD = 0;
	}
	dwNDR_HOLD		= dwNRD_CYCLE - dwNRD_SETUP		- dwNRD_PULSE;
	if ((int)dwNDR_HOLD <0)
	{
		dwNDR_HOLD = 0;
	}
	dwNCS_RD_HOLD		= dwNRD_CYCLE - dwNCS_RD_SETUP	- dwNCS_RD_PULSE;
	if ((int)dwNCS_RD_HOLD <0)
	{
		dwNCS_RD_HOLD = 0;
	}

	pSMC_SETUP	= &(pSMC->SMC_SETUP0) + 4*dwChipSelect;
	pSMC_PULSE	= &(pSMC->SMC_PULSE0) + 4*dwChipSelect;
	pSMC_CYCLE	= &(pSMC->SMC_CYCLE0) + 4*dwChipSelect;
	pSMC_CTRL		= &(pSMC->SMC_CTRL0 ) + 4*dwChipSelect;

	kprintf("---------------------------------------\r\n");
	kprintf("--- Configuring Chip Select %d       ---\r\n",dwChipSelect);
	kprintf("---------------------------------------\r\n");
	kprintf("---         Desired timings         ---\r\n");
	kprintf("---------------------------------------\r\n");
	kprintf("dwNWE_SETUP      %d\r\n", dwNWE_SETUP		);
	kprintf("dwNCS_WR_SETUP   %d\r\n", dwNCS_WR_SETUP	);
	kprintf("dwNRD_SETUP      %d\r\n", dwNRD_SETUP		);
	kprintf("dwNCS_RD_SETUP   %d\r\n", dwNCS_RD_SETUP	);
	kprintf("dwNWE_PULSE      %d\r\n", dwNWE_PULSE		);
	kprintf("dwNCS_WR_PULSE   %d\r\n", dwNCS_WR_PULSE	);
	kprintf("dwNRD_PULSE      %d\r\n", dwNRD_PULSE		);
	kprintf("dwNCS_RD_PULSE   %d\r\n", dwNCS_RD_PULSE	);
	kprintf("dwNRD_CYCLE      %d\r\n", dwNRD_CYCLE		);
	kprintf("dwNWE_CYCLE      %d\r\n", dwNWE_CYCLE		);

	// Getting Clock period frequency
	dwClockPeriod_ns = 1000000000/dwMasterClock;

	kprintf("dwClockPeriod_ns %d\r\n", dwClockPeriod_ns);

	// Verify that value are dwClockPeriod_ns compatible (round them to the integer clock value)
	if (dwNWE_SETUP%dwClockPeriod_ns)
	{
		dwNWE_SETUP = ROUND_UP_TO_NEAREST_MULTIPLE(dwNWE_SETUP, dwClockPeriod_ns);
	}
		
	if (dwNCS_WR_SETUP%dwClockPeriod_ns)
	{
		dwNCS_WR_SETUP = ROUND_UP_TO_NEAREST_MULTIPLE(dwNCS_WR_SETUP, dwClockPeriod_ns);
	}

	if (dwNWE_PULSE%dwClockPeriod_ns)
	{
		dwNWE_PULSE = ROUND_UP_TO_NEAREST_MULTIPLE(dwNWE_PULSE, dwClockPeriod_ns);
	}

	if (dwNCS_WR_PULSE%dwClockPeriod_ns)
	{
		dwNCS_WR_PULSE = ROUND_UP_TO_NEAREST_MULTIPLE(dwNCS_WR_PULSE, dwClockPeriod_ns);
	}

	if (dwNWE_CYCLE%dwClockPeriod_ns)
	{
		dwNWE_CYCLE = ROUND_UP_TO_NEAREST_MULTIPLE(dwNWE_CYCLE, dwClockPeriod_ns);
	}

	if (dwNWE_HOLD%dwClockPeriod_ns)
	{
		dwNWE_HOLD = ROUND_UP_TO_NEAREST_MULTIPLE(dwNWE_HOLD, dwClockPeriod_ns);
	}

	if (dwNCS_WR_HOLD%dwClockPeriod_ns)
	{
		dwNCS_WR_HOLD = ROUND_UP_TO_NEAREST_MULTIPLE(dwNCS_WR_HOLD, dwClockPeriod_ns);
	}

	if (dwNWE_CYCLE < dwNWE_SETUP + dwNWE_PULSE + dwNWE_HOLD)
	{
		dwNWE_CYCLE = dwNWE_SETUP + dwNWE_PULSE + dwNWE_HOLD;
	}

	if (dwNWE_CYCLE < dwNCS_WR_SETUP + dwNCS_WR_PULSE + dwNCS_WR_HOLD)
	{
		dwNWE_CYCLE = dwNCS_WR_SETUP + dwNCS_WR_PULSE + dwNCS_WR_HOLD;
	}

	if (dwNRD_SETUP%dwClockPeriod_ns)
	{
		dwNRD_SETUP = ROUND_UP_TO_NEAREST_MULTIPLE(dwNRD_SETUP, dwClockPeriod_ns);
	}

	if(dwNCS_RD_SETUP%dwClockPeriod_ns)
	{
		dwNCS_RD_SETUP = ROUND_UP_TO_NEAREST_MULTIPLE(dwNCS_RD_SETUP, dwClockPeriod_ns);
	}

	if (dwNRD_PULSE%dwClockPeriod_ns)
	{
		dwNRD_PULSE = ROUND_UP_TO_NEAREST_MULTIPLE(dwNRD_PULSE, dwClockPeriod_ns);
	}

	if (dwNCS_RD_PULSE%dwClockPeriod_ns)
	{
		dwNCS_RD_PULSE = ROUND_UP_TO_NEAREST_MULTIPLE(dwNCS_RD_PULSE, dwClockPeriod_ns);
	}

	if (dwNRD_CYCLE%dwClockPeriod_ns)
	{
		dwNRD_CYCLE = ROUND_UP_TO_NEAREST_MULTIPLE(dwNRD_CYCLE, dwClockPeriod_ns);
	}

	if (dwNDR_HOLD%dwClockPeriod_ns)
	{
		dwNDR_HOLD = ROUND_UP_TO_NEAREST_MULTIPLE(dwNDR_HOLD, dwClockPeriod_ns);
	}

	if (dwNCS_RD_HOLD%dwClockPeriod_ns)
	{
		dwNCS_RD_HOLD = ROUND_UP_TO_NEAREST_MULTIPLE(dwNCS_RD_HOLD, dwClockPeriod_ns);
	}

	if (dwNRD_CYCLE < dwNRD_SETUP + dwNRD_PULSE + dwNDR_HOLD)
	{
		dwNRD_CYCLE = dwNRD_SETUP + dwNRD_PULSE + dwNDR_HOLD;
	}

	if (dwNRD_CYCLE < dwNCS_RD_SETUP + dwNCS_RD_PULSE + dwNCS_RD_HOLD)
	{
		dwNRD_CYCLE = dwNCS_RD_SETUP + dwNCS_RD_PULSE + dwNCS_RD_HOLD;
	}

	// Compute differents register value
	
	//-------------------------------------------------------------------------
	//
	// NWE setup length = (128* NWE_SETUP[5] + NWE_SETUP[4:0]) clock cycles
	temp = ((dwNWE_SETUP+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NWE_SETUP		= (temp>0x1E?((temp>>7)&0x01)<<5:temp);

	// NCS setup length = (128* NCS_WR_SETUP[5] + NCS_WR_SETUP[4:0]) clock cycles
	temp = ((dwNCS_WR_SETUP+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NCS_WR_SETUP	= (temp>0x1E?((temp>>7)&0x01)<<5:temp);

	// NRD setup length = (128* NRD_SETUP[5] + NRD_SETUP[4:0]) clock cycles
	temp = ((dwNRD_SETUP+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NRD_SETUP		= (temp>0x1E?((temp>>7)&0x01)<<5:temp);

	// NCS setup length = (128* NCS_RD_SETUP[5] + NCS_RD_SETUP[4:0]) clock cycles
	temp = ((dwNCS_RD_SETUP+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NCS_RD_SETUP	= (temp>0x1E?((temp>>7)&0x01)<<5:temp);
	
	// SETUP Register
	dw_r_SETUP = (dw_r_NCS_RD_SETUP<<24) | (dw_r_NRD_SETUP<<16) | (dw_r_NCS_WR_SETUP<<8) | (dw_r_NWE_SETUP<<0);

	//-------------------------------------------------------------------------
	//
	// NWE pulse length = (256* NWE_PULSE[6] + NWE_PULSE[5:0]) clock cycles
	temp = ((dwNWE_PULSE+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NWE_PULSE		= (temp>0x3E?((temp>>8)&0x01)<<6:temp);
	//dw_r_NWE_PULSE		= (dw_r_NWE_PULSE?dw_r_NWE_PULSE:1);

	// NCS pulse length = (256* NCS_WR_PULSE[6] + NCS_WR_PULSE[5:0]) clock cycles
	temp = ((dwNCS_WR_PULSE+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NCS_WR_PULSE	= (temp>0x3E?((temp>>8)&0x01)<<6:temp);
	//dw_r_NCS_WR_PULSE	= (dw_r_NCS_WR_PULSE?dw_r_NCS_WR_PULSE:1);

	// NRD pulse length = (256* NRD_PULSE[6] + NRD_PULSE[5:0]) clock cycles
	temp = ((dwNRD_PULSE+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NRD_PULSE		= (temp>0x3E?((temp>>8)&0x01)<<6:temp);
	//dw_r_NRD_PULSE		= (dw_r_NRD_PULSE?dw_r_NRD_PULSE:1);

	// NCS pulse length = (256* NCS_RD_PULSE[6] + NCS_RD_PULSE[5:0]) clock cycles
	temp = ((dwNCS_RD_PULSE+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NCS_RD_PULSE	= (temp>0x3E?((temp>>8)&0x01)<<6:temp);
	//dw_r_NCS_RD_PULSE	= (dw_r_NCS_RD_PULSE?dw_r_NCS_RD_PULSE:1);
	
	// PULSE Register
	dw_r_PULSE = (dw_r_NCS_RD_PULSE<<24) | (dw_r_NRD_PULSE<<16) | (dw_r_NCS_WR_PULSE<<8) | (dw_r_NWE_PULSE<<0);

	//-------------------------------------------------------------------------
	//
	// Write cycle length = (NWE_CYCLE[8:7]*256 + NWE_CYCLE[6:0]) clock cycles
	temp = ((dwNWE_CYCLE+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NWE_CYCLE		= (temp>0x7E?((temp>>8)&0x03)<<7:temp);

	// Read cycle length = (NRD_CYCLE[8:7]*256 + NRD_CYCLE[6:0]) clock cycles
	temp = ((dwNRD_CYCLE+dwClockPeriod_ns-1)/dwClockPeriod_ns);
	dw_r_NRD_CYCLE		= (temp>0x7E?((temp>>8)&0x03)<<7:temp);

	// CYCLE Register
	dw_r_CYCLE = (dw_r_NRD_CYCLE<<16) | (dw_r_NWE_CYCLE<<0);

	// Writing all registers 
	*pSMC_SETUP = dw_r_SETUP;
	*pSMC_PULSE = dw_r_PULSE;
	*pSMC_CYCLE = dw_r_CYCLE;

	// Display small message resuming the changes:
	kprintf("---------------------------------------\r\n");
	kprintf("---           Real timings          ---\r\n");
	kprintf("---------------------------------------\r\n");
	kprintf("dwNWE_SETUP      %d\r\n", (((dw_r_NWE_SETUP		&0x0020)>>5)*128 + (dw_r_NWE_SETUP		&0x1F))*dwClockPeriod_ns);
	kprintf("dwNCS_WR_SETUP   %d\r\n", (((dw_r_NCS_WR_SETUP	&0x0020)>>5)*128 + (dw_r_NCS_WR_SETUP	&0x1F))*dwClockPeriod_ns);
	kprintf("dwNRD_SETUP      %d\r\n", (((dw_r_NRD_SETUP		&0x0020)>>5)*128 + (dw_r_NRD_SETUP		&0x1F))*dwClockPeriod_ns);
	kprintf("dwNCS_RD_SETUP   %d\r\n", (((dw_r_NCS_RD_SETUP	&0x0020)>>5)*128 + (dw_r_NCS_RD_SETUP	&0x1F))*dwClockPeriod_ns);
	kprintf("dwNWE_PULSE      %d\r\n", (((dw_r_NWE_PULSE		&0x0040)>>6)*256 + (dw_r_NWE_PULSE		&0x3F))*dwClockPeriod_ns);
	kprintf("dwNCS_WR_PULSE   %d\r\n", (((dw_r_NCS_WR_PULSE	&0x0040)>>6)*256 + (dw_r_NCS_WR_PULSE	&0x3F))*dwClockPeriod_ns);
	kprintf("dwNRD_PULSE      %d\r\n", (((dw_r_NRD_PULSE		&0x0040)>>6)*256 + (dw_r_NRD_PULSE		&0x3F))*dwClockPeriod_ns);
	kprintf("dwNCS_RD_PULSE   %d\r\n", (((dw_r_NCS_RD_PULSE	&0x0040)>>6)*256 + (dw_r_NCS_RD_PULSE	&0x3F))*dwClockPeriod_ns);
	kprintf("dwNRD_CYCLE      %d\r\n", (((dw_r_NRD_CYCLE		&0x0180)>>7)*256 + (dw_r_NRD_CYCLE		&0x7F))*dwClockPeriod_ns);
	kprintf("dwNWE_CYCLE      %d\r\n", (((dw_r_NWE_CYCLE		&0x0180)>>7)*256 + (dw_r_NWE_CYCLE		&0x7F))*dwClockPeriod_ns);


	return TRUE;
}


//------------------------------------------------------------------------------
//! @}


////////////////////////////////////////////////////////////////////////////////
// End of $URL: http://centaure/svn/interne-ce_atmel_common/TAGS/V150-Nadia1-19-Sept-2006/AT91SAM926x/MISC/ChipSelect/ChipSelect.c $
////////////////////////////////////////////////////////////////////////////////
//
// History : $Log$
// History : Revision 1.11  2006/06/23 11:55:02  jjhiblot
// History : Fixed a bug in timing computation
// History :
// History : Revision 1.10  2006/04/11 15:02:09  jjhiblot
// History : removed useless include
// History :
// History : Revision 1.9  2006/03/27 08:17:29  VBelloir
// History : Create a macro ROUND_UP_TO_NEAREST_MULTIPLE
// History : fixe some bugs
// History :
// History : Revision 1.8  2005/11/18 15:18:55  jjhiblot
// History : Enhanced and fixed doxygen support
// History :
// History : Revision 1.7  2005/11/04 16:18:34  jjhiblot
// History : modified log message
// History :
// History : Revision 1.6  2005/11/02 08:31:50  jjhiblot
// History : *** empty log message ***
// History :
// History : Revision 1.5  2005/09/20 14:55:15  dmartin
// History : *** empty log message ***
// History :
// History : Revision 1.4  2005/07/21 13:18:49  dmartin
// History : DMR : Modified headers (20ieme fois :p )
// History :

