/*    $ dooroos.RealTime: main_app.c v 0.1 2011/10/11 $    */

/*
 * Copyright (C) 2011    Jung Byuong-Oh
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: main_app.c
 *
 * Purpose: Create the user application.
 *
 */

#include <types.h>
#include <message.h>
#include <memory.h>
#include <thread.h>
#include <time.h>
#include <fs.h>

#include <taskheap.h>
#include <moduleloader.h>
#include <panic.h>
#include <hal.h>

#include <lib/string.h>


#include <mach/SAMA5D3X.h>

    
#define DBG_LVL_APP         0



#define DISK_PARTITION

STATIC PBYTE tUsbDiskExport[] =
{
    (PBYTE)"DSK1",
#ifdef DISK_PARTITION
    (PBYTE)"DSK2",
    (PBYTE)"DSK3",
#endif
};

#define USB_DISK_NUM    (sizeof(tUsbDiskExport)/sizeof(tUsbDiskExport[0]))


#if 1

#else

PUBLIC UINT32
RomFsReadFunction(PBYTE pBuffer, UINT32 StartOffset, INT32 ReadLen )
{
#define ROM_BASE        0x2FC00000
    memcpy( pBuffer, ROM_BASE + StartOffset, ReadLen );

    return  ReadLen;
}

#endif

 BOOL bDebug = TRUE;

//===========================================================================================
// for speedup for access time of very big size disk images
//#define MAX_FS_BUFFER_ORDER         7           // should be greater than 2
#define MAX_FS_BUFFER_ORDER         7           // should be greater than 2
//#define MAX_FS_BUFFER_ORDER         3           // should be greater than 2

#define KMALLOC_PAGES		256 //128
//===========================================================================================

static char string[256];
static char string_buf[4096*2];
static int ibuf = 0;
static int imax;
static char *readstring( HANDLE fh )
{
    char *pRet = NULL;
    char c;
    int index = 0;
    int nRead;

for(;;)
{
    if( ibuf < imax )
    {
        c = string_buf[ibuf++];
//kprintf("read c %c\n", nRead, c );

        if( c == ' ' || c == '\t' || c == '\n' || c == '\r')
        {
            if( index != 0 )
            {
                string[index] = 0;
                return string;
            }
            continue;
        }
        else
        if( c == ';' )
        {
            // comment
            for(;;) 
            {
                nRead = FsRead( fh, &c, 1);
                if( ibuf >= imax )
                    break;
                else
                {
                    c = string_buf[ibuf++];
                    if( c == '\n' || c == '\r' )
                        break;
                }
            }
        }
        else
        {
            string[index] = c;
            index++;
            if( index >= 255 )
            {
                string[255] = 0;
                return string;
            }
        }
    }
    else
    {
        if( index != 0 )
        {
            string[index] = 0;
            return string;
        }
        else
        {
            return NULL;
        }
    }
}

    return NULL;
}


STATIC VOID DrawProg( UINT32 Color )
{
    #define BASE_FRAMEBUFFER       0x07000000
    #define OFFSET_NOCACHE         0x20000000

#ifdef _800x480_
    #define LCD_WIDTH       800
    #define LCD_HEIGHT      480
#endif

#ifdef _800x600_
    #define LCD_WIDTH       800
    #define LCD_HEIGHT      600
#endif

    #define BAR_HEIGHT      10
    #define BAR_OFFSET      5

#if 1
    STATIC UINT32 iBar = 0;

    int i;
    for( i = 0; i < 3; i ++ )
    {
        UINT32 ColorBar = (i == iBar)?Color:0xFFD0D0D0;
        UINT32 *p = (UINT32*)BASE_FRAMEBUFFER;
        p += LCD_WIDTH*(LCD_HEIGHT - BAR_HEIGHT - BAR_OFFSET ); // y offset
        p += BAR_OFFSET + i*(BAR_HEIGHT + BAR_OFFSET);
        UINT32 *nc = (UINT32 *)((UINT32)p + OFFSET_NOCACHE);
        UINT32 h, w;
        for( h = 0; h < BAR_HEIGHT; h++ )
        {
            UINT32 *q = p;
            for( w = 0; w < BAR_HEIGHT ; w++) {*q++ = ColorBar; *nc++ = ColorBar;}
            p += LCD_WIDTH;
            nc = (UINT32 *)((UINT32)p + OFFSET_NOCACHE); ;
        }
    }
    iBar++;
    if( iBar >= 3 ) iBar = 0;
        
#else
    UINT32 *p = (UINT32*)BASE_FRAMEBUFFER;
    p += LCD_WIDTH*(LCD_HEIGHT - BAR_HEIGHT - BAR_OFFSET ); // y offset
    p += BAR_OFFSET;

    UINT32 h, w;
    Percent = Percent * (LCD_WIDTH - 2*BAR_OFFSET) / 100;

    for( h = 0; h < BAR_HEIGHT; h++ )
    {
        UINT32 *q = p;
        for( w = 0; w < Percent ; w++) *q++ = Color;
        p += LCD_WIDTH;
    }
#endif
    return;
}


STATIC HANDLE hUsbDisk = NULL;
INT32
th_AppLoader( UINT32 Arg )
{
    BOOL		    bRet;
    BOOL            bUhc;
    DRIVER_ARG      DriverArg;

    BOOL bExecuteInUsb = FALSE;
#define BAR_COLOR 0xFFFF0000
DrawProg( BAR_COLOR );
    DEBUGMSG(DBG_LVL_APP, "%s (%d): Start\n", __FUNCTION__, __LINE__);
    Sleep(10);
    DEBUGMSG(DBG_LVL_APP, "%s (%d):\n", __FUNCTION__, __LINE__ );

    EXTERN THREADCONFIG UGDDev_Thread;
    ThreadCreate( &UGDDev_Thread );
    /*========================================================================*/
    /*                      setup the filesystem                              */
    /*========================================================================*/

    /*------------------------------------------------------------------------*/
    /*               load the driver for ram filesystem                       */
    /*------------------------------------------------------------------------*/
    //the file "fs_ram.drv" should be in the directory "dooroos"
    bRet = LoadDriver ((PBYTE)"fs_ram.drv", 0 );
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    bRet = FsSetRoot( FSTYPE_RAM, NULL /* not used */, (PBYTE)"Ram");      // "Ram" name has no effect
    //FsAttachDevice( FSTYPE_RAM, NULL /* not used */, (PBYTE)"Ram");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    //the file "fs_rom.drv" should be in the directory "dooroos"
    DEBUGMSG(DBG_LVL_APP,"Load RomFs driver\n");
    LoadDriver ((PBYTE)"fs_rom.drv", 0 );

    EXTERN UINT32 RomFsReadFunction(PBYTE pBuffer, UINT32 StartOffset, INT32 ReadLen );
    //FsSetRoot( FSTYPE_ROM, KernelRomFsReadFunction, (PBYTE)"Rom");		// "Rom" name has no effect
    bRet = FsAttachDevice( FSTYPE_ROM, RomFsReadFunction, (PBYTE)"Rom");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    DEBUGMSG(DBG_LVL_APP,"Change to the filesystem usage\n");
    LoaderFsAccess (TRUE);
DrawProg( BAR_COLOR );

    /*------------------------------------------------------------------------*/
    /*                      set the default directory                         */
    /*------------------------------------------------------------------------*/

    bRet = FsChangeDir( (PBYTE)"/");
    DEBUGMSG(DBG_LVL_APP, "%s : FsChangeDir => %x\n", __FUNCTION__, bRet );

    /* set library path */
    /*------------------------------------------------------------------------*/
    /*                          set library path                              */
    /*------------------------------------------------------------------------*/
    //SetLibraryPath ((PBYTE)"/usbdisk3/dooroos_boot/dll;/usbdisk3/dooroos_boot/drv;/sdc/dll;/sdc/drv;/PROJECT/dll;/PROJECT/drv;/SYSTEM/dll;/SYSTEM/drv;/dooroos;/dll;/drv");
    SetLibraryPath ((PBYTE)"/usbdisk3/dooroo/dll;/usbdisk3/dooroo/drv;/sdc/drv;/sdc/dll;/nand/dll;/nand/drv;/Rom/dooroos;/Rom/dll;/Rom/drv");
    //SetLibraryPath ((PBYTE)"/dooroos;/dll;/drv;/SYSTEM/dll;/SYSTEM/drv;/usbdisk3/hmi/dll;/usbdisk3/hmi/drv;");

    /*------------------------------------------------------------------------*/
    /*               load the default libraries                               */
    /*------------------------------------------------------------------------*/
    bRet = LoadLibrary ((PBYTE)"debug.dll");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    bRet = LoadLibrary ((PBYTE)"taskheap.dll");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    TaskHeapInit( KMALLOC_PAGES<<MEMPOOL_SHIFT );
//#if 0
//    bRet = LoadLibrary ((PBYTE)"drvlib.dll");
//    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//#endif
DrawProg( BAR_COLOR );
    /*========================================================================*/
    /*               load the libraries for the driver                        */
    /*========================================================================*/

    bRet = LoadLibrary ((PBYTE)"drvlib.dll");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    /*------------------------------------------------------------------------*/
    /*                      load the usb host driver                          */
    /*------------------------------------------------------------------------*/
    bUhc = LoadDriver ((PBYTE)"ehci.drv", THREAD_PRIORITY_DRIVER_HIGH + 1);
//#if 0
//    /*========================================================================*/
//    /*               load the libraries for the driver                        */
//    /*========================================================================*/
//
//    bRet = LoadLibrary ((PBYTE)"drvlib.dll");
//    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//#endif
//DrawProg( BAR_COLOR );
    /*------------------------------------------------------------------------*/
    /*          load the libraries of the fat filesystem             */
    /*------------------------------------------------------------------------*/
    bRet = LoadLibrary ((PBYTE)"convchar.dll");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    bRet = LoadLibrary ((PBYTE)"fat_nls.dll");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    bRet = LoadDriver ((PBYTE)"fs_fat.drv", (PVOID)MAX_FS_BUFFER_ORDER );
DrawProg( BAR_COLOR );    
//#if 0
//    /*------------------------------------------------------------------------*/
//    /*                      load the usb host driver                          */
//    /*------------------------------------------------------------------------*/
//    bRet = LoadDriver ((PBYTE)"ehci.drv", THREAD_PRIORITY_DRIVER_HIGH + 1);
//    if (bRet)
//    {
//        bRet = LoadDriver ((PBYTE)"uhc_storage.drv", NULL);
//        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//    }
//#endif

    /*------------------------------------------------------------------------*/
    /*                          load the nand driver                          */
    /*------------------------------------------------------------------------*/
    // Load the nand flash block device driver
    DEBUGMSG(DBG_LVL_APP,"RegisterDrivers NAND Driver\n");
    bRet = LoadDriver ((PBYTE)"nand.drv", NULL);
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
DrawProg( BAR_COLOR );

	DEBUGMSG(DBG_LVL_APP,"RegisterDrivers DSK1\n");
    bRet = LoadDriver ((PBYTE)"mtd.drv", (VOID*)0); // minor number 0 --> refer the mtd block device driver
	DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
DrawProg( BAR_COLOR );

#ifdef DISK_PARTITION
    DEBUGMSG(DBG_LVL_APP,"RegisterDrivers DSK2\n");
    bRet = LoadDriver ((PBYTE)"mtd.drv", (VOID*)1); // minor number 1 --> refer the mtd block device driver
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
DrawProg( BAR_COLOR );

    DEBUGMSG(DBG_LVL_APP,"RegisterDrivers DSK3\n");
    bRet = LoadDriver ((PBYTE)"mtd.drv", (VOID*)2); // minor number 2 --> refer the mtd block device driver
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
#endif
DrawProg( BAR_COLOR );

    if (bUhc)
    {
        bRet = LoadDriver ((PBYTE)"uhc_storage.drv", NULL);
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
    }
    
//#if 0
//    /*------------------------------------------------------------------------*/
//    /*          load the libraries and drivers for fat filesystem             */
//    /*------------------------------------------------------------------------*/
//    bRet = LoadLibrary ((PBYTE)"convchar.dll");
//    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//
//    bRet = LoadLibrary ((PBYTE)"fat_nls.dll");
//    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//
//    bRet = LoadDriver ((PBYTE)"fs_fat.drv", (PVOID)MAX_FS_BUFFER_ORDER );
//#endif
//
//#if 0
//    /*========================================================================*/
//    /*               load the libraries for the driver                        */
//    /*========================================================================*/
//
//    bRet = LoadLibrary ((PBYTE)"drvlib.dll");
//    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//#endif
    /*========================================================================*/
    /*                     load the device drivers                            */
    /*========================================================================*/
    bRet = LoadDriver ((PBYTE)"usbdev.drv", 0);
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
	if(bRet)
    {
		DEBUGMSG(DBG_LVL_APP," RegisterDrivers USB DEVICE Class driver for Mass Storage\n");
		DriverArg.Arg = (UINT32)tUsbDiskExport;
		DriverArg.Priority = THREAD_PRIORITY_DRIVER_HIGH;
		DriverArg.Minor = 0;
		DriverArg.User[0] = USB_DISK_NUM;  // nDisk
		DriverArg.User[1] = TRUE;   //bRegister
		bRet = LoadDriver ((PBYTE)"udc_msd.drv", &DriverArg);
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
	}

DrawProg( BAR_COLOR );
//#if 0
//    /*------------------------------------------------------------------------*/
//    /*                      load the usb host driver                          */
//    /*------------------------------------------------------------------------*/
////    bRet = LoadDriver ((PBYTE)"ehci.drv", THREAD_PRIORITY_REALTIMEDRIVER_HIGH );//THREAD_PRIORITY_DRIVER_HIGH + 1);
//    bRet = LoadDriver ((PBYTE)"ehci.drv", THREAD_PRIORITY_DRIVER_HIGH + 1);
//    if (bRet)
//    {
////    if (bRet)
////    {
////        Sleep(10);
//        UINT32 OnFlag = (1<<0)|(1<<1);
//        #define IOCTRL_HAL_UHCPOWER              ( 252|(MESSAGE_KERNEL<<MESSAGE_CLASS_SHIFT)|(MESSAGE_DEVICE<<MESSAGE_IOCONTROL_SHIFT))
//        KernelIoControl(IOCTRL_HAL_UHCPOWER, &OnFlag, sizeof(OnFlag), NULL, NULL);
////    }
//        bRet = LoadDriver ((PBYTE)"uhc_storage.drv", NULL);
//        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
///*
//        bRet = LoadDriver ((PBYTE)"uhc_hid_mouse.drv", NULL);
//        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//
//        bRet = LoadDriver ((PBYTE)"uhc_hid_kbd.drv", NULL);
//        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//
//        bRet = LoadDriver ((PBYTE)"uhc_printer.drv", NULL);
//        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//
//        bRet = LoadDriver ((PBYTE)"uhc_uvc.drv", NULL);
//        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
//*/
//    }
//DrawProg( BAR_COLOR );
//#endif

    /*------------------------------------------------------------------------*/
    /*          register disk of the nand flash                               */
    /*------------------------------------------------------------------------*/
	bRet = FsAttachDevice( FSTYPE_FAT, "DSK1", (PBYTE)"nand");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
#ifdef DISK_PARTITION
    bRet = FsAttachDevice( FSTYPE_FAT, "DSK2", (PBYTE)"nnd");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
    bRet = FsAttachDevice( FSTYPE_FAT, "DSK3", (PBYTE)"data");
    DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
#endif

    if( 1 )
    {
        int i = 0;
        int boot = 0;
        while( i++ < 50 )
        {
//            Sleep(10);
            if( HAL_DebugReadByte() == 'p' )
            {
                kprintf("Paused 0\n");
                boot = 1;
                break;
            }
        }

        while( boot )
        {
            Sleep(1000);
        }
    }
DrawProg( BAR_COLOR );

    HANDLE fh = NULL;
    char drv_dll[256];
    bRet = FsChangeDir( (PBYTE)"/");
    DEBUGMSG(DBG_LVL_APP, "%s : FsChangeDir => %x\n", __FUNCTION__, bRet );
    bRet = FsChangeDir( (PBYTE)"/nand");
    DEBUGMSG(DBG_LVL_APP, "%s : FsChangeDir => %x\n", __FUNCTION__, bRet );
    if( !bRet )
    {
        kprintf("no nand disk found : format the disk\n");
    }
    else
    {
        fh = FsOpen( (BYTE*)"/nand/load.txt");
        if( fh == NULL )
        {
            kprintf("no load file : load.txt\n");
        }
        else
        {
            imax = FsRead( fh, string_buf, 4096 );
            FsClose( fh );

            for(;;)
            {
                char *p = readstring(fh); //read command
//kprintf("p = %s\n", p );
                if( !p ) break;

                if( !strcmp(p, "Driver") )
                {
                    BOOL bRet ;
                    char *p;
                    memset( &DriverArg, 0, sizeof(DriverArg) );
                    p = readstring(fh); if(!p) goto LINE_READ_DONE;//read driver name
                    strcpy( drv_dll, p );
                    p = readstring(fh); if(!p) goto LINE_READ_DONE;//read arg
                    DriverArg.Arg = atoi(p);
                    p = readstring(fh); if(!p) goto LINE_READ_DONE;//read stack size
                    DriverArg.pStack = (UINT32)NULL;
                    DriverArg.StackSize = atoi(p);
                    p = readstring(fh); if(!p) goto LINE_READ_DONE;//read priority
                    DriverArg.Priority = atoi(p);
                    p = readstring(fh); if(!p) goto LINE_READ_DONE;//read minor number
                    DriverArg.Minor = atoi(p);
                    p = readstring(fh); if(!p) goto DRIVER_READ_DONE;//read user 0
                    DriverArg.User[0] = atoi(p);
                    p = readstring(fh); if(!p) goto DRIVER_READ_DONE;//read user 1
                    DriverArg.User[1] = atoi(p);
                    p = readstring(fh); if(!p) goto DRIVER_READ_DONE;//read user 2
                    DriverArg.User[2] = atoi(p);
DRIVER_READ_DONE:
//kprintf("DriverArg.Arg = %d\n", DriverArg.Arg );
//kprintf("DriverArg.StackSize = %d\n", DriverArg.StackSize );
//kprintf("DriverArg.Priority = %d\n", DriverArg.Priority );
//kprintf("DriverArg.Minor = %d\n", DriverArg.Minor );

                    bRet = LoadDriver ((PBYTE)drv_dll, &DriverArg);
                    if( !bRet )
                        kprintf("fail to load driver : %s\n", drv_dll );
                }
                else
                if( !strcmp(p, "Driver0") )
                {
                    char *p;
                    p = readstring(fh); if(!p) goto LINE_READ_DONE;//read driver name
                    strcpy( drv_dll, p );
                    p = readstring(fh); if(!p) goto LINE_READ_DONE;//read priority
                    DriverArg.Priority = atoi(p);
//kprintf("DriverArg.Priority = %d\n", DriverArg.Priority );
                    BOOL bRet = LoadDriver ((PBYTE)drv_dll, (VOID *)DriverArg.Priority);
                    if( !bRet )
                        kprintf("fail to load driver : %s\n", drv_dll );
                }
                else
                if( !strcmp(p, "Library") )
                {
                    char *plib = readstring(fh); if(!p) goto LINE_READ_DONE;//read library name
                    BOOL bRet = LoadLibrary( plib );
                    if( !bRet )
                        kprintf("fail to load library : %s\n", plib );
                }
                else
                if( !strcmp(p, "Exec") )
                {
//DrawProg( BAR_COLOR );
                    char *pexec = readstring(fh); if(!p) goto LINE_READ_DONE;//read execute file name
                    if( !bExecuteInUsb )
                    {
                        BOOL bRet = LoadExecutable( pexec, 0 );
                        if( !bRet )
                            kprintf("fail to execute : %s\n", pexec );
                    }
                }
LINE_READ_DONE:
                if( hUsbDisk == NULL )
                {
                    hUsbDisk = IoOpen( (BYTE *)"DSK4");
//kprintf("hUsbDisk4 = %x\n", hUsbDisk );
                    if( hUsbDisk )
                    {
                        IoClose( hUsbDisk );
                    }
//Sleep(100);
                }

//kprintf("hUsbDisk = %x\n", hUsbDisk );
                if( hUsbDisk != NULL && bExecuteInUsb == FALSE)
                {
                    //bRet = FsChangeDir( (BYTE*)"/usbdisk3");
                    FILEINFO    FileInfo;
                    bRet = FsGetFileInfo( (BYTE*)"/usbdisk3/dooroo/dooroo_usb.exe", &FileInfo );
//kprintf("3 bRet = %d\n", bRet );
                    if( bRet )
                        bExecuteInUsb = TRUE;
//                    else
//                        Sleep(10);
//                        Sleep(100);
                }
//kprintf("before bExecuteInUsb = %d\n", bExecuteInUsb );
DrawProg( BAR_COLOR );
            }
        }
    }

    if( fh == NULL )
    {
        do
        {
                bRet = FsChangeDir( (BYTE*)"/usbdisk3");
kprintf("insert usb disk : \n", bRet );
                if( bRet )
                    bExecuteInUsb = TRUE;
                else
                    Sleep(100);
        }while( !bExecuteInUsb );

        /*========================================================================*/
        /*                     load the dll libraries                             */
        /*========================================================================*/
        bRet = LoadLibrary ((PBYTE)"zlib.dll");
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
        bRet = LoadDriver ((PBYTE)"display.drv", NULL);
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
        bRet = LoadDriver ((PBYTE)"touch.drv", NULL);
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
        bRet = LoadLibrary ((PBYTE)"imgbmp.dll");
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
        bRet = LoadLibrary ((PBYTE)"imggif.dll");
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
        bRet = LoadLibrary ((PBYTE)"ttffont.dll");
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );
        bRet = LoadLibrary ((PBYTE)"gl.dll");
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

        bRet = LoadLibrary ((PBYTE)"widgetproc.dll");
        DEBUGMSG(DBG_LVL_APP, "%s (%d): %x\n", __FUNCTION__, __LINE__, bRet );

    }

    // excute in USB program
    if( bExecuteInUsb )
    {
//kprintf("bExecuteInUsb = %d\n", bExecuteInUsb );
//                bRet = FsChangeDir( (BYTE*)"/usbdisk3");
//kprintf("bRet = %d\n", bRet );
//                if( bRet )
        {
DrawProg( 0xFF0000FF );//BAR_COLOR );
            // now check the usb disk memory
            bRet = LoadExecutable( (BYTE*)"/usbdisk3/dooroo/dooroo_usb.exe", 0 );
            if( bRet )
            {
//                        bExecuteInUsb = TRUE;
                // execute in the usb disk 
                kprintf("OK : execute in the usb disk\n" );
//                        break;
            }
        }
    }



	for(;;)
    {
        Sleep(1000);
    }
    return bRet;
}

					STATIC THREADCONFIG AppLoaderThread = { NULL, th_AppLoader, NULL, 4096, 254 };//THREAD_PRIORITY_USER_HIGH + 20 };
                    INT32
                    app_main( VOID )
                    {
                        HANDLE 	hThread = NULL;

                        hThread = ThreadCreate( &AppLoaderThread );
//DEBUGMSG(DBG_LVL_APP, "%s (%d): hThread = %x\n", __FUNCTION__, __LINE__, hThread );

                        return hThread;
                    }


